@@ -1,5 +1,5 @@
-import sys
-sys.path.append("..")
+#!/usr/bin/env python
+import iam_sympy_example
 
 from sympy import Symbol, Basic
 from sample import sample
@@ -29,7 +29,7 @@ def mplot3d(f, var1, var2, show=True):
     ax.set_xlabel('X')
     ax.set_ylabel('Y')
     ax.set_zlabel('Z')
-    
+
     if show:
         p.show()
 
