@@ -1,5 +1,5 @@
-import sys
-sys.path.append("..")
+#!/usr/bin/env python
+import iam_sympy_example
 
 import sympy
 e=sympy.Rational(2)**50/sympy.Rational(10)**50
