@@ -1,3 +1,6 @@
+#!/usr/bin/env python
+import iam_sympy_example
+
 """
 http://www.lncc.br/~portugal/Canon.html
 http://www.lncc.br/~portugal/Invar.html
@@ -5,9 +8,6 @@ http://www.ginac.de/tutorial/Indexed-objects.html
 http://grtensor.phy.queensu.ca/
 
 """
-import sys
-sys.path.append(".")
-sys.path.append("..")
 
 from sympy import exp, Symbol, sin, Rational, Derivative, dsolve
 
@@ -60,8 +60,8 @@ class nu(Function):
 class lam(Function):
     pass
 
-gdd=Matrix(( 
-    (-exp(nu(r)),0,0,0), 
+gdd=Matrix((
+    (-exp(nu(r)),0,0,0),
     (0, exp(lam(r)), 0, 0),
     (0, 0, r**2, 0),
     (0, 0, 0, r**2*sin(theta)**2)
