@@ -10,17 +10,12 @@ See the webpage for more information and documentation:
     http://code.google.com/p/sympy/
 """
 
-__version__ = "0.5.15"
-
-#put path to pyglet into the search path, so that it can be imported
-import os.path
-import sys
-sys.path.insert(0, os.path.join(os.path.abspath(os.path.dirname(__file__)), "thirdparty", \
-        "pyglet"))
+__version__ = "0.6.1"
 
 import symbol as stdlib_symbol
 from sympy.core import *
 
+from polys import *
 from series import *
 from functions import *
 from ntheory import *
@@ -29,16 +24,16 @@ from simplify import *
 from solvers import *
 from matrices import *
 from geometry import *
-from polynomials import *
 from utilities import *
 from integrals import *
 # This module is slow to import:
 #from physics import units
 from plotting import Plot, textplot
 from printing import pretty, pretty_print, pprint, pprint_use_unicode, \
-    pprint_try_use_unicode, print_gtk
+    pprint_try_use_unicode, print_gtk, print_tree
 from printing import latex, preview, view, pngview, pdfview, dviview
+from printing import python, print_python
+from polynomials import factor
 
-import abc
-
-from polys import *
+# This is slow to import:
+#import abc
