@@ -181,10 +181,30 @@ class Add(AssocOp, RelMeths, ArithMeths):
 
         # Now we need to sort the factors in Add, which are in "rest". Any
         # ordering is fine, but some ordering looks better and some looks bad.
-
         # This particular solution is slow, but it ensures a sane ordering. It
         # can of course be improved:
+
         rest = list(rest)
+
+        # HACK: the pattern matching in compare_terms ends up with
+        # infinite recursion if there are Wilds present. The following code
+        # can be removed if a better fix is found in .match()
+        def cleanup_wild(x):
+            if isinstance(x, Wild):
+                return Symbol(x.name), True
+            L = []
+            modified = False
+            for g in x.args:
+                g, modified_ = cleanup_wild(g)
+                modified = modified or modified_
+                L.append(g)
+            if modified:
+                x = x.func(*L)
+            return x, modified
+
+        rest = [cleanup_wild(t)[0] for t in rest]
+        # END HACK
+
         rest.sort(self.compare_terms)
 
         l = []
@@ -256,34 +276,11 @@ class Add(AssocOp, RelMeths, ArithMeths):
         r = xrange(len(numers))
         return Add(*[Mul(*(denoms[:i]+[numers[i]]+denoms[i+1:])) for i in r]),Mul(*denoms)
 
-    def _calc_splitter(self, d):
-        if d.has_key(self):
-            return d[self]
-        coeff, factors = self.as_coeff_factors()
-        r = self.__class__(*[t._calc_splitter(d) for t in factors])
-        if r.is_Add and 0:
-            for e,t in d.items():
-                w = Wild('w')
-                d1 = r.match(e+w)
-                if d1 is not None:
-                    r1 = t + d1[w]
-                    break
-        if d.has_key(r):
-            return coeff + d[r]
-        s = d[r] = Temporary()
-        return s + coeff
-
     def count_ops(self, symbolic=True):
         if symbolic:
             return Add(*[t.count_ops(symbolic) for t in self[:]]) + Symbol('ADD') * (len(self[:])-1)
         return Add(*[t.count_ops(symbolic) for t in self.args[:]]) + (len(self.args[:])-1)
 
-    def _eval_integral(self, s):
-        return Add(*[f.integral(s) for f in self])
-
-    def _eval_defined_integral(self, s,a,b):
-        return Add(*[f.integral(s==[a,b]) for f in self])
-
     def _eval_is_polynomial(self, syms):
         for term in self.args:
             if not term._eval_is_polynomial(syms):
@@ -305,7 +302,7 @@ class Add(AssocOp, RelMeths, ArithMeths):
             return Add(*l[1:]).is_even
 
     def _eval_is_irrational(self):
-        for t in self:
+        for t in self.args:
             a = t.is_irrational
             if a: return True
             if a is None: return
