@@ -1,8 +1,7 @@
 """Base class for all objects in sympy"""
 
-type_class = type
+import sympy.mpmath as mpmath
 
-import decimal
 from assumptions import AssumeMeths
 from sympify import _sympify, sympify, SympifyError
 from cache import cacheit, Memoizer, MemoizerArg
@@ -109,6 +108,11 @@ class BasicMeta(BasicType):
         for k in dir(cls):
             if not k.startswith('is_'):
                 continue
+
+            # this is not an assumption (e.g. is_Integer)
+            if k[3:] not in AssumeMeths._assume_defined:
+                continue
+
             v = getattr(cls, k)
             k = k[3:]
             if isinstance(v,(bool,int,long)):
@@ -149,8 +153,7 @@ class Basic(AssumeMeths):
     """
     Base class for all objects in sympy.
 
-    Conventions
-    ===========
+    Conventions:
 
     1)
     When you want to access parameters of some instance, always use .args:
@@ -201,6 +204,7 @@ class Basic(AssumeMeths):
 
     def __new__(cls, *args, **assumptions):
         obj = object.__new__(cls)
+        obj._assumptions = None
         obj.assume(**assumptions)
         obj._mhash = None # will be set by __hash__ method.
         obj._args = args  # all items in args must be Basic objects
@@ -385,18 +389,20 @@ class Basic(AssumeMeths):
         """
         Set the representation level used for repr() printing,
         returning the current level. The available levels are:
-            0: Lowest level printing. Expressions printing should be
-               be able to be evaluated through Python's eval()
-               function
-            1: Higher level printing. Expressions are printed in a
-               one-dimensional fashion, are easier to read than
-               level 1, but cannot be parsed through eval()
-            2: Highest level printing. Expressions are simply
-               two-dimensional, "pretty" versions of the expressions
-               that are only useful for readability purposes.
+
+        flag == 0:
+            Lowest level printing. Expressions printing should be be able to be
+            evaluated through Python's eval() function
+        flag == 1:
+            Higher level printing. Expressions are printed in a one-dimensional
+            fashion, are easier to read than level 1, but cannot be parsed
+            through eval()
+        flag == 2:
+            Highest level printing. Expressions are simply two-dimensional,
+            "pretty" versions of the expressions that are only useful for
+            readability purposes.
 
         Notes:
-        ======
             - Level 2 printing is done through the printing module in
               smpy.printing.pretty.
         """
@@ -439,43 +445,49 @@ class Basic(AssumeMeths):
 
         return self.torepr()
 
+    def atoms(self, *types):
+        """Returns the atoms that form the current object.
 
+           >>> from sympy import *
+           >>> x,y = symbols('xy')
 
-    @Memoizer('Basic', MemoizerArg((type, type(None), tuple), name='type'), return_value_converter = lambda obj: obj.copy())
-    def atoms(self, type=None):
-        """Returns the atoms that form current object.
+           >>> sorted((x+y**2 + 2*x*y).atoms())
+           [2, x, y]
 
-        Example:
-        >>> from sympy import *
-        >>> x = Symbol('x')
-        >>> y = Symbol('y')
-        >>> (x+y**2+ 2*x*y).atoms()
-        set([2, x, y])
+           You can also filter the results by a given type(s) of object:
 
-        You can also filter the results by a given type(s) of object
-        >>> (x+y+2+y**2*sin(x)).atoms(type=Symbol)
-        set([x, y])
+           >>> sorted((x+y+2+y**2*sin(x)).atoms(Symbol))
+           [x, y]
 
-        >>> (x+y+2+y**2*sin(x)).atoms(type=Number)
-        set([2])
+           >>> sorted((x+y+2+y**3*sin(x)).atoms(Number))
+           [2, 3]
+
+           >>> sorted((x+y+2+y**2*sin(x)).atoms(Symbol, Number))
+           [2, x, y]
+
+           Or by a type of on object in an impliciy way:
+
+           >>> sorted((x+y+2+y**2*sin(x)).atoms(x))
+           [x, y]
 
-        >>> (x+y+2+y**2*sin(x)).atoms(type=(Symbol, Number))
-        set([2, x, y])
         """
-        result = set()
-        if type is not None and not isinstance(type, (type_class, tuple)):
-            type = sympify(type).__class__
+        if len(types) == 1 and not isinstance(types[0], type):
+            types = (sympify(types[0]).__class__,)
+
+        result = set([])
+
         if self.is_Atom:
-            if type is None or isinstance(self, type):
+            if not types or isinstance(self, types):
                 result.add(self)
         else:
-            for obj in self.args:
-                result = result.union(obj.atoms(type=type))
+            for obj in self.iter_basic_args():
+                result |= obj.atoms(*types)
+
         return result
 
-    def is_hypergeometric(self, arg):
+    def is_hypergeometric(self, k):
         from sympy.simplify import hypersimp
-        return hypersimp(self, arg, simplify=False) is not None
+        return hypersimp(self, k) is not None
 
     @property
     def is_number(self):
@@ -492,12 +504,9 @@ class Basic(AssumeMeths):
            True
 
         """
-        for obj in self.args:
-            if isinstance(obj, Basic):
-                if not obj.is_number:
-                    return False
-            else:
-                raise TypeError
+        for obj in self.iter_basic_args():
+            if not obj.is_number:
+                return False
         else:
             return True
 
@@ -506,19 +515,18 @@ class Basic(AssumeMeths):
         """
         The top-level function in an expression.
 
-        The following should hold for all objects:
+        The following should hold for all objects::
 
-            x = x.func(*x.args)
+            >> x == x.func(*x.args)
 
         """
         return self.__class__
 
     @property
     def args(self):
-        """Returns a tuple of arguments of "self".
+        """Returns a tuple of arguments of 'self'.
 
-        Example
-        -------
+        Example:
 
         >>> from sympy import symbols, cot
         >>> x, y = symbols('xy')
@@ -542,6 +550,10 @@ class Basic(AssumeMeths):
         """
         return self._args[:]
 
+    def iter_basic_args(self):
+        """Iterates arguments of 'self' with are Basic instances. """
+        return iter(self.args)
+
     def is_fraction(self, *syms):
         p, q = self.as_numer_denom()
 
@@ -558,16 +570,13 @@ class Basic(AssumeMeths):
         if syms:
             syms = map(sympify, syms)
         else:
-            syms = list(self.atoms(type=Symbol))
+            syms = list(self.atoms(Symbol))
 
         if not syms: # constant polynomial
             return True
         else:
             return self._eval_is_polynomial(syms)
 
-    def as_polynomial(self, *syms, **kwargs): # remove
-        return C.Polynomial(self, var=(syms or None), **kwargs)
-
     def as_poly(self, *symbols, **flags):
         """Converts 'self' to a polynomial or returns None.
 
@@ -576,11 +585,18 @@ class Basic(AssumeMeths):
            There are situations when it is easier (simpler or prettier)
            to receive None on failure.
 
+           If no symbols were given and 'self' isn't already a polynomial
+           then all  available symbols will be collected and used to form
+           a new polynomial.
+
            >>> from sympy import *
            >>> x,y = symbols('xy')
 
+           >>> print (x**2 + x*y).as_poly()
+           Poly(x**2 + x*y, x, y)
+
            >>> print (x**2 + x*y).as_poly(x, y)
-           Poly((1, 1), ((2, 0), (1, 1)), (x, y), 'grlex')
+           Poly(x**2 + x*y, x, y)
 
            >>> print (x**2 + sin(y)).as_poly(x, y)
            None
@@ -589,6 +605,12 @@ class Basic(AssumeMeths):
         from sympy.polys import Poly, PolynomialError
 
         try:
+            if not symbols:
+                if isinstance(self, Poly):
+                    return self
+                else:
+                    symbols = sorted(self.atoms(Symbol))
+
             return Poly(self, *symbols, **flags)
         except PolynomialError:
             return None
@@ -765,10 +787,13 @@ class Basic(AssumeMeths):
             return True
         else:
             def search(expr):
-                if expr.is_Symbol:
-                    return expr in syms
+                if expr.is_Atom:
+                    if expr.is_Symbol:
+                        return expr in syms
+                    else:
+                        return False
                 else:
-                    for term in expr.args:
+                    for term in expr.iter_basic_args():
                         if search(term):
                             return True
                     else:
@@ -796,11 +821,11 @@ class Basic(AssumeMeths):
             return True
         else:
             def search(expr):
-                if expr.is_Symbol:
-                    if expr in syms:
+                if expr.is_Atom:
+                    if expr.is_Symbol and expr in syms:
                         syms.remove(expr)
                 else:
-                    for term in expr.args:
+                    for term in expr.iter_basic_args():
                         if not syms:
                             break
                         else:
@@ -825,9 +850,12 @@ class Basic(AssumeMeths):
         if p.is_Symbol and not isinstance(p, Wild): # speeds up
             return p in self.atoms(p.__class__)
         if isinstance(p, BasicType):
-            #XXX hack, this is very fragile:
+            #XXX this is very fragile:
             if str(self).find(str(p.__name__)) == -1:
-                #didn't find p in self
+                #didn't find p in self, let's try corner cases
+                if p is Derivative:
+                    if str(self).find("D(") != -1:
+                        return True
                 return False
             else:
                 return True
@@ -848,12 +876,6 @@ class Basic(AssumeMeths):
     def _eval_derivative(self, s):
         return
 
-    def _eval_integral(self, s):
-        return
-
-    def _eval_defined_integral(self, s, a, b):
-        return
-
     def _eval_fapply(self, *args, **assumptions):
         return
 
@@ -870,12 +892,6 @@ class Basic(AssumeMeths):
     def _eval_apply_subs(cls, *args):
         return
 
-    def _calc_apply_positive(self, *args):
-        return
-
-    def _calc_apply_real(self, *args):
-        return
-
     def _eval_conjugate(self):
         if self.is_real:
             return self
@@ -984,17 +1000,16 @@ class Basic(AssumeMeths):
         return pattern.matches(self, {})
 
     def solve4linearsymbol(eqn, rhs, symbols = None):
-        """ Solve equation
-          eqn == rhs
-        with respect to some linear symbol in eqn.
-        Returns (symbol, solution). If eqn is nonlinear
-        with respect to all symbols, then return
-        trivial solution (eqn, rhs).
+        """
+        Solve equation "eqn == rhs" with respect to some linear symbol in eqn.
+
+        Returns (symbol, solution). If eqn is nonlinear with respect to all
+        symbols, then return trivial solution (eqn, rhs).
         """
         if eqn.is_Symbol:
             return (eqn, rhs)
         if symbols is None:
-            symbols = eqn.atoms(type=Symbol)
+            symbols = eqn.atoms(Symbol)
         if symbols:
             # find  symbol
             for s in symbols:
@@ -1005,22 +1020,6 @@ class Basic(AssumeMeths):
         # no linear symbol, return trivial solution
         return eqn, rhs
 
-    def _calc_splitter(self, d):
-        if d.has_key(self):
-            return d[self]
-        r = self.__class__(*[t._calc_splitter(d) for t in self])
-        if d.has_key(r):
-            return d[r]
-        s = d[r] = Temporary()
-        return s
-
-    def splitter(self):
-        d = {}
-        r = self._calc_splitter(d)
-        l = [(s.dummy_index,s,e) for e,s in d.items()]
-        l.sort()
-        return [(s,e) for i,s,e in l]
-
     @cacheit
     def count_ops(self, symbolic=True):
         """ Return the number of operations in expressions.
@@ -1347,17 +1346,6 @@ class Basic(AssumeMeths):
                 return self, tuple()
         return S.One, (self,)
 
-    def as_indep_terms(self, x):
-        coeff, terms = self.as_coeff_terms()
-        indeps = [coeff]
-        new_terms = []
-        for t in terms:
-            if t.has(x):
-                new_terms.append(x)
-            else:
-                indeps.append(x)
-        return Mul(*indeps), Mul(*new_terms)
-
     def as_coeff_factors(self, x=None):
         # a -> c + f
         if x is not None:
@@ -1374,23 +1362,6 @@ class Basic(AssumeMeths):
             return S.One, base ** (-exp)
         return self, S.One
 
-    def as_expr_orders(self):
-        """ Split expr + Order(..) to (expr, Order(..)).
-        """
-        l1 = []
-        l2 = []
-        if self.is_Add:
-            for f in self:
-                if isinstance(f, C.Order):
-                    l2.append(f)
-                else:
-                    l1.append(f)
-        elif self.is_Order:
-            l2.append(self)
-        else:
-            l1.append(self)
-        return Add(*l1), Add(*l2)
-
     def normal(self):
         n, d = self.as_numer_denom()
         if d is S.One:
@@ -1430,57 +1401,85 @@ class Basic(AssumeMeths):
     def __call__(self, *args, **removeme):
         return Function(self[0])(*args)
 
-    def _eval_evalf(self):
-        return
-
-    def _seq_eval_evalf(self):
-        return self.__class__(*[s.evalf() for s in self.args])
-
     def __float__(self):
-        result = self.evalf(precision=16)
-
+        result = self.evalf()
         if result.is_Number:
             return float(result)
         else:
             raise ValueError("Symbolic value, can't compute")
 
-    def evalf(self, precision=None):
-        if precision is None:
-            r = self._eval_evalf()
-        else:
-            old_precision = Basic.set_precision(precision)
-            r = self._eval_evalf()
-            Basic.set_precision(old_precision)
+    def _evalf(self, prec):
+        """Helper for evalf. Does the same thing but takes binary precision"""
+        r = self._eval_evalf(prec)
         if r is None:
             r = self
         return r
 
-    @staticmethod
-    def set_precision(prec = None):
-        """
-        Set precision for Decimal number operations and return previous precision value.
-        """
-        context = decimal.getcontext()
-        oldprec = context.prec
-        if prec is not None:
-            context.prec = prec
-        return oldprec
+    def _eval_evalf(self, prec):
+        return
 
+    def _seq_eval_evalf(self, prec):
+        return self.__class__(*[s._evalf(prec) for s in self.args])
+
+    def evalf(self, prec=15):
+        """Numerically evaluate to the specified precision."""
+        return self._evalf(mpmath.lib.dps_to_prec(prec))
+
+    def _to_mpmath(self, prec, allow_ints=True):
+        # mpmath functions accept ints as input
+        errmsg = "cannot convert to mpmath number"
+        if allow_ints and self.is_Integer:
+            return self.p
+        v = self._eval_evalf(prec)
+        if v is None:
+            raise ValueError(errmsg)
+        if v.is_Real:
+            return mpmath.make_mpf(v._mpf_)
+        # Number + Number*I is also fine
+        re, im = v.as_real_imag()
+        if allow_ints and re.is_Integer:
+            re = mpmath.lib.from_int(re.p)
+        elif re.is_Real:
+            re = re._mpf_
+        else:
+            raise ValueError(errmsg)
+        if allow_ints and im.is_Integer:
+            im = mpmath.lib.from_int(im.p)
+        elif im.is_Real:
+            im = im._mpf_
+        else:
+            raise ValueError(errmsg)
+        return mpmath.make_mpc((re, im))
+
+    @staticmethod
+    def _from_mpmath(x, prec):
+        if hasattr(x, "_mpf_"):
+            return C.Real._new(x._mpf_, prec)
+        elif hasattr(x, "_mpc_"):
+            re, im = x._mpc_
+            re = C.Real._new(re, prec)
+            im = C.Real._new(im, prec)*S.ImaginaryUnit
+            return re+im
+        else:
+            raise TypeError("expected mpmath number (mpf or mpc)")
 
     ###################################################################################
     ##################### SERIES, LEADING TERM, LIMIT, ORDER METHODS ##################
     ###################################################################################
 
-    def series(self, x, point=0, n=6):
+    def series(self, x, point=0, n=6, with_order=True):
         """
-        Usage
-        =====
+        Series expansion of "self" around "point".
+
+        Usage:
             Returns the Taylor (Laurent or generalized) series of "self" around
             the point "point" (default 0) with respect to "x" until the n-th
             term (default n is 6).
 
-        Notes
-        =====
+            with_order .... if False, the order term (see the class Order) is
+                not appended
+
+        Notes:
             This method is the most high level method and it returns the
             series including the O(x**n) term.
 
@@ -1497,7 +1496,9 @@ class Basic(AssumeMeths):
         r = self.oseries(o)
         if r==self:
             return self
-        return r + o
+        if with_order:
+            r += o
+        return r
 
     @cacheit
     def oseries(self, order):
@@ -1625,19 +1626,12 @@ class Basic(AssumeMeths):
             return d[wc], d[we]
         return self, S.Zero
 
-    def ldegree(self, x):
-        x = sympify(x)
-        c,e = self.as_leading_term(x).as_coeff_exponent(x)
-        if not c.has(x):
-            return e
-        raise ValueError("cannot compute ldegree(%s, %s), got c=%s" % (self, x, c))
-
     def leadterm(self, x):
         x = sympify(x)
         c,e = self.as_leading_term(x).as_coeff_exponent(x)
         if not c.has(x):
             return c,e
-        raise ValueError("cannot compute ldegree(%s, %s), got c=%s" % (self, x, c))
+        raise ValueError("cannot compute leadterm(%s, %s), got c=%s" % (self, x, c))
 
 
     ##########################################################################
@@ -1670,25 +1664,12 @@ class Atom(Basic):
     def as_numer_denom(self):
         return self, S.One
 
-    def _calc_splitter(self, d):
-        return self
-
     def count_ops(self, symbolic=True):
         return S.Zero
 
     def doit(self, **hints):
         return self
 
-    def _eval_integral(self, s):
-        if s==self:
-            return self**2/2
-        return self*s
-
-    def _eval_defined_integral(self, s, a, b):
-        if s==self:
-            return (b**2-a**2)/2
-        return self*(b-a)
-
     def _eval_is_polynomial(self, syms):
         return True
 
