@@ -40,6 +40,8 @@ from symbol import Symbol
 from add    import Add
 from multidimensional import vectorize
 
+from sympy import mpmath
+
 class FunctionClass(BasicMeta):
     """
     Base class for function classes. FunctionClass is a subclass of type.
@@ -126,6 +128,8 @@ class Function(Basic, ArithMeths, RelMeths):
             if opt in options:
                 del options[opt]
         # up to here.
+        if options.get('evaluate') is False:
+            return Basic.__new__(cls, *args, **options)
         r = cls.canonize(*args, **options)
         if isinstance(r, Basic):
             return r
@@ -156,7 +160,7 @@ class Function(Basic, ArithMeths, RelMeths):
         def canonize(cls, arg):
             if arg is S.NaN:
                 return S.NaN
-            if arg is S.Zero: return S.One
+            if arg is S.Zero: return S.Zero
             if arg.is_positive: return S.One
             if arg.is_negative: return S.NegativeOne
             if isinstance(arg, C.Mul):
@@ -185,11 +189,33 @@ class Function(Basic, ArithMeths, RelMeths):
     def _eval_expand_basic(self, *args):
         return None
 
-    def _eval_evalf(self):
-        obj = self.func._eval_apply_evalf(*self.args[:])
-        if obj is None:
-            return self
-        return obj
+    def _eval_evalf(self, prec):
+        # Lookup mpmath function based on name
+        fname = self.func.__name__
+        try:
+            if not hasattr(mpmath, fname):
+                from sympy.utilities.lambdify import MPMATH_TRANSLATIONS
+                fname = MPMATH_TRANSLATIONS[fname]
+            func = getattr(mpmath, fname)
+        except (AttributeError, KeyError):
+            return
+
+        # Convert all args to mpf or mpc
+        try:
+            args = [arg._to_mpmath(prec) for arg in self.args]
+        except ValueError:
+            return
+
+        # Set mpmath precision and apply. Make sure precision is restored
+        # afterwards
+        orig = mpmath.mp.prec
+        try:
+            mpmath.mp.prec = prec
+            v = func(*args)
+        finally:
+            mpmath.mp.prec = orig
+
+        return Basic._from_mpmath(v, prec)
 
     def _eval_is_comparable(self):
         if self.is_Function:
@@ -224,15 +250,6 @@ class Function(Basic, ArithMeths, RelMeths):
             if not c: r = False
         return r
 
-    def _calc_positive(self):
-        return self.func._calc_apply_positive(*self[:])
-
-    def _calc_real(self):
-        return self.func._calc_apply_real(*self[:])
-
-    def _calc_unbounded(self):
-        return self.func._calc_apply_unbounded(*self[:])
-
     def _eval_eq_nonzero(self, other):
         if isinstance(other.func, self.func.__class__) and len(self)==len(other):
             for a1,a2 in zip(self,other):
@@ -329,7 +346,7 @@ class Function(Basic, ArithMeths, RelMeths):
 
     @classmethod
     def _eval_apply_evalf(cls, arg):
-        arg = arg.evalf()
+        arg = arg.evalf(prec)
 
         #if cls.nargs == 1:
         # common case for functions with 1 argument
@@ -446,10 +463,8 @@ class Derivative(Basic, ArithMeths, RelMeths):
                         yield copy_s
                 else:
                     yield s
-            elif isinstance(s, Function):
-                yield s
             else:
-                 raise TypeError("Derivative argument must be Symbol|Integer|Function instance (got %s)" % (s.__class__.__name__))
+                yield s
 
     def __new__(cls, expr, *symbols, **assumptions):
         expr = sympify(expr)
@@ -593,7 +608,7 @@ class Lambda(Function):
         nargs = len(args)
 
         expression = args[nargs-1]
-        funargs = [Symbol(str(arg),dummy=True) for arg in args[:nargs-1]]
+        funargs = [Symbol(arg.name, dummy=True) for arg in args[:nargs-1]]
         #probably could use something like foldl here
         for arg,funarg in zip(args[:nargs-1],funargs):
             expression = expression.subs(arg,funarg)
