@@ -1,11 +1,12 @@
 
 from basic import Basic
+from sympify import _sympify
 
 class Interval(Basic):
 
     def __new__(cls, start, end, **assumptions):
-        start = Basic.sympify(start)
-        end = Basic.sympify(end)
+        start = _sympify(start)
+        end = _sympify(end)
         return Basic.__new__(cls, start, end, **assumptions)
 
     @property
@@ -22,4 +23,9 @@ class Interval(Basic):
             r = '(%s)' % (r)
         return r
 
-    
+
+# /cyclic/
+import sympify as _
+_.Interval  = Interval
+del _
+
