@@ -10,6 +10,8 @@ from symbol import Symbol, Wild
 # from add   import Add /cyclic/
 # from power import Pow /cyclic/
 
+import sympy.mpmath as mpmath
+
 class Mul(AssocOp, RelMeths, ArithMeths):
 
     __slots__ = []
@@ -290,8 +292,8 @@ class Mul(AssocOp, RelMeths, ArithMeths):
         #if e.atoms(Wild):
         #    return Mul(*[t**e for t in b])
 
-    def _eval_evalf(self):
-        return self.__class__(*[s.evalf() for s in self.args]).expand()
+    def _eval_evalf(self, prec):
+        return AssocOp._eval_evalf(self, prec).expand()
 
     @property
     def precedence(self):
@@ -471,33 +473,6 @@ class Mul(AssocOp, RelMeths, ArithMeths):
             return Add(*[t.count_ops(symbolic) for t in self[:]]) + Symbol('MUL') * (len(self[:])-1)
         return Add(*[t.count_ops(symbolic) for t in self.args[:]]) + (len(self.args)-1)
 
-    def _eval_integral(self, s):
-        coeffs = []
-        terms = []
-        for t in self:
-            if not t.has(s): coeffs.append(t)
-            else: terms.append(t)
-        if coeffs:
-            return Mul(*coeffs) * Mul(*terms).integral(s)
-        u = self[0].integral(s)
-        v = Mul(*(self[1:]))
-        uv = u * v
-        return uv - (u*v.diff(s)).integral(s)
-
-    def _eval_defined_integral(self, s, a, b):
-        coeffs = []
-        terms = []
-        for t in self:
-            if not t.has(s): coeffs.append(t)
-            else: terms.append(t)
-        if coeffs:
-            return Mul(*coeffs) * Mul(*terms).integral(s==[a,b])
-        # (u'v) -> (uv) - (uv')
-        u = self[0].integral(s)
-        v = Mul(*(self[1:]))
-        uv = u * v
-        return (uv.subs(s,b) - uv.subs(s,a)) - (u*v.diff(s)).integral(s==[a,b])
-
     def _eval_is_polynomial(self, syms):
         for term in self.args:
             if not term._eval_is_polynomial(syms):
@@ -580,7 +555,14 @@ class Mul(AssocOp, RelMeths, ArithMeths):
         if terms1==terms2: # (2*a).subs(3*a,y) -> 2/3*y
             return new * coeff1/coeff2
         l1,l2 = len(terms1),len(terms2)
-        if l2<l1: # (a*b*c*d).subs(b*c,x) -> a*x*d
+        if l2 == 0:
+            # if old is just a number, go through the self.args one by one
+            return Mul(*[x.subs(old, new) for x in self.args])
+        elif l2<l1:
+            # old is some something more complex, like:
+            # (a*b*c*d).subs(b*c,x) -> a*x*d
+            # then we need to search where in self.args the "old" is, and then
+            # correctly substitute both terms and coefficients.
             for i in xrange(l1-l2+1):
                 if terms2==terms1[i:i+l2]:
                     m1 = Mul(*terms1[:i]).subs(old,new)
