@@ -1,6 +1,11 @@
 import math
+import sympy.mpmath as mpmath
+import sympy.mpmath.lib as mlib
+import sympy.mpmath.libmpc as mlibc
 import decimal
-import decimal_math
+
+rnd = mlib.round_nearest
+
 from basic import Basic, Atom, Singleton, S, C, Memoizer, MemoizerArg
 from sympify import _sympify, SympifyError, _sympifyit
 from methods import NoRelMeths, RelMeths, ArithMeths
@@ -14,21 +19,26 @@ from power import integer_nthroot
 _gcdcache = {}
 
 # TODO caching with decorator, but not to degrade performance
-def gcd(a, b):
-    """
-    Returns the Greatest Common Divisor, implementing Euclid's algorithm.
+def igcd(a, b):
+    """Computes integer greates common divisor of two numbers.
+
+       The algorithm is based on the well known Euclid's algorithm. To
+       improve speed igcd() has its own caching mechanizm implemented.
     """
     try:
         return _gcdcache[(a,b)]
     except KeyError:
-        key = (a,b)
-        while a:
-            a, b = b%a, a
+        if a and b:
+            if b < 0:
+                b = -b
 
-        _gcdcache[key] = b
-        return b
+            while b:
+                a, b = b, a % b
+        else:
+            a = abs(a or b)
 
-igcd = gcd # TODO: rename gcd -> igcd in all modules
+        _gcdcache[(a,b)] = a
+        return a
 
 def ilcm(a, b):
     """Computes integer least common multiple of two numbers. """
@@ -37,6 +47,47 @@ def ilcm(a, b):
     else:
         return a * b / igcd(a, b)
 
+def igcdex(a, b):
+    """Returns x, y, g such that g = x*a + y*b = gcd(a, b).
+
+       >>> igcdex(2, 3)
+       (-1, 1, 1)
+       >>> igcdex(10, 12)
+       (-1, 1, 2)
+
+       >>> x, y, g = igcdex(100, 2004)
+       >>> x, y, g
+       (-20, 1, 4)
+       >>> x*100 + y*2004
+       4
+
+    """
+    if (not a) and (not b):
+        return (0, 1, 0)
+
+    if not a:
+        return (0, b/abs(b), abs(b))
+    if not b:
+        return (a/abs(a), 0, abs(a))
+
+    if a < 0:
+        a, x_sign = -a, -1
+    else:
+        x_sign = 1
+
+    if b < 0:
+        b, y_sign = -b, -1
+    else:
+        y_sign = 1
+
+    x, y, r, s = 1, 0, 0, 1
+
+    while b:
+        (c, q) = (a % b, a / b)
+        (a, b, r, s, x, y) = (b, c, x-q*r, y-q*s, r, s)
+
+    return (x*x_sign, y*y_sign, a)
+
 @Memoizer((int, long), return_value_converter = lambda d: d.copy())
 def factor_trial_division(n):
     """
@@ -96,6 +147,9 @@ class Number(Atom, RelMeths, ArithMeths):
 
     __slots__ = []
 
+    # Used to make max(x._prec, y._prec) return x._prec when only x is a float
+    _prec = -1
+
     is_Number = True
 
     def __new__(cls, *obj):
@@ -104,21 +158,26 @@ class Number(Atom, RelMeths, ArithMeths):
             return Integer(obj)
         if isinstance(obj,tuple) and len(obj)==2:
             return Rational(*obj)
-        if isinstance(obj, (str,float,decimal.Decimal)):
+        if isinstance(obj, (str,float,mpmath.mpf,decimal.Decimal)):
             return Real(obj)
         if isinstance(obj, Number):
             return obj
         raise TypeError("expected str|int|long|float|Decimal|Number object but got %r" % (obj))
 
-    def _eval_evalf(self):
-        r = self._as_decimal()
-        return Real(r)
+    def _as_mpf_val(self, prec):
+        """Evaluate to mpf tuple accurate to at least prec bits"""
+        raise NotImplementedError('%s needs ._as_mpf_val() method' % \
+            (self.__class__.__name__))
+
+    def _eval_evalf(self, prec):
+        return Real._new(self._as_mpf_val(prec), prec)
+
+    def _as_mpf_op(self, prec):
+        prec = max(prec, self._prec)
+        return self._as_mpf_val(prec), prec
 
     def __float__(self):
-        return float(self._as_decimal())
-
-    def _as_decimal(self):
-        raise NotImplementedError('%s needs ._as_decimal() method' % (self.__class__.__name__))
+        return mlib.to_float(self._as_mpf_val(53))
 
     def _eval_derivative(self, s):
         return S.Zero
@@ -130,36 +189,14 @@ class Number(Atom, RelMeths, ArithMeths):
         # Order(5, x, y) -> Order(1,x,y)
         return C.Order(S.One, *symbols)
 
-    def sqrt(self): return Real(decimal_math.sqrt(self._as_decimal()))
-    def exp(self): return Real(decimal_math.exp(self._as_decimal()))
-    def log(self): return Real(decimal_math.log(self._as_decimal()))
-    def sin(self): return Real(decimal_math.sin(self._as_decimal()))
-    def cos(self): return Real(decimal_math.cos(self._as_decimal()))
-    def tan(self): return Real(decimal_math.tan(self._as_decimal()))
-    def cot(self): return Real(decimal_math.cot(self._as_decimal()))
-    def asin(self): return Real(decimal_math.asin(self._as_decimal()))
-    def acos(self): return Real(decimal_math.acos(self._as_decimal()))
-    def atan(self): return Real(decimal_math.atan(self._as_decimal()))
-    def acot(self): return Real(decimal_math.acot(self._as_decimal()))
-    def sinh(self): return Real(decimal_math.sinh(self._as_decimal()))
-    def cosh(self): return Real(decimal_math.cosh(self._as_decimal()))
-    def tanh(self): return Real(decimal_math.tanh(self._as_decimal()))
-    def coth(self): return Real(decimal_math.coth(self._as_decimal()))
-    def asinh(self): return Real(decimal_math.asinh(self._as_decimal()))
-    def acosh(self): return Real(decimal_math.acosh(self._as_decimal()))
-    def atanh(self): return Real(decimal_math.atanh(self._as_decimal()))
-    def acoth(self): return Real(decimal_math.acoth(self._as_decimal()))
-    def floor(self): return Real(decimal_math.floor(self._as_decimal()))
-    def ceiling(self): return Real(decimal_math.ceiling(self._as_decimal()))
-
     def __eq__(self, other):
-        raise NotImplementedError,'%s needs .__eq__() method' % (self.__class__.__name__)
+        raise NotImplementedError('%s needs .__eq__() method' % (self.__class__.__name__))
     def __ne__(self, other):
-        raise NotImplementedError,'%s needs .__ne__() method' % (self.__class__.__name__)
+        raise NotImplementedError('%s needs .__ne__() method' % (self.__class__.__name__))
     def __lt__(self, other):
-        raise NotImplementedError,'%s needs .__lt__() method' % (self.__class__.__name__)
+        raise NotImplementedError('%s needs .__lt__() method' % (self.__class__.__name__))
     def __le__(self, other):
-        raise NotImplementedError,'%s needs .__le__() method' % (self.__class__.__name__)
+        raise NotImplementedError('%s needs .__le__() method' % (self.__class__.__name__))
 
     def __gt__(self, other):
         return _sympify(other).__lt__(self)
@@ -170,22 +207,6 @@ class Number(Atom, RelMeths, ArithMeths):
         # a -> c * t
         return self, tuple()
 
-decimal_to_Number_cls = {
-    decimal.Decimal('0').as_tuple():'Zero',
-    decimal.Decimal('-0').as_tuple():'Zero',
-    decimal.Decimal('1').as_tuple():'One',
-    decimal.Decimal('-1').as_tuple():'NegativeOne',
-    decimal.Decimal('Infinity').as_tuple():'Infinity',
-    decimal.Decimal('-Infinity').as_tuple():'NegativeInfinity',
-    decimal.Decimal('NaN').as_tuple():'NaN',
-    }
-
-def convert_to_Decimal(num):
-    if isinstance(num, str):
-        num = decimal.Decimal(num)
-    elif isinstance(num, float):
-        num = Real.float_to_decimal(num)
-    return +num
 
 class Real(Number):
     """
@@ -205,77 +226,86 @@ class Real(Number):
     is_irrational = False
     is_integer = False
 
-    __slots__ = ['num']
+    __slots__ = ['_mpf_', '_prec']
+
+    # mpz can't be pickled
+    def __getstate__(self):
+        d = Basic.__getstate__(self).copy()
+        del d["_mpf_"]
+        return mlib.to_pickable(self._mpf_), d
+
+    def __setstate__(self, state):
+        _mpf_, d = state
+        _mpf_ = mlib.from_pickable(_mpf_)
+        self._mpf_ = _mpf_
+        Basic.__setstate__(self, d)
 
     is_Real = True
 
-    @Memoizer(type, MemoizerArg((str, int, long, float, decimal.Decimal), convert_to_Decimal))
-    def __new__(cls, num):
+    def floor(self):
+        return C.Integer(int(mlib.to_int(mlib.ffloor(self._mpf_, self._prec))))
+
+    def ceiling(self):
+        return C.Integer(int(mlib.to_int(mlib.fceil(self._mpf_, self._prec))))
+
+    @property
+    def num(self):
+        return mpmath.mpf(self._mpf_)
+
+    def _as_mpf_val(self, prec):
+        return self._mpf_
+
+    def _as_mpf_op(self, prec):
+        return self._mpf_, max(prec, self._prec)
+
+    def __new__(cls, num, prec=15):
+        prec = mlib.dps_to_prec(prec)
         if isinstance(num, (int, long)):
             return Integer(num)
-
-        singleton_cls_name = decimal_to_Number_cls.get(num.as_tuple(), None)
-        if singleton_cls_name is not None:
-            return getattr(C, singleton_cls_name)()
+        if isinstance(num, (str, decimal.Decimal)):
+            _mpf_ = mlib.from_str(str(num), prec)
+        elif isinstance(num, tuple) and len(num) == 4:
+            _mpf_ = num
+        else:
+            _mpf_ = mpmath.mpf(num)._mpf_
+        if not num:
+            return C.Zero()
         obj = Basic.__new__(cls)
-        obj.num = num
+        obj._mpf_ = _mpf_
+        obj._prec = prec
         return obj
 
-    @staticmethod
-    def float_to_decimal(f):
-        "Convert a floating point number to a Decimal with no loss of information"
-        # Transform (exactly) a float to a mantissa (0.5 <= abs(m) < 1.0) and an
-        # exponent.  Double the mantissa until it is an integer.  Use the integer
-        # mantissa and exponent to compute an equivalent Decimal.  If this cannot
-        # be done exactly, then retry with more precision.
-
-        try:
-            mantissa, exponent = math.frexp(f)
-        except OverflowError:
-            return decimal.Inf
-
-        while mantissa != int(mantissa):
-            mantissa *= 2.0
-            exponent -= 1
-        mantissa = int(mantissa)
-
-        oldcontext = decimal.getcontext()
-        decimal.setcontext(decimal.Context(traps=[decimal.Inexact]))
-        try:
-            while True:
-                try:
-                    return mantissa * decimal.Decimal(2) ** exponent
-                except decimal.Inexact:
-                    decimal.getcontext().prec += 1
-        finally:
-            decimal.setcontext(oldcontext)
+    @classmethod
+    def _new(cls, _mpf_, _prec):
+        if _mpf_ == mlib.fzero:
+            return S.Zero
+        obj = Basic.__new__(cls)
+        obj._mpf_ = _mpf_
+        obj._prec = _prec
+        return obj
 
     def _hashable_content(self):
-        return (self.num,)
+        return (self._mpf_, self._prec)
 
     def tostr(self, level=0):
-        r = str(self.num.normalize())
+        r = mlib.to_str(self._mpf_, mlib.prec_to_dps(self._prec))
         if self.precedence<=level:
             return '(%s)' % (r)
         return r
 
     def torepr(self):
-        return '%s(%r)' % (self.__class__.__name__, str(self.num))
+        dps = mlib.prec_to_dps(self._prec)
+        r = mlib.to_str(self._mpf_, dps+3)
+        return '%s(%r, prec=%i)' % (self.__class__.__name__, r, dps)
 
     def _eval_is_positive(self):
-        return self.num.as_tuple()[0] == 0
+        return self.num > 0
 
     def _eval_is_negative(self):
-        return self.num.as_tuple()[0] == 1
-
-    def _eval_evalf(self):
-        return self
-
-    def _as_decimal(self):
-        return self.num
+        return self.num < 0
 
     def __neg__(self):
-        return Real(-self.num)
+        return Real._new(mlib.fneg(self._mpf_), self._prec)
 
     def __mul__(self, other):
         try:
@@ -283,7 +313,8 @@ class Real(Number):
         except SympifyError:
             return NotImplemented
         if isinstance(other, Number):
-            return Real(self.num * other._as_decimal())
+            rhs, prec = other._as_mpf_op(self._prec)
+            return Real._new(mlib.fmul(self._mpf_, rhs, prec, rnd), prec)
         return Number.__mul__(self, other)
 
     def __add__(self, other):
@@ -294,7 +325,8 @@ class Real(Number):
         if (other is S.NaN) or (self is NaN):
             return S.NaN
         if isinstance(other, Number):
-            return Real(self.num + other._as_decimal())
+            rhs, prec = other._as_mpf_op(self._prec)
+            return Real._new(mlib.fadd(self._mpf_, rhs, prec, rnd), prec)
         return Number.__add__(self, other)
 
     def _eval_power(b, e):
@@ -307,27 +339,22 @@ class Real(Number):
         """
         if isinstance(e, Number):
             if isinstance(e, Integer):
-                e = e.p
-                return Real(decimal_math.pow(b.num, e))
-
-            e2 = e._as_decimal()
-            if b.is_negative and not e.is_integer:
-                m = decimal_math.pow(-b.num, e2)
-                a = decimal_math.pi() * e2
-                s = m * decimal_math.sin(a)
-                c = m * decimal_math.cos(a)
-                return Real(s) + Real(c) * S.ImaginaryUnit
-            return Real(decimal_math.pow(b.num, e2))
-        return
+                prec = b._prec
+                return Real._new(mlib.fpowi(b._mpf_, e.p, prec, rnd), prec)
+            e, prec = e._as_mpf_op(b._prec)
+            b = b._mpf_
+            try:
+                y = mlib.fpow(b, e, prec, rnd)
+                return Real._new(y, prec)
+            except mlib.ComplexResult:
+                re, im = mlibc.mpc_pow((b, mlib.fzero), (e, mlib.fzero), prec, rnd)
+                return Real._new(re, prec) + Real._new(im, prec) * S.ImaginaryUnit
 
     def __abs__(self):
-        return Real(abs(self.num))
+        return Real._new(mlib.fabs(self._mpf_), self._prec)
 
     def __int__(self):
-        return int(self.num)
-
-    def __float__(self):
-        return float(self.num)
+        return int(mlib.to_int(self._mpf_))
 
     def __eq__(self, other):
         try:
@@ -339,8 +366,7 @@ class Real(Number):
             return other.__eq__(self)
         if other.is_comparable: other = other.evalf()
         if isinstance(other, Number):
-            return bool(self._as_decimal()==other._as_decimal())
-
+            return bool(mlib.feq(self._mpf_, other._as_mpf_val(self._prec)))
         return False    # Real != non-Number
 
     def __ne__(self, other):
@@ -353,8 +379,7 @@ class Real(Number):
             return other.__ne__(self)
         if other.is_comparable: other = other.evalf()
         if isinstance(other, Number):
-            return bool(self._as_decimal()!=other._as_decimal())
-
+            return bool(not mlib.feq(self._mpf_, other._as_mpf_val(self._prec)))
         return True     # Real != non-Number
 
     def __lt__(self, other):
@@ -366,8 +391,9 @@ class Real(Number):
             return other.__ge__(self)
         if other.is_comparable: other = other.evalf()
         if isinstance(other, Number):
-            return bool(self._as_decimal() < other._as_decimal())
+            return bool(mlib.flt(self._mpf_, other._as_mpf_val(self._prec)))
         return RelMeths.__lt__(self, other)
+
     def __le__(self, other):
         try:
             other = _sympify(other)
@@ -377,7 +403,7 @@ class Real(Number):
             return other.__gt__(self)
         if other.is_comparable: other = other.evalf()
         if isinstance(other, Number):
-            return bool(self._as_decimal()<=other._as_decimal())
+            return bool(mlib.fle(self._mpf_, other._as_mpf_val(self._prec)))
         return RelMeths.__le__(self, other)
 
     def epsilon_eq(self, other, epsilon="10e-16"):
@@ -471,7 +497,7 @@ class Rational(Number):
         if q<0:
             q = -q
             p = -p
-        n = gcd(abs(p), q)
+        n = igcd(abs(p), q)
         if n>1:
             p /= n
             q /= n
@@ -513,7 +539,7 @@ class Rational(Number):
         if (other is S.NaN) or (self is S.NaN):
             return S.NaN
         if isinstance(other, Real):
-            return Real(self._as_decimal() * other.num)
+            return other * self
         if isinstance(other, Rational):
             return Rational(self.p * other.p, self.q * other.q)
         return Number.__mul__(self, other)
@@ -524,7 +550,7 @@ class Rational(Number):
         if (other is S.NaN) or (self is S.NaN):
             return S.NaN
         if isinstance(other, Real):
-            return Real(self._as_decimal() + other.num)
+            return other + self
         if isinstance(other, Rational):
             if self.is_unbounded:
                 if other.is_bounded:
@@ -541,7 +567,7 @@ class Rational(Number):
         if isinstance(e, Number):
             if (e is S.NaN): return S.NaN
             if isinstance(e, Real):
-                return Real(decimal_math.pow(b._as_decimal(), e.num))
+                return b._eval_evalf(e._prec) ** e
             if e.is_negative:
                 # (3/4)**-2 -> (4/3)**2
                 ne = -e
@@ -574,8 +600,8 @@ class Rational(Number):
 
         return
 
-    def _as_decimal(self):
-        return decimal.Decimal(self.p) / decimal.Decimal(self.q)
+    def _as_mpf_val(self, prec):
+        return mlib.from_rational(self.p, self.q, prec, rnd)
 
     def __abs__(self):
         return Rational(abs(self.p), self.q)
@@ -596,7 +622,7 @@ class Rational(Number):
         if other.is_comparable and not isinstance(other, Rational): other = other.evalf()
         if isinstance(other, Number):
             if isinstance(other, Real):
-                return bool(self._as_decimal()==other._as_decimal())
+                return bool(mlib.feq(self._as_mpf_val(other._prec), other._mpf_))
             return bool(self.p==other.p and self.q==other.q)
 
         return False    # Rational != non-Number
@@ -612,7 +638,7 @@ class Rational(Number):
         if other.is_comparable and not isinstance(other, Rational): other = other.evalf()
         if isinstance(other, Number):
             if isinstance(other, Real):
-                return bool(self._as_decimal()!=other._as_decimal())
+                return bool(not mlib.feq(self._as_mpf_val(other._prec), other._mpf_))
             return bool(self.p!=other.p or self.q!=other.q)
 
         return True     # Rational != non-Number
@@ -627,9 +653,10 @@ class Rational(Number):
         if other.is_comparable and not isinstance(other, Rational): other = other.evalf()
         if isinstance(other, Number):
             if isinstance(other, Real):
-                return bool(self._as_decimal() < other._as_decimal())
+                return bool(mlib.flt(self._as_mpf_val(other._prec), other._mpf_))
             return bool(self.p * other.q < self.q * other.p)
         return RelMeths.__lt__(self, other)
+
     def __le__(self, other):
         try:
             other = _sympify(other)
@@ -640,7 +667,7 @@ class Rational(Number):
         if other.is_comparable and not isinstance(other, Rational): other = other.evalf()
         if isinstance(other, Number):
             if isinstance(other, Real):
-                return bool(self._as_decimal()<=other._as_decimal())
+                return bool(mlib.fle(self._as_mpf_val(other._prec), other._mpf_))
             return bool(self.p * other.q <= self.q * other.p)
         return RelMeths.__le__(self, other)
 
@@ -683,6 +710,9 @@ class Integer(Rational):
 
     __slots__ = ['p']
 
+    def _as_mpf_val(self, prec):
+        return mlib.from_int(self.p)
+
     # TODO caching with decorator, but not to degrade performance
     def __new__(cls, i):
         try:
@@ -800,14 +830,11 @@ class Integer(Rational):
     def torepr(self):
         return '%s(%r)' % (self.__class__.__name__, self.p)
 
-    def _eval_evalf(self):
-        return self
-
     def _eval_power(b, e):
         if isinstance(e, Number):
             if e is S.NaN: return S.NaN
             if isinstance(e, Real):
-                return Real(decimal_math.pow(b._as_decimal(), e.num))
+                return b._eval_evalf(e._prec) ** e
             if e.is_negative:
                 # (3/4)**-2 -> (4/3)**2
                 ne = -e
@@ -877,7 +904,7 @@ class Integer(Rational):
                             if sqr_gcd == 0:
                                 sqr_gcd = ex
                             else:
-                                sqr_gcd = gcd(sqr_gcd, ex)
+                                sqr_gcd = igcd(sqr_gcd, ex)
 
                         for k,v in sqr_dict.iteritems():
                             sqr_int *= k**(v/sqr_gcd)
@@ -961,6 +988,9 @@ class One(Singleton, Integer):
 
     __slots__ = []
 
+    def _eval_evalf(self, prec):
+        return self
+
     @staticmethod
     def __abs__():
         return S.One
@@ -982,6 +1012,9 @@ class NegativeOne(Singleton, Integer):
 
     __slots__ = []
 
+    def _eval_evalf(self, prec):
+        return self
+
     @staticmethod
     def __abs__():
         return S.One
@@ -995,10 +1028,7 @@ class NegativeOne(Singleton, Integer):
         if e.is_even: return S.One
         if isinstance(e, Number):
             if isinstance(e, Real):
-                a = e.num * decimal_math.pi()
-                s = decimal_math.sin(a)
-                c = decimal_math.cos(a)
-                return Real(s) + Real(c) * S.ImaginaryUnit
+                return Real(-1.0) ** e
             if e is S.NaN:
                 return S.NaN
             if e is S.Infinity  or  e is S.NegativeInfinity:
@@ -1069,8 +1099,12 @@ class Infinity(Singleton, Rational):
             return b ** d
         return
 
-    def _as_decimal(self):
-        return decimal.Decimal('Infinity')
+    def _as_mpf_val(self, prec):
+        return mlib.finf
+
+    def _sage_(self):
+        import sage.all as sage
+        return sage.oo
 
 class NegativeInfinity(Singleton, Rational):
 
@@ -1120,8 +1154,8 @@ class NegativeInfinity(Singleton, Rational):
             return S.NegativeOne**e * S.Infinity ** e
         return
 
-    def _as_decimal(self):
-        return decimal.Decimal('-Infinity')
+    def _as_mpf_val(self, prec):
+        return mlib.fninf
 
 class NaN(Singleton, Rational):
 
@@ -1139,14 +1173,18 @@ class NaN(Singleton, Rational):
     def tostr(self, level=0):
         return 'nan'
 
-    def _as_decimal(self):
-        return decimal.Decimal('NaN')
+    def _as_mpf_val(self, prec):
+        return mlib.fnan
 
     def _eval_power(b, e):
         if e is S.Zero:
             return S.One
         return b
 
+    def _sage_(self):
+        import sage.all as sage
+        return sage.NaN
+
 class ComplexInfinity(Singleton, Atom, NoRelMeths, ArithMeths):
 
     is_commutative = True
@@ -1197,8 +1235,12 @@ class NumberSymbol(Singleton, Atom, RelMeths, ArithMeths):
         If not implemented, then return None.
         """
 
+    def _eval_evalf(self, prec):
+        return Real._new(self._as_mpf_val(prec), prec)
+
     def _eval_derivative(self, s):
         return S.Zero
+
     def __eq__(self, other):
         try:
             other = _sympify(other)
@@ -1268,8 +1310,8 @@ class Exp1(NumberSymbol):
     def tostr(self, level=0):
         return 'E'
 
-    def _eval_evalf(self):
-        return Real(decimal_math.e())
+    def _as_mpf_val(self, prec):
+        return mlib.fe(prec)
 
     def approximation_interval(self, number_cls):
         if issubclass(number_cls,Integer):
@@ -1280,6 +1322,10 @@ class Exp1(NumberSymbol):
     def _eval_power(self, exp):
         return C.exp(exp)
 
+    def _sage_(self):
+        import sage.all as sage
+        return sage.e
+
 class Pi(NumberSymbol):
 
     is_real = True
@@ -1293,8 +1339,8 @@ class Pi(NumberSymbol):
     def __abs__():
         return S.Pi
 
-    def _eval_evalf(self):
-        return Real(decimal_math.pi())
+    def _as_mpf_val(self, prec):
+        return mlib.fpi(prec)
 
     def approximation_interval(self, number_cls):
         if issubclass(number_cls, Integer):
@@ -1305,6 +1351,10 @@ class Pi(NumberSymbol):
     def tostr(self, level=0):
         return 'pi'
 
+    def _sage_(self):
+        import sage.all as sage
+        return sage.pi
+
 class GoldenRatio(NumberSymbol):
 
     is_real = True
@@ -1314,8 +1364,8 @@ class GoldenRatio(NumberSymbol):
 
     __slots__ = []
 
-    def _eval_evalf(self):
-        return Real(decimal_math.golden_ratio())
+    def _as_mpf_val(self, prec):
+        return mlib.from_man_exp(mpmath.specfun.phi_fixed(prec+10), -prec-10)
 
     def _eval_expand_func(self, *args):
         return S.Half + S.Half*S.Sqrt(5)
@@ -1329,6 +1379,10 @@ class GoldenRatio(NumberSymbol):
     def tostr(self, level=0):
         return 'GoldenRatio'
 
+    def _sage_(self):
+        import sage.all as sage
+        return sage.golden_ratio
+
 class EulerGamma(NumberSymbol):
 
     is_real = True
@@ -1338,8 +1392,8 @@ class EulerGamma(NumberSymbol):
 
     __slots__ = []
 
-    def _eval_evalf(self):
-        return
+    def _as_mpf_val(self, prec):
+        return mlib.from_man_exp(mpmath.specfun.euler_fixed(prec+10), -prec-10)
 
     def approximation_interval(self, number_cls):
         if issubclass(number_cls, Integer):
@@ -1350,6 +1404,10 @@ class EulerGamma(NumberSymbol):
     def tostr(self, level=0):
         return 'EulerGamma'
 
+    def _sage_(self):
+        import sage.all as sage
+        return sage.euler_gamma
+
 class Catalan(NumberSymbol):
 
     is_real = True
@@ -1359,8 +1417,8 @@ class Catalan(NumberSymbol):
 
     __slots__ = []
 
-    def _eval_evalf(self):
-        return
+    def _as_mpf_val(self, prec):
+        return mlib.from_man_exp(mpmath.specfun.catalan_fixed(prec+10), -prec-10)
 
     def approximation_interval(self, number_cls):
         if issubclass(number_cls, Integer):
@@ -1371,6 +1429,10 @@ class Catalan(NumberSymbol):
     def tostr(self, level=0):
         return 'Catalan'
 
+    def _sage_(self):
+        import sage.all as sage
+        return sage.catalan
+
 class ImaginaryUnit(Singleton, Atom, RelMeths, ArithMeths):
 
     is_commutative = True
@@ -1384,6 +1446,9 @@ class ImaginaryUnit(Singleton, Atom, RelMeths, ArithMeths):
     def __abs__():
         return S.One
 
+    def _eval_evalf(self, prec):
+        return self
+
     def tostr(self, level=0):
         return 'I'
 
@@ -1422,6 +1487,10 @@ class ImaginaryUnit(Singleton, Atom, RelMeths, ArithMeths):
     def as_base_exp(self):
         return -S.One, S.Half
 
+    def _sage_(self):
+        import sage.all as sage
+        return sage.I
+
 # /cyclic/
 import basic as _
 _.Number    = Number
