@@ -21,6 +21,8 @@ class AssocOp(Basic):
 
     @cacheit
     def __new__(cls, *args, **assumptions):
+        if assumptions.get('evaluate') is False:
+            return Basic.__new__(cls, *map(_sympify, args), **assumptions)
         if len(args)==0:
             return cls.identity()
         if len(args)==1:
@@ -46,7 +48,7 @@ class AssocOp(Basic):
         if cls is C.Composition:
             s = Symbol('x',dummy=True)
             return Lambda(s,s)
-        raise NotImplementedError,"identity not defined for class %r" % (cls.__name__)
+        raise NotImplementedError("identity not defined for class %r" % (cls.__name__))
 
     @classmethod
     def flatten(cls, seq):
@@ -80,7 +82,7 @@ class AssocOp(Basic):
         wild_part = []
         exact_part = []
         for p in pattern.args:
-            if p.atoms(type=(Wild, WildFunction)):
+            if p.atoms(Wild, WildFunction):
                 wild_part.append(p)
             else:
                 exact_part.append(p)
@@ -117,4 +119,3 @@ class AssocOp(Basic):
         return r
 
     _eval_evalf = Basic._seq_eval_evalf
-
