@@ -4,6 +4,8 @@ from sympify import _sympify
 from methods import ArithMeths, RelMeths
 from cache import cacheit
 
+from sympy import mpmath
+
 from symbol import Symbol, Wild, Temporary
 # from numbers import Number, Rational, Integer     /cyclic/
 # from add import Add   /cyclic/
@@ -23,10 +25,11 @@ def integer_nthroot(y, n):
     >>> integer_nthroot(26,2)
     (5, False)
     """
-    if y < 0: raise ValueError, "y must not be negative"
-    if n < 1: raise ValueError, "n must be positive"
+    if y < 0: raise ValueError("y must not be negative")
+    if n < 1: raise ValueError("n must be positive")
     if y in (0, 1): return y, True
     if n == 1: return y, True
+    if n > y: return 1, False
     # Get initial estimate for Newton's method. Care must be taken to
     # avoid overflow
     try:
@@ -60,6 +63,8 @@ class Pow(Basic, ArithMeths, RelMeths):
     def __new__(cls, a, b, **assumptions):
         a = _sympify(a)
         b = _sympify(b)
+        if assumptions.get('evaluate') is False:
+            return Basic.__new__(cls, a, b, **assumptions)
         if b is S.Zero:
             return S.One
         if b is S.One:
@@ -336,41 +341,35 @@ class Pow(Basic, ArithMeths, RelMeths):
                         else:
                             return Integer(c)/k + I*d/k
 
-                # Consider polynomial:
-                #
-                #   P(x) = sum_{i=0}^n p_i x^k
-                #
-                # and its m-th exponent:
-                #
-                #   P(x)^m = sum_{k=0}^{m n} a(m,k) x^k
-                #
-                # The coefficients a(m,k) can be computed using
-                # the J.C.P. Miller Pure Recurrence:
-                #
-                #  a(m,k) = 1/(k p_0) sum_{i=1}^n p_i ((m+1)i-k) a(m,k-i)
-                #
-                # where a(m,0) = p_0^m.
-                #
-                # For more information refer to:
-                #
-                # [1] D.E.Knuth, The Art of Computer Programming:
-                #     Seminumerical Algorithms, v.2, Addison
-                #     Wesley, Reading, 1981, pp. 751
-
                 p = other_terms
-                m = len(p)-1
-                cache = {0: p[0] ** n}
-                p0 = [t/p[0] for t in p]
-                l = [cache[0]]
-                for k in xrange(1, n * m + 1):
-                    a = []
-                    for i in xrange(1,m+1):
-                        if i<=k:
-                            a.append(Mul(Rational((n+1)*i-k,k), p0[i], cache[k-i]).expand())
-                    a = Add(*a)
-                    cache[k] = a
-                    l.append(a)
-                return Add(*l)
+                # (x+y)**3 -> x**3 + 3*x**2*y + 3*x*y**2 + y**3
+                # in this particular example:
+                # p = [x,y]; n = 3
+                # so now it's easy to get the correct result -- we get the
+                # coefficients first:
+                from sympy import multinomial_coefficients
+                expansion_dict = multinomial_coefficients(len(p), n)
+                # in our example: {(3, 0): 1, (1, 2): 3, (0, 3): 1, (2, 1): 3}
+                # and now construct the expression.
+
+                # An elegant way would be to use Poly, but unfortunately it is
+                # slower than the direct method below, so it is commented out:
+                #b = {}
+                #for k in expansion_dict:
+                #    b[k] = Integer(expansion_dict[k])
+                #return Poly(b, *p).as_basic()
+
+                # use a direct method which is faster:
+                l = []
+                for powers, k in expansion_dict.iteritems():
+                    # assemble a term, e.g. 3*x**2*y
+                    term = [k]
+                    for i,e in enumerate(powers):
+                        term.append(Pow(p[i], powers[i]))
+                    l.append(Mul(*term))
+                # sum up all terms
+                result = Add(*l)
+                return result
             else:
                 if n == 2:
                     return Add(*[f*g for f in base.args for g in base.args])
@@ -397,74 +396,22 @@ class Pow(Basic, ArithMeths, RelMeths):
         dexp = self.exp.diff(s)
         return self * (dexp * C.log(self.base) + dbase * self.exp/self.base)
 
-    def _eval_evalf(self):
+    def _eval_evalf(self, prec):
         base, exp = self.as_base_exp()
-
-        base = base.evalf()
-        exp = exp.evalf()
-
+        base = base._evalf(prec)
+        if not exp.is_Integer:
+            exp = exp._evalf(prec)
         if exp < 0 and not base.is_real:
-            base = base.conjugate() / (base * base.conjugate()).evalf()
+            base = base.conjugate() / (base * base.conjugate())._evalf(prec)
             exp = -exp
-
         return (base ** exp).expand()
 
-    def _calc_splitter(self, d):
-        if d.has_key(self):
-            return d[self]
-        base = self.base._calc_splitter(d)
-        exp = self.exp._calc_splitter(d)
-        if exp.is_Integer:
-            if abs(exp.p)>2:
-                n = exp.p//2
-                r = exp.p - n
-                if n!=r:
-                    p1 = (base ** n)._calc_splitter(d)
-                    p2 = (base ** r)._calc_splitter(d)
-                    r = p1*p2
-                else:
-                    r = (base ** n)._calc_splitter(d) ** 2
-            elif exp.p==-2:
-                r = (1/base)._calc_splitter(d) ** 2
-            else:
-                r = base ** exp
-        else:
-            r = base ** exp
-        if d.has_key(r):
-            return d[r]
-        s = d[r] = Temporary()
-        return s
-
     @cacheit
     def count_ops(self, symbolic=True):
         if symbolic:
             return Add(*[t.count_ops(symbolic) for t in self[:]]) + Symbol('POW')
         return Add(*[t.count_ops(symbolic) for t in self.args[:]]) + 1
 
-    def _eval_integral(self, s):
-        if not self.exp.has(s):
-            if self.base==s:
-                n = self.exp+1
-                return self.base ** n/n
-            y = Symbol('y',dummy=True)
-            x,ix = self.base.solve4linearsymbol(y,symbols=set([s]))
-            if x.is_Symbol:
-                dx = 1/self.base.diff(x)
-                if not dx.has(s):
-                    return (y**self.exp*dx).integral(y).subs(y, self.base)
-
-    def _eval_defined_integral(self, s, a, b):
-        if not self.exp.has(s):
-            if self.base==s:
-                n = self.exp+1
-                return (b**n-a**n)/n
-            x,ix = self.base.solve4linearsymbol(s)
-            if x.is_Symbol:
-                dx = ix.diff(x)
-                if dx.is_Number:
-                    y = Symbol('y',dummy=True)
-                    return (y**self.exp*dx).integral(y==[self.base.subs(s,a), self.base.subs(s,b)])
-
     def _eval_is_polynomial(self, syms):
         if self.exp.has(*syms):
             return False
