@@ -82,7 +82,11 @@ def sympify(a, sympify_lists=False, locals= {}):
     # let's see if 'a' implements conversion methods such as '_sympy_' or
     # '__int__', that returns a SymPy (by definition) or SymPy compatible
     # expression, so we just use it
-    for methname, conv in [('_sympy_',None), ('__int__', Integer)]:
+    for methname, conv in [
+            ('_sympy_',None),
+            ('__float__', Real),
+            ('__int__', Integer),
+            ]:
         meth = getattr(a, methname, None)
         if meth is None:
             continue
@@ -102,10 +106,7 @@ def sympify(a, sympify_lists=False, locals= {}):
     else:
         # XXX this is here because of cyclic-import issues
         from sympy.matrices import Matrix
-        from sympy.polynomials import Polynomial
 
-        if isinstance(a, Polynomial):
-            return a
         if isinstance(a, Matrix):
             raise NotImplementedError('matrix support')
 
@@ -175,7 +176,11 @@ def _sympify(a):
     # let's see if 'a' implements conversion methods such as '_sympy_' or
     # '__int__', that returns a SymPy (by definition) or SymPy compatible
     # expression, so we just use it
-    for methname, conv in [('_sympy_',None), ('__int__', Integer)]:
+    for methname, conv in [
+            ('_sympy_',None),
+            ('__float__', Real),
+            ('__int__', Integer),
+            ]:
         meth = getattr(a, methname, None)
         if meth is None:
             continue
