@@ -1,11 +1,93 @@
 from sympy import Basic, S, Symbol, Real, Integer, Rational,  \
     sin, cos, exp, log, oo, sqrt, symbols, Integral, sympify, \
-    WildFunction
+    WildFunction, Poly, Function, Derivative, Number
 
 import py
 
-x = Symbol("x")
-y = Symbol("y")
+class DummyNumber(object):
+    """
+    Minimal implementation of a number that works with SymPy.
+
+    If one has a Number class (e.g. Sage Integer, or some other custom class)
+    that one wants to work well with SymPy, one has to implement at least the
+    methods of this class DummyNumber, resp. it's subclasses I5 and F1_1.
+
+    Basically, one just needs to implement either __int__() or __float__() and
+    then one needs to make sure that the class works with Python integers and
+    with itself.
+    """
+
+    def __radd__(self, a):
+        if isinstance(a, (int, float)):
+            return a + self.number
+        return NotImplemented
+
+    def __add__(self, a):
+        if isinstance(a, (int, float, DummyNumber)):
+            return self.number + a
+        return NotImplemented
+
+    def __rsub__(self, a):
+        if isinstance(a, (int, float)):
+            return a - self.number
+        return NotImplemented
+
+    def __sub__(self, a):
+        if isinstance(a, (int, float, DummyNumber)):
+            return self.number - a
+        return NotImplemented
+
+    def __rmul__(self, a):
+        if isinstance(a, (int, float)):
+            return a * self.number
+        return NotImplemented
+
+    def __mul__(self, a):
+        if isinstance(a, (int, float, DummyNumber)):
+            return self.number * a
+        return NotImplemented
+
+    def __rdiv__(self, a):
+        if isinstance(a, (int, float)):
+            return a / self.number
+        return NotImplemented
+
+    def __div__(self, a):
+        if isinstance(a, (int, float, DummyNumber)):
+            return self.number / a
+        return NotImplemented
+
+    def __rpow__(self, a):
+        if isinstance(a, (int, float)):
+            return a ** self.number
+        return NotImplemented
+
+    def __pow__(self, a):
+        if isinstance(a, (int, float, DummyNumber)):
+            return self.number ** a
+        return NotImplemented
+
+    def __pos__(self):
+        return self.number
+
+    def __neg__(self):
+        return - self.number
+
+class I5(DummyNumber):
+    number = 5
+    def __int__(self):
+        return self.number
+
+class F1_1(DummyNumber):
+    number = 1.1
+    def __float__(self):
+        return self.number
+
+x,y,z,t = symbols('xyzt')
+
+i5 = I5()
+f1_1 = F1_1()
+
 
 # basic sympy objects
 basic_objs = [
@@ -20,6 +102,8 @@ basic_objs = [
 all_objs = basic_objs + [
     5,
     5.5,
+    i5,
+    f1_1
 ]
 
 def dotest(s):
@@ -56,25 +140,38 @@ def test_basic_nostr():
         for op in ['+','-','*','/','**']:
             py.test.raises(TypeError, "obj %s '1'" % op)
 
-def test_ldegree():
-    assert (1/x**2+1+x+x**2).ldegree(x)==-2
-    assert (1/x+1+x+x**2).ldegree(x)==-1
-    assert (x**2+1/x).ldegree(x)==-1
-    assert (1+x**2).ldegree(x)==0
-    assert (x+1).ldegree(x)==0
-    assert (x+x**2).ldegree(x)==1
-    assert (x**2).ldegree(x)==2
-
 def test_leadterm():
-    assert (3+2*x**(log(3)/log(2)-1)).leadterm(x)==(3,0)
+    assert (3+2*x**(log(3)/log(2)-1)).leadterm(x) == (3,0)
+
+    assert (1/x**2+1+x+x**2).leadterm(x)[1] == -2
+    assert (1/x+1+x+x**2).leadterm(x)[1] == -1
+    assert (x**2+1/x).leadterm(x)[1] == -1
+    assert (1+x**2).leadterm(x)[1] == 0
+    assert (x+1).leadterm(x)[1] == 0
+    assert (x+x**2).leadterm(x)[1] == 1
+    assert (x**2).leadterm(x)[1] == 2
 
 def test_atoms():
-   assert list((1+x).atoms()) == [1,x]
-   assert list(x.atoms()) == [x]
-   assert list((1+2*cos(x)).atoms(type=Symbol)) == [x]
-   assert list((2*(x**(y**x))).atoms()) == [2,x,y]
-   assert list(Rational(1,2).atoms()) == [Rational(1,2)]
-   assert list(Rational(1,2).atoms(type=type(oo))) == []
+   assert sorted(list(x.atoms())) == [x]
+   assert sorted(list((1+x).atoms())) == [1, x]
+
+   assert sorted(list((1+2*cos(x)).atoms(Symbol))) == [x]
+   assert sorted(list((1+2*cos(x)).atoms(Symbol,Number))) == [1, 2, x]
+
+   assert sorted(list((2*(x**(y**x))).atoms())) == [2, x, y]
+
+   assert sorted(list(Rational(1,2).atoms())) == [S.Half]
+   assert sorted(list(Rational(1,2).atoms(Symbol))) == []
+
+   assert sorted(list(sin(oo).atoms(oo))) == [oo]
+
+   assert sorted(list(Poly(0, x).atoms())) == [S.Zero]
+   assert sorted(list(Poly(1, x).atoms())) == [S.One]
+   assert sorted(list(Poly(x, x).atoms())) == [S.One, x]
+   assert sorted(list(Poly(x, x, y).atoms())) == [S.One, x]
+   assert sorted(list(Poly(x + y, x, y).atoms())) == [S.One, x, y]
+   assert sorted(list(Poly(x + y, x, y, z).atoms())) == [S.One, x, y]
+   assert sorted(list(Poly(x + y*t, x, y, z).atoms())) == [S.One, t, x, y]
 
 def test_is_polynomial():
     z = Symbol('z')
@@ -232,6 +329,14 @@ def test_args():
     assert (x**y).args[0] == x
     assert (x**y).args[1] == y
 
+def test_iter_basic_args():
+    assert list(sin(x*y).iter_basic_args()) == [x*y]
+    assert list((x**y).iter_basic_args()) == [x, y]
+
+    assert list(Poly(0, x).iter_basic_args()) == [S.Zero]
+    assert list(Poly(1, x).iter_basic_args()) == [S.One]
+    assert list(Poly(x, x).iter_basic_args()) == [S.One, x]
+
 def test_noncommutative_expand_issue658():
     A, B, C = symbols('ABC', commutative=False)
     assert A*B - B*A != 0
@@ -278,6 +383,25 @@ def test_subs_list():
     assert (x+y)._subs_list([(x, 3), (y, x**2)]) == 3 + x**2
     assert (x+y)._subs_list([(y, x**2), (x, 3)]) == 12
 
+def test_has():
+    x, y = symbols("xy")
+    f = Function("f")
+    g = Function("g")
+    assert sin(x).has(x)
+    assert sin(x).has(sin)
+    assert not sin(x).has(y)
+    assert not sin(x).has(cos)
+    assert f(x).has(x)
+    assert f(x).has(f)
+    assert not f(x).has(y)
+    assert not f(x).has(g)
+
+    assert f(x).diff(x).has(x)
+    assert f(x).diff(x).has(f)
+    assert f(x).diff(x).has(Derivative)
+    assert not f(x).diff(x).has(y)
+    assert not f(x).diff(x).has(g)
+    assert not f(x).diff(x).has(sin)
 
 def test_has_any_symbols():
     x,y,z,t,u = symbols('xyztu')
@@ -302,6 +426,23 @@ def test_has_any_symbols():
     assert expr.has_any_symbols(x, y, z, t) == True
     assert expr.has_any_symbols(x, y, z, t, u)  == True
 
+    from sympy.physics.units import m, s
+
+    assert (x*m/s).has_any_symbols(x) == True
+    assert (x*m/s).has_all_symbols(x) == True
+
+    assert (x*m/s).has_any_symbols(y, z) == False
+    assert (x*m/s).has_all_symbols(x, y) == False
+
+    poly = Poly(x**2 + x*y*sin(z), x, y, t)
+
+    assert poly.has_any_symbols(x) == True
+    assert poly.has_any_symbols(x, y, z) == True
+    assert poly.has_any_symbols(x, y, z, t) == True
+
+    assert poly.has_all_symbols(x, y, z) == True
+    assert poly.has_all_symbols(x, y, z, t) == False
+
 def test_has_all_symbols():
     x,y,z,t,u = symbols('xyztu')
 
@@ -331,9 +472,15 @@ def test_as_poly_basic():
 
     f = x**2 + 2*x*y
 
+    assert f.as_poly().as_basic() == f
     assert f.as_poly(x, y).as_basic() == f
+
     assert (f + sin(x)).as_poly(x, y) is None
 
+    p = Poly(f, x, y)
+
+    assert p.as_poly() == p
+
 def test_nonzero():
     assert bool(S.Zero) == False
     assert bool(S.One)  == True
