@@ -1,4 +1,5 @@
-from sympy.core import *
+from sympy.core import Rational, Symbol, Basic, S, Real, Integer
+from sympy.functions.elementary.miscellaneous import sqrt
 
 def test_rational():
     a = Rational(1, 5)
@@ -20,7 +21,7 @@ def test_negative_real():
     def feq(a,b):
         return abs(a - b) < 1E-10
 
-    assert feq(Basic.One() / Real(-0.5), -Integer(2))
+    assert feq(S.One / Real(-0.5), -Integer(2))
 
 def test_expand():
     x = Symbol('x')
@@ -28,18 +29,20 @@ def test_expand():
 
 def test_issue153():
     #test that is runs:
-    a = Basic.sqrt(2*(1+Basic.sqrt(2)))
+    a = sqrt(2*(1+sqrt(2)))
 
 def test_issue350():
     #test if powers are simplified correctly
     a = Symbol('a')
     assert ((a**Rational(1,3))**Rational(2)) == a**Rational(2,3)
-    assert ((a**Rational(3))**Rational(2,5)) != a**Rational(6,5)
+    assert ((a**Rational(3))**Rational(2,5)) == a**Rational(6,5)
 
     a = Symbol('a', real = True)
     assert (a**Rational(3))**Rational(2,5) == a**Rational(6,5)
-    
+
     #assert Number(5)**Rational(2,3)==Number(25)**Rational(1,3)
 
-    
+def test_issue767():
+    assert --sqrt(sqrt(5)-1)==sqrt(sqrt(5)-1)
+
 test_issue350()
