@@ -1,6 +1,6 @@
 from sympy import Lambda, Symbol, Function, WildFunction, Derivative, sqrt, \
         log, exp, Rational, Real, sign, Basic, sin, cos, diff, I, re, im, \
-        oo, zoo, nan, E, expand
+        oo, zoo, nan, E, expand, pi
 from sympy.utilities.pytest import XFAIL
 from sympy.utilities.test import REPR0
 from sympy.abc import x, y
@@ -215,6 +215,10 @@ def test_Lambda():
     p = x, y, z, t
     assert Lambda(p, t*(x+y+z))(*p) == t * (x + y + z)
 
+    d = Symbol('d', dummy=True)
+
+    assert Lambda(d, d**2).tostr() == "Lambda(_d, _d**2)"
+
 def test_expand_function():
     assert expand(x+y) == x + y
     assert expand(x+y, complex=True) == I*im(x) + I*im(y) + re(x) + re(y)
@@ -249,3 +253,48 @@ def test_deriv1():
     g=Function('g')
     x = Symbol('x')
     assert f(g(x)).diff(x) == Derivative(f(g(x)), g(x)) * Derivative(g(x), x)
+
+def test_deriv2():
+    f=Function('f')
+    g=Function('g')
+    x = Symbol('x')
+
+    assert f(x).diff(x) == Derivative(f(x), x)
+    assert f(2*x).diff(x) == 2*Derivative(f(2*x), 2*x)
+    assert (f(x)**3).diff(x) == 3*f(x)**2*f(x).diff(x)
+    assert (f(2*x)**3).diff(x) == 6*f(2*x)**2*Derivative(f(2*x), 2*x)
+
+    assert f(2+x).diff(x) == Derivative(f(2+x), 2+x)
+    assert f(2+3*x).diff(x) == 3*Derivative(f(2+3*x), 2+3*x)
+    assert f(sin(x)).diff(x) == Derivative(f(sin(x)), sin(x)) * cos(x)
+    assert f(3*sin(x)).diff(x) == 3*Derivative(f(3*sin(x)), 3*sin(x)) * cos(x)
+
+def test_deriv3():
+    f=Function('f')
+    g=Function('g')
+    x = Symbol('x')
+
+    assert (x**3).diff(x) == 3*x**2
+    assert (x**3).diff(x, evaluate=False) != 3*x**2
+    assert (x**3).diff(x, evaluate=False) == Derivative(x**3, x)
+
+    assert diff(x**3, x) == 3*x**2
+    assert diff(x**3, x, evaluate=False) != 3*x**2
+    assert diff(x**3, x, evaluate=False) == Derivative(x**3, x)
+
+def test_suppressed_evaluation():
+    a = sin(0,evaluate=False)
+    assert a != 0
+    assert str(a) == "sin(0)"
+    assert a.func is sin
+    assert a.args == (0,)
+
+def test_function_evalf():
+    def eq(a,b,eps):
+        return abs(a-b) < eps
+    assert eq(sin(1).evalf(15), Real("0.841470984807897"), 1e-13)
+    assert eq(sin(2).evalf(25), Real("0.9092974268256816953960199",25), 1e-23)
+    assert eq(sin(1+I).evalf(15), Real("1.29845758141598") + Real("0.634963914784736")*I, 1e-13)
+    assert eq(exp(1+I).evalf(15), Real("1.46869393991588") + Real("2.28735528717884239")*I, 1e-13)
+    assert eq(log(pi+sqrt(2)*I).evalf(15), Real("1.23699044022052") + Real("0.422985442737893")*I, 1e-13)
+    assert eq(cos(100).evalf(15), Real("0.86231887228768"), 1e-13)
