@@ -3,7 +3,27 @@ from sympy import Rational, Symbol, Real, I, sqrt, oo, nan, pi, Integer, \
 from sympy.core.power import integer_nthroot
 import py
 
-from sympy.core.numbers import ilcm
+from sympy.core.numbers import igcd, ilcm, igcdex
+
+def test_igcd():
+    assert igcd(0, 0) == 0
+    assert igcd(0, 1) == 1
+    assert igcd(1, 0) == 1
+    assert igcd(0, 7) == 7
+    assert igcd(7, 0) == 7
+    assert igcd(7, 1) == 1
+    assert igcd(1, 7) == 1
+    assert igcd(-1, 0) == 1
+    assert igcd(0, -1) == 1
+    assert igcd(-1, -1) == 1
+    assert igcd(-1, 7) == 1
+    assert igcd(7, -1) == 1
+    assert igcd(8, 2) == 2
+    assert igcd(4, 8) == 4
+    assert igcd(8, 16) == 8
+    assert igcd(7, -3) == 1
+    assert igcd(-7, 3) == 1
+    assert igcd(-7, -3) == 1
 
 def test_ilcm():
     assert ilcm(0, 0) == 0
@@ -15,6 +35,11 @@ def test_ilcm():
     assert ilcm(8, 6) == 24
     assert ilcm(8, 7) == 56
 
+def test_igcdex():
+    assert igcdex(2, 3) == (-1, 1, 1)
+    assert igcdex(10, 12) == (-1, 1, 2)
+    assert igcdex(100, 2004) == (-20, 1, 4)
+
 def test_Rational():
     n1 = Rational(1,4)
     n2 = Rational(1,3)
@@ -195,6 +220,10 @@ def test_powers():
 
     assert str((-4)**Rational(1,2)) == str(2*I)
 
+    # Test that this is fast
+    assert integer_nthroot(2,10**10) == (1, False)
+    assert str(2**Rational(1,10**10)) == "2**(1/10000000000)"
+
 def test_abs1():
     assert str(abs(Rational(1,6))) == "1/6"
     assert str(abs(Rational(-1,6))) == "1/6"
@@ -212,7 +241,7 @@ def test_int():
 
 def test_real_bug():
     x = Symbol("x")
-    assert str(2.0*x*x) in ["(2*x)*x","2*x**2"]
+    assert str(2.0*x*x) in ["(2.0*x)*x","2.0*x**2"]
     assert str(2.1*x*x)!="(2.0*x)*x"
 
 def test_bug_sqrt():
