@@ -33,3 +33,10 @@ def test_lt_gt():
     assert (x > 0) == StrictInequality(0, x)
 
     assert (x**2+4*x+1 > 0) == StrictInequality(0, x**2+4*x+1)
+
+def test_no_len():
+    # there should be no len for numbers
+    x = Symbol('x')
+    xxl = Symbol('xxl')
+    py.test.raises(TypeError, "len(x)")
+    py.test.raises(TypeError, "len(xxl)")
