@@ -42,3 +42,12 @@ def test_var():
     py.test.raises(NameError, "z2")
     make_z2()
     assert z2 == Symbol("z2")
+
+def test_var_return():
+    v1 = var('')
+    v2 = var('q')
+    v3 = var('q p')
+
+    assert v1 == None
+    assert v2 == Symbol('q')
+    assert v3 == (Symbol('q'), Symbol('p'))
