@@ -230,6 +230,16 @@ class abs(Function):
     def _eval_conjugate(self):
         return self
 
+    def _eval_power(self,other):
+        if self.args[0].is_real and other.is_integer:
+            if other.is_even:
+                return self.args[0]**other
+        return
+
+    def _sage_(self):
+        import sage.all as sage
+        return sage.abs_symbolic(self.args[0]._sage_())
+
 class arg(Function):
 
     nargs = 1
