@@ -113,7 +113,7 @@ class exp(Function):
         arg = self.args[0]
         if x is not None:
             c,f = arg.as_coeff_factors(x)
-            return self.func(c), tuple( self.func(a) for a in f.args )
+            return self.func(c), tuple( self.func(a) for a in f )
         if arg.is_Add:
             return S.One, tuple( self.func(a) for a in arg.args )
         return S.One,(self,)
@@ -241,9 +241,17 @@ class exp(Function):
             return expr
         return self.func(arg)
 
+    def _eval_rewrite_as_sin(self, arg):
+        I = S.ImaginaryUnit
+        return C.sin(I*arg+S.Pi/2) - I*C.sin(I*arg)
+
+    def _eval_rewrite_as_cos(self, arg):
+        I = S.ImaginaryUnit
+        return C.cos(I*arg) + I*C.cos(I*arg+S.Pi/2)
+
     def _sage_(self):
         import sage.all as sage
-        return sage.exp(self[0]._sage_())
+        return sage.exp(self.args[0]._sage_())
 
 class log(Function):
 
@@ -272,6 +280,8 @@ class log(Function):
 
             if base is not S.Exp1:
                 return cls(arg)/cls(base)
+            else:
+                return cls(arg)
 
         arg = sympify(arg)
 
@@ -321,12 +331,6 @@ class log(Function):
         #why is this here:?
         return exp, S.NegativeOne
 
-    def _calc_apply_positive(self, x):
-        if x.is_positive and x.is_unbounded: return True
-
-    def _calc_apply_unbounded(self, x):
-        return x.is_unbounded
-
     @staticmethod
     @cacheit
     def taylor_term(n, x, *previous_terms): # of log(1+x)
@@ -437,7 +441,7 @@ class log(Function):
 
     def _sage_(self):
         import sage.all as sage
-        return sage.log(self[0]._sage_())
+        return sage.log(self.args[0]._sage_())
 
 # MrvLog is used by limit.py
 class MrvLog(log):
