@@ -97,10 +97,31 @@ class sin(Function):
                         except KeyError:
                             pass
 
-                coeff, terms = arg.as_coeff_terms()
-
-                if coeff.is_negative:
+                if arg.is_Mul and arg.args[0].is_negative:
                     return -cls(-arg)
+                if arg.is_Add:
+                    x, m = arg.as_independent(S.Pi)
+                    if m in [S.Pi/2, S.Pi]:
+                        return sin(m)*cos(x)+cos(m)*sin(x)
+                    # normalize sin(-x-y) to -sin(x+y)
+                    if arg.args[0].is_Mul:
+                        if arg.args[0].args[0].is_negative:
+                            # e.g. arg = -x - y
+                            if (-arg).args[0].is_Mul:
+                                if (-arg).args[0].args[0].is_negative:
+                                    # This is to prevent infinite recursion in
+                                    # the case sin(-x+y), for which
+                                    # -arg = -y + x. See also #838 for the
+                                    # root of the problem here.
+                                    return
+                            # convert sin(-x-y) to -sin(x+y)
+                            return -cls(-arg)
+                    if arg.args[0].is_negative:
+                        if (-arg).args[0].is_negative:
+                            # This is to avoid infinite recursion in the case
+                            # sin(-x-1)
+                            return
+                        return -cls(-arg)
 
 
     @staticmethod
@@ -264,10 +285,31 @@ class cos(Function):
                         except KeyError:
                             pass
 
-                coeff, terms = arg.as_coeff_terms()
-
-                if coeff.is_negative:
+                if arg.is_Mul and arg.args[0].is_negative:
                     return cls(-arg)
+                if arg.is_Add:
+                    x, m = arg.as_independent(S.Pi)
+                    if m in [S.Pi/2, S.Pi]:
+                        return cos(m)*cos(x)-sin(m)*sin(x)
+                    # normalize cos(-x-y) to cos(x+y)
+                    if arg.args[0].is_Mul:
+                        if arg.args[0].args[0].is_negative:
+                            # e.g. arg = -x - y
+                            if (-arg).args[0].is_Mul:
+                                if (-arg).args[0].args[0].is_negative:
+                                    # This is to prevent infinite recursion in
+                                    # the case cos(-x+y), for which
+                                    # -arg = -y + x. See also #838 for the
+                                    # root of the problem here.
+                                    return
+                            # convert cos(-x-y) to cos(x+y)
+                            return cls(-arg)
+                    if arg.args[0].is_negative:
+                        if (-arg).args[0].is_negative:
+                            # This is to avoid infinite recursion in the case
+                            # sin(-x-1)
+                            return
+                        return cls(-arg)
 
 
     @staticmethod
@@ -493,6 +535,10 @@ class tan(Function):
         if arg.is_imaginary:
             return True
 
+    def _sage_(self):
+        import sage.all as sage
+        return sage.tan(self.args[0]._sage_())
+
 class cot(Function):
     """
     Usage
@@ -611,6 +657,10 @@ class cot(Function):
     def _eval_is_real(self):
         return self.args[0].is_real
 
+    def _sage_(self):
+        import sage.all as sage
+        return sage.cot(self.args[0]._sage_())
+
 ###############################################################################
 ########################### TRIGONOMETRIC INVERSES ############################
 ###############################################################################
@@ -708,6 +758,10 @@ class asin(Function):
     def _eval_is_real(self):
         return self.args[0].is_real and (self.args[0]>=-1 and self.args[0]<=1)
 
+    def _sage_(self):
+        import sage.all as sage
+        return sage.asin(self.args[0]._sage_())
+
 class acos(Function):
     """
     Usage
@@ -791,6 +845,10 @@ class acos(Function):
     def _eval_is_real(self):
         return self.args[0].is_real and (self.args[0]>=-1 and self.args[0]<=1)
 
+    def _sage_(self):
+        import sage.all as sage
+        return sage.acos(self.args[0]._sage_())
+
 class atan(Function):
     """
     Usage
@@ -873,6 +931,10 @@ class atan(Function):
     def _eval_is_real(self):
         return self.args[0].is_real
 
+    def _sage_(self):
+        import sage.all as sage
+        return sage.atan(self.args[0]._sage_())
+
 class acot(Function):
     """
     Usage
@@ -953,6 +1015,10 @@ class acot(Function):
     def _eval_is_real(self):
         return self.args[0].is_real
 
+    def _sage_(self):
+        import sage.all as sage
+        return sage.acot(self.args[0]._sage_())
+
 class atan2(Function):
     """
     Returns the atan(y/x) taking two arguments y and x. Signs of
