@@ -1,7 +1,9 @@
 
-from sympy.core import Basic, S
+from sympy.core import Basic, Add, S, C, sympify
 from sympy.core.function import Function
 from zeta_functions import zeta
+from sympy.functions.elementary.exponential import log
+from sympy.functions.elementary.miscellaneous import sqrt
 
 ###############################################################################
 ############################ COMPLETE GAMMA FUNCTION ##########################
@@ -13,25 +15,23 @@ class gamma(Function):
 
     def fdiff(self, argindex=1):
         if argindex == 1:
-            return gamma(self[0])*polygamma(0, self[0])
+            return gamma(self.args[0])*polygamma(0, self.args[0])
         else:
             raise ArgumentIndexError(self, argindex)
 
     @classmethod
     def canonize(cls, arg):
-        arg = Basic.sympify(arg)
-
-        if isinstance(arg, Basic.Number):
-            if isinstance(arg, Basic.NaN):
+        if arg.is_Number:
+            if arg is S.NaN:
                 return S.NaN
-            elif isinstance(arg, Basic.Infinity):
+            elif arg is S.Infinity:
                 return S.Infinity
-            elif isinstance(arg, Basic.Integer):
+            elif arg.is_Integer:
                 if arg.is_positive:
-                    return Basic.Factorial(arg-1)
+                    return C.Factorial(arg-1)
                 else:
                     return S.ComplexInfinity
-            elif isinstance(arg, Basic.Rational):
+            elif arg.is_Rational:
                 if arg.q == 2:
                     n = abs(arg.p) / arg.q
 
@@ -49,32 +49,32 @@ class gamma(Function):
                         coeff *= i
 
                     if arg.is_positive:
-                        return coeff*Basic.sqrt(S.Pi) / 2**n
+                        return coeff*sqrt(S.Pi) / 2**n
                     else:
-                        return 2**n*Basic.sqrt(S.Pi) / coeff
+                        return 2**n*sqrt(S.Pi) / coeff
 
 
     def _eval_expand_func(self, *args):
-        arg = self[0]._eval_expand_basic()
+        arg = self.args[0].expand()
 
-        if isinstance(arg, Basic.Add):
-            for i, coeff in enumerate(arg[:]):
-                if isinstance(arg[i], Basic.Number):
-                    terms = Basic.Add(*(arg[:i] + arg[i+1:]))
+        if arg.is_Add:
+            for i, coeff in enumerate(arg.args[:]):
+                if arg.args[i].is_Number:
+                    terms = C.Add(*(arg.args[:i] + arg.args[i+1:]))
 
-                    if isinstance(coeff, Basic.Rational):
+                    if coeff.is_Rational:
                         if coeff.q != 1:
-                            terms += Basic.Rational(1, coeff.q)
-                            coeff = Basic.Integer(int(coeff))
+                            terms += C.Rational(1, coeff.q)
+                            coeff = C.Integer(int(coeff))
                     else:
                         continue
 
-                    return gamma(terms)*Basic.RisingFactorial(terms, coeff)
+                    return gamma(terms)*C.RisingFactorial(terms, coeff)
 
-        return self.func(*self[:])
+        return self.func(*self.args)
 
     def _eval_is_real(self):
-        return self[0].is_real
+        return self.args[0].is_real
 
 
 ###############################################################################
@@ -88,14 +88,14 @@ class lowergamma(Function):
 
     @classmethod
     def canonize(cls, a, x):
-        if isinstance(a, Basic.Number):
-            if isinstance(a, Basic.One):
-                return S.One - Basic.exp(-x)
-            elif isinstance(a, Basic.Integer):
+        if a.is_Number:
+            if a is S.One:
+                return S.One - C.exp(-x)
+            elif a.is_Integer:
                 b = a - 1
 
                 if b.is_positive:
-                    return b*cls(b, x) - x**b * Basic.exp(-x)
+                    return b*cls(b, x) - x**b * C.exp(-x)
 
 
 class uppergamma(Function):
@@ -106,28 +106,28 @@ class uppergamma(Function):
     def fdiff(self, argindex=2):
         if argindex == 2:
             a, z = self[0:2]
-            return -Basic.exp(-z)*z**(a-1)
+            return -C.exp(-z)*z**(a-1)
         else:
             raise ArgumentIndexError(self, argindex)
 
     @classmethod
     def canonize(cls, a, z):
-        if isinstance(z, Basic.Number):
-            if isinstance(z, Basic.NaN):
+        if z.is_Number:
+            if z is S.NaN:
                 return S.NaN
-            elif isinstance(z, Basic.Infinity):
+            elif z is S.Infinity:
                 return S.Zero
-            elif isinstance(z, Basic.Zero):
+            elif z is S.Zero:
                 return gamma(a)
 
-        if isinstance(a, Basic.Number):
-            if isinstance(a, Basic.One):
-                return Basic.exp(-z)
-            elif isinstance(a, Basic.Integer):
+        if a.is_Number:
+            if a is S.One:
+                return C.exp(-z)
+            elif a.is_Integer:
                 b = a - 1
 
                 if b.is_positive:
-                    return b*cls(b, z) + z**b * Basic.exp(-z)
+                    return b*cls(b, z) + z**b * C.exp(-z)
 
 
 
@@ -141,63 +141,63 @@ class polygamma(Function):
 
     def fdiff(self, argindex=2):
         if argindex == 2:
-            n, z = self[0:2]
+            n, z = self.args[0:2]
             return polygamma(n+1, z)
         else:
             raise ArgumentIndexError(self, argindex)
 
     @classmethod
     def canonize(cls, n, z):
-        n, z = map(Basic.sympify, (n, z))
+        n, z = map(sympify, (n, z))
 
         if n.is_integer:
             if n.is_negative:
                 return loggamma(z)
             else:
-                if isinstance(z, Basic.Number):
-                    if isinstance(z, Basic.NaN):
+                if z.is_Number:
+                    if z is S.NaN:
                         return S.NaN
-                    elif isinstance(z, Basic.Infinity):
-                        if isinstance(n, Basic.Number):
-                            if isinstance(n, Basic.Zero):
+                    elif z is S.Infinity:
+                        if n.is_Number:
+                            if n is S.Zero:
                                 return S.Infinity
                             else:
                                 return S.Zero
-                    elif isinstance(z, Basic.Integer):
+                    elif z.is_Integer:
                         if z.is_nonpositive:
                             return S.ComplexInfinity
                         else:
-                            if isinstance(n, Basic.Zero):
-                                return -S.EulerGamma + Basic.harmonic(z-1, 1)
+                            if n is S.Zero:
+                                return -S.EulerGamma + C.harmonic(z-1, 1)
                             elif n.is_odd:
-                                return (-1)**(n+1)*Basic.Factorial(n)*zeta(n+1, z)
+                                return (-1)**(n+1)*C.Factorial(n)*zeta(n+1, z)
 
 
     def _eval_expand_func(self, *args):
-        n, z = self[0], self[1].expand(func=True)
+        n, z = self.args[0], self.args[1].expand(func=True)
 
-        if isinstance(n, Basic.Integer) and n.is_nonnegative:
-            if isinstance(z, Basic.Add):
+        if n.is_Integer and n.is_nonnegative:
+            if z.is_Add:
                 coeff, factors = z.as_coeff_factors()
 
-                if isinstance(coeff, Basic.Integer):
+                if coeff.is_Integer:
                     tail = Add(*[ z + i for i in xrange(0, int(coeff)) ])
-                    return self(n, z-coeff) + (-1)**n*Basic.Factorial(n)*tail
-            elif isinstance(z, Basic.Mul):
+                    return polygamma(n, z-coeff) + (-1)**n*C.Factorial(n)*tail
+            elif z.is_Mul:
                 coeff, terms = z.as_coeff_terms()
 
-                if isinstance(coeff, Basic.Integer) and coeff.is_positive:
-                    tail = [ self(n, z + i/coeff) for i in xrange(0, int(coeff)) ]
+                if coeff.is_Integer and coeff.is_positive:
+                    tail = [ polygamma(n, z + i/coeff) for i in xrange(0, int(coeff)) ]
 
-                    if isinstance(n, Basic.Zero):
-                        return S.Log(coeff) + Add(*tail)/coeff**(n+1)
+                    if n is S.Zero:
+                        return log(coeff) + Add(*tail)/coeff**(n+1)
                     else:
                         return Add(*tail)/coeff**(n+1)
 
-        return self(n, z)
+        return polygamma(n, z)
 
     def _eval_rewrite_as_zeta(self, n, z):
-        return (-1)**(n+1)*Basic.Factorial(n)*zeta(n+1, z-1)
+        return (-1)**(n+1)*C.Factorial(n)*zeta(n+1, z-1)
 
 class loggamma(Function):
 
