@@ -1,4 +1,4 @@
-from sympy import Symbol, zeta, nan, Rational, pi, dirichlet_eta, log, zoo
+from sympy import Symbol, zeta, nan, Rational, Real, pi, dirichlet_eta, log, zoo
 
 x = Symbol('x')
 
@@ -52,6 +52,9 @@ def test_zeta():
     assert zeta(0, 2) == -Rational(3, 2)
     assert zeta(0, -2) == Rational(3, 2)
 
+    assert str(zeta(3)) == "zeta(3)"
+    assert zeta(3).evalf(20).epsilon_eq(Real("1.2020569031595942854",20), 1e-19)
+
 def test_dirichlet_eta():
 
     assert dirichlet_eta(0) == Rational(1,2)
