@@ -90,7 +90,7 @@ class Ellipse(GeometryEntity):
             return c
 
         hr, vr = self.hradius, self.vradius
-        if hr.atoms(type=C.Symbol) or vr.atoms(type=C.Symbol):
+        if hr.atoms(C.Symbol) or vr.atoms(C.Symbol):
             raise Exception("foci can only be determined on non-symbolic radii")
 
         v = sqrt(abs(vr**2 - hr**2))
@@ -342,7 +342,7 @@ class Circle(Ellipse):
 
     def intersection(self, o):
         if isinstance(o, Circle):
-            dx,dy = o._c - self.center
+            dx,dy = o.center - self.center
             d = sqrt( simplify(dy**2 + dx**2) )
             a = simplify((self.radius**2 - o.radius**2 + d**2) / (2*d))
 
