@@ -2,8 +2,9 @@ from sympy import Basic, Symbol
 from sympy.core import sympify
 
 from sympy.core.basic import S, C
+from sympy.polys import Poly, roots
+from sympy.simplify import simplify
 
-import sympy.polynomials
 import random
 
 class NonSquareMatrixException(Exception):
@@ -24,7 +25,7 @@ class _MatrixAsBasic(Basic):
     proxy.
 
     see #420
-"""
+    """
     def __init__(self, m):
         self.m = m
     def torepr(self):
@@ -34,6 +35,10 @@ class _MatrixAsBasic(Basic):
 
 class Matrix(object):
 
+    # Added just for numpy compatibility
+    # TODO: investigate about __array_priority__
+    __array_priority__ = 10.0
+
     def __init__(self, *args):
         """
         Matrix can be constructed with values or a rule.
@@ -62,19 +67,23 @@ class Matrix(object):
             self.lines=args[0]
             self.cols=args[1]
             mat = args[2]
-            self.mat=[]
-            for j in range(self.lines):
-                for i in range(self.cols):
-                    self.mat.append(sympify(mat[j*self.cols+i]))
+            if len(mat) != self.lines*self.cols:
+                raise MatrixError('List length should be equal to rows*columns')
+            self.mat = map(lambda i: sympify(i), mat)
         else:
             if len(args) == 1:
                 mat = args[0]
             else:
                 mat = args
-            if hasattr(mat, "__array__"):
+            if isinstance(mat, Matrix):
+                self.lines = mat.lines
+                self.cols = mat.cols
+                self.mat = mat[:]
+                return
+            elif hasattr(mat, "__array__"):
                 # NumPy array or matrix or some other object that implements
-                # __array__ (like SymPy Matrix). So let's first use this method
-                # to get a numpy.array() and then make a python list out of it.
+                # __array__. So let's first use this method to get a
+                # numpy.array() and then make a python list out of it.
                 mat = list(mat.__array__())
             elif not isinstance(mat[0], (list, tuple)):
                 # make each element a singleton
@@ -98,41 +107,59 @@ class Matrix(object):
             raise IndexError("Index out of range: a[%s]"%repr(key))
         return i,j
 
-    def __getattr__(self,name):
+    def transpose(self):
         """
+        Matrix transposition.
+
         >>> from sympy import *
         >>> m=Matrix(((1,2+I),(3,4)))
         >>> m  #doctest: +NORMALIZE_WHITESPACE
         [1, 2 + I]
         [3,     4]
-        >>> m.T #doctest: +NORMALIZE_WHITESPACE
+        >>> m.transpose() #doctest: +NORMALIZE_WHITESPACE
         [    1, 3]
         [2 + I, 4]
+        >>> m.T == m.transpose()
+        True
+        """
+        a = [0]*self.cols*self.lines
+        for i in xrange(self.cols):
+            a[i*self.lines:(i+1)*self.lines] = self.mat[i::self.cols]
+        return Matrix(self.cols,self.lines,a)
+
+    T = property(transpose,None,None,"Matrix transposition.")
+
+    def conjugate(self):
+        """By-element conjugation."""
+        out = Matrix(self.lines,self.cols,
+                lambda i,j: self[i,j].conjugate())
+        return out
+
+    C = property(conjugate,None,None,"By-element conjugation.")
+
+    @property
+    def H(self):
+        """
+        Hermite conjugation.
+
+        >>> from sympy import *
+        >>> m=Matrix(((1,2+I),(3,4)))
+        >>> m  #doctest: +NORMALIZE_WHITESPACE
+        [1, 2 + I]
+        [3,     4]
         >>> m.H #doctest: +NORMALIZE_WHITESPACE
         [    1, 3]
         [2 - I, 4]
-
         """
-        if name == "T":
-            #transposition
-            out = Matrix(self.cols,self.lines, lambda i,j: self[j,i])
-            return out
-        if name == "C":
-            #by-element conjugation
-            out = Matrix(self.lines,self.cols,
-                    lambda i,j: self[i,j].conjugate())
-            return out
-        if name == "H":
-            #hermite conjugation
-            out = self.T.C
-            return out
-        if name == "D":
-            #dirac conjugation
-            from sympy.physics.matrices import mgamma
-            out = self.H * mgamma(0)
-            return out
-        raise AttributeError("'%s' object has no attribute '%s'"%
-                (self.__class__.__name__, name))
+        out = self.T.C
+        return out
+
+    @property
+    def D(self):
+        """Dirac conjugation."""
+        from sympy.physics.matrices import mgamma
+        out = self.H * mgamma(0)
+        return out
 
     def __getitem__(self,key):
         """
@@ -147,18 +174,25 @@ class Matrix(object):
         2 - I
 
         """
-        # row-wise decomposition of matrix
-        if isinstance(key, slice) or isinstance(key, int):
-            return self.mat[key]
-        # proper 2-index access
-        assert len(key) == 2
-        if isinstance(key[0], int) and isinstance(key[1], int):
-            i,j=self.key2ij(key)
-            return self.mat[i*self.cols+j]
-        elif isinstance(key[0], slice) or isinstance(key[1], slice):
-            return self.submatrix(key)
+        if isinstance(key, (list, tuple)):
+            if len(key) == 2:
+                # proper 2-index access
+                if isinstance(key[0], slice) or isinstance(key[1], slice):
+                    return self.submatrix(key)
+                else:
+                    k1, k2 = a2idx(key[0]), a2idx(key[1])
+                    if k1 is not None and k2 is not None:
+                        i,j=self.key2ij([k1, k2])
+                        return self.mat[i*self.cols+j]
         else:
-            raise IndexError("Index out of range: a[%s]"%repr(key))
+            # row-wise decomposition of matrix
+            if isinstance(key, slice):
+                return self.mat[key]
+            else:
+                k = a2idx(key)
+                if k is not None:
+                    return self.mat[k]
+        raise IndexError("Invalid index: a[%s]"%repr(key))
 
     def __setitem__(self,key,value):
         """
@@ -173,19 +207,53 @@ class Matrix(object):
         [9,     4]
 
         """
-        assert len(key) == 2
-        if isinstance(key[0], slice) or isinstance(key[1], slice):
-            if isinstance(value, Matrix):
-                self.copyin_matrix(key, value)
-            if isinstance(value, (list, tuple)):
-                self.copyin_list(key, value)
+        if isinstance(key, (list, tuple)):
+            if len(key) == 2:
+                # proper 2-index access
+                if isinstance(key[0], slice) or isinstance(key[1], slice):
+                    if isinstance(value, Matrix):
+                        self.copyin_matrix(key, value)
+                        return
+                    if isinstance(value, (list, tuple)):
+                        self.copyin_list(key, value)
+                        return
+                else:
+                    k1, k2 = a2idx(key[0]), a2idx(key[1])
+                    if k1 is not None and k2 is not None:
+                        i,j=self.key2ij([k1, k2])
+                        self.mat[i*self.cols + j] = sympify(value)
+                        return
         else:
-            i,j=self.key2ij(key)
-            self.mat[i*self.cols + j] = sympify(value)
+            # row-wise decomposition of matrix
+            if isinstance(key, slice):
+                raise IndexError("Vector slices not implemented yet.")
+            else:
+                k = a2idx(key)
+                if k is not None:
+                    self.mat[k] = sympify(value)
+                    return
+        raise IndexError("Invalid index: a[%s]"%repr(key))
 
     def __array__(self):
         return matrix2numpy(self)
 
+    def tolist(self):
+        """
+        Return the Matrix converted in a python list.
+
+        >>> from sympy import *
+        >>> m=Matrix(3, 3, range(9))
+        >>> m
+        [0, 1, 2]
+        [3, 4, 5]
+        [6, 7, 8]
+        >>> m.tolist()
+        [[0, 1, 2], [3, 4, 5], [6, 7, 8]]
+        """
+        ret = [0]*self.lines
+        for i in xrange(self.lines):
+            ret[i] = self.mat[i*self.cols:(i+1)*self.cols]
+        return ret
 
     def copyin_matrix(self, key, value):
         rlo, rhi = self.slice2bounds(key[0], self.lines)
@@ -209,36 +277,30 @@ class Matrix(object):
         return (self.lines, self.cols)
 
     def __rmul__(self,a):
-        assert not isinstance(a,Matrix)
-        r=self.zeronm(self.lines,self.cols)
-        for i in range(self.lines):
-            for j in range(self.cols):
-                r[i,j]=a*self[i,j]
-        return r
+        if hasattr(a, "__array__"):
+            return matrix_multiply(a,self)
+        out = Matrix(self.lines,self.cols,map(lambda i: a*i,self.mat))
+        return out
 
     def expand(self):
-        out = self[:,:]
-        out[:,:] = Matrix(self.lines, self.cols, lambda i,j: self[i,j].expand())
+        out = Matrix(self.lines,self.cols,map(lambda i: i.expand(), self.mat))
         return out
 
     def combine(self):
-        out = self[:,:]
-        out[:,:] = Matrix(self.lines, self.cols, lambda i,j: self[i,j].combine())
+        out = Matrix(self.lines,self.cols,map(lambda i: i.combine(),self.mat))
         return out
 
-    def subs(self,a,b):
-        out = self[:,:]
-        out[:,:] = Matrix(self.lines, self.cols, lambda i,j: self[i,j].subs(a,b))
+    def subs(self, *args):
+        out = Matrix(self.lines,self.cols,map(lambda i: i.subs(*args),self.mat))
         return out
 
     def __sub__(self,a):
         return self + (-a)
 
     def __mul__(self,a):
-        if isinstance(a,Matrix):
-            return self.multiply(a)
-        out = self[:,:]
-        out[:,:] = Matrix(self.lines, self.cols, lambda i,j: self[i,j]*a)
+        if hasattr(a, "__array__"):
+            return matrix_multiply(self,a)
+        out = Matrix(self.lines,self.cols,map(lambda i: i*a,self.mat))
         return out
 
     def __pow__(self, num):
@@ -259,46 +321,43 @@ class Matrix(object):
         raise NotImplementedError('Can only rise to the power of an integer for now')
 
     def __add__(self,a):
-        return self.add(a)
+        return matrix_add(self,a)
+
+    def __radd__(self,a):
+        return matrix_add(a,self)
 
     def __div__(self,a):
         return self * (S.One/a)
 
+    def __truediv__(self,a):
+        return self.__div__(a)
+
     def multiply(self,b):
         """Returns self*b """
-
-        def dotprod(a,b,i,j):
-            if a.cols != b.lines:
-                raise ShapeError()
-
-            r=0
-            for x in range(a.cols):
-                r+=a[i,x]*b[x,j]
-            return r.expand() # .expand() is a test
-
-        return Matrix(self.lines,b.cols, lambda i,j: dotprod(self,b,i,j))
+        return matrix_multiply(self,b)
 
     def add(self,b):
-        """Returns self+b """
-
-        assert self.lines == b.lines
-        assert self.cols == b.cols
-        out = self[:,:]
-        out[:,:] = Matrix(self.lines, self.cols, lambda i,j: self[i,j]+b[i,j])
-        return out
+        """Return self+b """
+        return matrix_add(self,b)
 
     def __neg__(self):
         return -1*self
 
-    def __eq__(self,a):
+    def __eq__(self, a):
         if not isinstance(a, (Matrix, Basic)):
             a = sympify(a)
-        return self.hash() == a.hash()
+        if isinstance(a, Matrix):
+            return self.hash() == a.hash()
+        else:
+            return False
 
     def __ne__(self,a):
         if not isinstance(a, (Matrix, Basic)):
             a = sympify(a)
-        return self.hash() != a.hash()
+        if isinstance(a, Matrix):
+            return self.hash() != a.hash()
+        else:
+            return True
 
     # hook Basic.__repr__ & Basic.__str__
     def __repr__(self):
@@ -335,14 +394,25 @@ class Matrix(object):
         return self._format_str(lambda elem: str(elem))
 
     def inv(self, method="GE"):
+        """
+        Calculates the matrix inverse.
+
+        According to the "method" parameter, it calls the appropriate method:
+
+          GE .... inverse_GE()
+          LU .... inverse_LU()
+          ADJ ... inverse_ADJ()
+
+        """
         assert self.cols==self.lines
-        # gaussian by default, also can be done by LU
         if method == "GE":
             return self.inverse_GE()
         elif method == "LU":
             return self.inverse_LU()
+        elif method == "ADJ":
+            return self.inverse_ADJ()
         else:
-            raise "Inversion method unrecognized"
+            raise Exception("Inversion method unrecognized")
 
 
     def __mathml__(self):
@@ -391,7 +461,17 @@ class Matrix(object):
         self.cols -= 1
 
     def row_join(self, rhs):
-        # concatenates two matrices along self's last and rhs's first col
+        """
+        Concatenates two matrices along self's last and rhs's first column
+
+        >>> from sympy import *
+        >>> M = Matrix(3,3,lambda i,j: i+j)
+        >>> V = Matrix(3,1,lambda i,j: 3+i+j)
+        >>> M.row_join(V)
+        [0, 1, 2, 3]
+        [1, 2, 3, 4]
+        [2, 3, 4, 5]
+        """
         assert self.lines == rhs.lines
         newmat = self.zeronm(self.lines, self.cols + rhs.cols)
         newmat[:,:self.cols] = self[:,:]
@@ -399,12 +479,77 @@ class Matrix(object):
         return newmat
 
     def col_join(self, bott):
+        """
+        Concatenates two matrices along self's last and bott's first row
+
+        >>> from sympy import *
+        >>> M = Matrix(3,3,lambda i,j: i+j)
+        >>> V = Matrix(1,3,lambda i,j: 3+i+j)
+        >>> M.col_join(V)
+        [0, 1, 2]
+        [1, 2, 3]
+        [2, 3, 4]
+        [3, 4, 5]
+        """
         assert self.cols == bott.cols
         newmat = self.zeronm(self.lines+bott.lines, self.cols)
         newmat[:self.lines,:] = self[:,:]
         newmat[self.lines:,:] = bott
         return newmat
 
+    def row_insert(self, pos, mti):
+        """
+        >>> from sympy import *
+        >>> M = Matrix(3,3,lambda i,j: i+j)
+        >>> M
+        [0, 1, 2]
+        [1, 2, 3]
+        [2, 3, 4]
+        >>> V = zeronm(1,3)
+        >>> V
+        [0, 0, 0]
+        >>> M.row_insert(1,V)
+        [0, 1, 2]
+        [0, 0, 0]
+        [1, 2, 3]
+        [2, 3, 4]
+        """
+        if pos is 0:
+            return mti.col_join(self)
+        assert self.cols == mti.cols
+        newmat = self.zeronm(self.lines + mti.lines, self.cols)
+        newmat[:pos,:] = self[:pos,:]
+        newmat[pos:pos+mti.lines,:] = mti[:,:]
+        newmat[pos+mti.lines:,:] = self[pos:,:]
+        return newmat
+
+    def col_insert(self, pos, mti):
+        """
+        >>> from sympy import *
+        >>> M = Matrix(3,3,lambda i,j: i+j)
+        >>> M
+        [0, 1, 2]
+        [1, 2, 3]
+        [2, 3, 4]
+        >>> V = zeronm(3,1)
+        >>> V
+        [0]
+        [0]
+        [0]
+        >>> M.col_insert(1,V)
+        [0, 0, 1, 2]
+        [1, 0, 2, 3]
+        [2, 0, 3, 4]
+        """
+        if pos is 0:
+            return mti.row_join(self)
+        assert self.lines == mti.lines
+        newmat = self.zeronm(self.lines, self.cols + mti.cols)
+        newmat[:,:pos] = self[:,:pos]
+        newmat[:,pos:pos+mti.cols] = mti[:,:]
+        newmat[:,pos+mti.cols:] = self[:,pos:]
+        return newmat
+
     def trace(self):
         assert self.cols == self.lines
         trace = 0
@@ -435,7 +580,11 @@ class Matrix(object):
         clo, chi = self.slice2bounds(keys[1], self.cols)
         if not ( 0<=rlo<=rhi and 0<=clo<=chi ):
             raise IndexError("Slice indices out of range: a[%s]"%repr(keys))
-        return Matrix(rhi-rlo, chi-clo, lambda i,j: self[i+rlo, j+clo])
+        outLines, outCols = rhi-rlo, chi-clo
+        outMat = [0]*outLines*outCols
+        for i in xrange(outLines):
+            outMat[i*outCols:(i+1)*outCols] = self.mat[(i+rlo)*self.cols+clo:(i+rlo)*self.cols+chi]
+        return Matrix(outLines,outCols,outMat)
 
     def slice2bounds(self, key, defmax):
         """
@@ -475,10 +624,11 @@ class Matrix(object):
         [4, 6]
         """
         assert callable(f)
-        out = self[:,:]
-        for i in range(self.lines):
-            for j in range(self.cols):
-                out[i,j] = f(self[i,j])
+        out = Matrix(self.lines,self.cols,map(f,self.mat))
+        return out
+
+    def evalf(self):
+        out = self.applyfunc(Basic.evalf)
         return out
 
     def reshape(self, _rows, _cols):
@@ -529,6 +679,10 @@ class Matrix(object):
         print s
 
     def LUsolve(self, rhs):
+        """
+        Solve the linear system Ax = b.
+        self is the coefficient matrix A and rhs is the right side b.
+        """
         assert rhs.lines == self.lines
         A, perm = self.LUdecomposition_Simple()
         n = self.lines
@@ -545,6 +699,9 @@ class Matrix(object):
         return b
 
     def LUdecomposition(self):
+        """
+        Returns the decompositon LU and the row swaps p.
+        """
         combined, p = self.LUdecomposition_Simple()
         L = self.zero(self.lines)
         U = self.zero(self.lines)
@@ -559,8 +716,10 @@ class Matrix(object):
         return L, U, p
 
     def LUdecomposition_Simple(self):
-        # returns A compused of L,U (L's diag entries are 1) and
-        # p which is the list of the row swaps (in order)
+        """
+        Returns A compused of L,U (L's diag entries are 1) and
+        p which is the list of the row swaps (in order).
+        """
         assert self.lines == self.cols
         n = self.lines
         A = self[:,:]
@@ -590,8 +749,11 @@ class Matrix(object):
 
 
     def LUdecompositionFF(self):
-        # returns 4 matrices P, L, D, U such that PA = L D**-1 U
-        # from the paper "fraction-free matrix factors..." by Zhou and Jeffrey
+        """
+        Returns 4 matrices P, L, D, U such that PA = L D**-1 U.
+
+        From the paper "fraction-free matrix factors..." by Zhou and Jeffrey
+        """
         n, m = self.lines, self.cols
         U, L, P = self[:,:], eye(n), eye(n)
         DD = zero(n) # store it smarter since it's just diagonal
@@ -630,28 +792,32 @@ class Matrix(object):
         DD[n-1,n-1] = oldpivot
         return P, L, DD, U
 
-    def cofactorMatrix(self):
-        out = self[:,:]
-        out[:,:] = Matrix(self.lines, self.cols, lambda i,j: self.cofactor(i,j))
+    def cofactorMatrix(self, method="berkowitz"):
+        out = Matrix(self.lines, self.cols, lambda i,j:
+                self.cofactor(i, j, method))
         return out
 
-    def minorEntry(self, i, j):
+    def minorEntry(self, i, j, method="berkowitz"):
         assert 0 <= i < self.lines and 0 <= j < self.cols
-        return self.minorMatrix(i,j).det()
+        return self.minorMatrix(i,j).det(method)
 
     def minorMatrix(self, i, j):
         assert 0 <= i < self.lines and 0 <= j < self.cols
         return self.delRowCol(i,j)
 
-    def cofactor(self, i, j):
+    def cofactor(self, i, j, method="berkowitz"):
         if (i+j) % 2 == 0:
-            return self.minorEntry(i,j)
+            return self.minorEntry(i, j, method)
         else:
-            return -1 * self.minorEntry(i,j)
+            return -1 * self.minorEntry(i, j, method)
 
     def jacobian(self, varlist):
-        # self is a vector of expression representing functions f_i(x_1, ..., x_n)
-        # varlist is the set of x_i's in order
+        """
+        Calculates the Jacobian matrix (derivative of a vectorial function).
+
+        self is a vector of expression representing functions f_i(x_1, ...,
+        x_n).  varlist is the set of x_i's in order.
+        """
         assert self.lines == 1
         m = self.cols
         if isinstance(varlist, Matrix):
@@ -674,9 +840,11 @@ class Matrix(object):
         return J
 
     def QRdecomposition(self):
-        # TODO: still doesn't work for large expressions, there's a bug in an eval somewhere
-        # return Q*R where Q is orthogonal and R is upper triangular
-        # assume full-rank square, for now
+        """
+        Return Q*R where Q is orthogonal and R is upper triangular.
+
+        Assumes full-rank square (for now).
+        """
         assert self.lines == self.cols
         n = self.lines
         Q, R = self.zero(n), self.zero(n)
@@ -694,19 +862,12 @@ class Matrix(object):
                 R[i,j] = Q[:,i].dot(self[:,j])
         return Q,R
 
+    # TODO: QRsolve
+
     # Utility functions
     def simplify(self):
-        for i in range(self.lines):
-            for j in range(self.cols):
-                try:
-                    self[i,j] = self[i,j].simplify()
-                except:
-                    pass
-
-    def expand(self):
-        for i in range(self.lines):
-            for j in range(self.cols):
-                self[i,j] = self[i,j].expand()
+        for i in xrange(len(self.mat)):
+            self.mat[i] = simplify(self.mat[i])
 
     #def evaluate(self):    # no more eval() so should be removed
     #    for i in range(self.lines):
@@ -747,11 +908,11 @@ class Matrix(object):
     def normalized(self):
         assert self.lines == 1 or self.cols == 1
         norm = self.norm()
-        out = self[:,:].applyfunc(lambda i: i / norm)
+        out = self.applyfunc(lambda i: i / norm)
         return out
 
     def project(self, v):
-        # project ONTO v
+        """Project onto v."""
         return v * (self.dot(v) / v.dot(v))
 
     def permuteBkwd(self, perm):
@@ -767,7 +928,7 @@ class Matrix(object):
         return copy
 
     def delRowCol(self, i, j):
-        #used only for cofactors, makes a copy
+        # used only for cofactors, makes a copy
         M = self[:,:]
         M.row_del(i)
         M.col_del(j)
@@ -776,15 +937,17 @@ class Matrix(object):
     def zeronm(self, n, m):
         # used so that certain functions above can use this
         # then only this func need be overloaded in subclasses
-        return Matrix(n,m,lambda i,j:0)
+        return Matrix(n,m,[S.Zero]*n*m)
 
     def zero(self, n):
-        return Matrix(n,n,lambda i,j:0)
+        """Returns a n x n matrix of zeros."""
+        return Matrix(n,n,[S.Zero]*n*n)
 
     def eye(self, n):
+        """Returns the identity matrix of size n."""
         tmp = self.zero(n)
         for i in range(tmp.lines):
-            tmp[i,i] = 1
+            tmp[i,i] = S.One
         return tmp
 
     @property
@@ -808,14 +971,30 @@ class Matrix(object):
     def is_symbolic(self):
         for i in range(self.cols):
             for j in range(self.lines):
-                if not self[i,j].atoms(type=Symbol):
+                if self[i,j].atoms(Symbol):
                     return True
         return False
 
     def clone(self):
         return Matrix(self.lines, self.cols, lambda i, j: self[i, j])
 
-    def det(self):
+    def det(self, method="bareis"):
+        """
+        Computes the matrix determinant using the method "method".
+
+        Possible values for "method":
+          bareis ... det_bareis
+          berkowitz ... berkowitz_det
+        """
+
+        if method == "bareis":
+            return self.det_bareis()
+        elif method == "berkowitz":
+            return self.berkowitz_det()
+        else:
+            raise Exception("Determinant method unrecognized")
+
+    def det_bareis(self):
         """Compute matrix determinant using Bareis' fraction-free
            algorithm which is an extension of the well known Gaussian
            elimination method. This approach is best suited for dense
@@ -825,8 +1004,6 @@ class Matrix(object):
 
            TODO: Implement algorithm for sparse matrices (SFF).
         """
-        from sympy.simplify import cancel
-
         if not self.is_square:
             raise NonSquareMatrixException()
 
@@ -863,40 +1040,77 @@ class Matrix(object):
                         if D.is_Atom:
                             M[i, j] = D
                         else:
-                            M[i, j] = cancel(D)
+                            M[i, j] = Poly.cancel(D)
 
             det = sign * M[n-1, n-1]
 
         return det.expand()
 
+    def adjugate(self, method="berkowitz"):
+        """
+        Returns the adjugate matrix.
+
+        Adjugate matrix is the transpose of the cofactor matrix.
+
+        http://en.wikipedia.org/wiki/Adjugate
+
+        See also: .cofactorMatrix(), .T
+        """
+
+        return self.cofactorMatrix(method).T
+
+
     def inverse_LU(self):
+        """
+        Calculates the inverse using LU decomposition.
+        """
         return self.LUsolve(self.eye(self.lines))
 
     def inverse_GE(self):
+        """
+        Calculates the inverse using Gaussian elimination.
+        """
         assert self.lines == self.cols
         assert self.det() != 0
         big = self.row_join(self.eye(self.lines))
         red = big.rref()
         return red[0][:,big.lines:]
 
-    def rref(self):
-        # take any matrix and return reduced row-ech form and indices of pivot vars
+    def inverse_ADJ(self):
+        """
+        Calculates the inverse using the adjugate matrix and a determinant.
+        """
+        assert self.lines == self.cols
+        d = self.berkowitz_det()
+        assert d != 0
+        return self.adjugate()/d
+
+    def rref(self,simplified=False):
+        """
+        Take any matrix and return reduced row-echelon form and indices of pivot vars
+
+        To simplify elements before finding nonzero pivots set simplified=True
+        """
         # TODO: rewrite inverse_GE to use this
         pivots, r = 0, self[:,:]        # pivot: index of next row to contain a pivot
         pivotlist = []                  # indices of pivot variables (non-free)
         for i in range(r.cols):
             if pivots == r.lines:
                 break
+            if simplified:
+                r[pivots,i] = simplify(r[pivots,i])
             if r[pivots,i] == 0:
                 for k in range(pivots, r.lines):
+                    if simplified and k>pivots:
+                        r[k,i] = simplify(r[k,i])
                     if r[k,i] != 0:
                         break
-                if k == r.lines - 1:
+                if k == r.lines - 1 and r[k,i] == 0:
                     continue
                 r.row_swap(pivots,k)
             scale = r[pivots,i]
             r.row(pivots, lambda x, _: x/scale)
-            for j in range(0, r.lines):
+            for j in range(r.lines):
                 if j == pivots:
                     continue
                 scale = r[j,i]
@@ -905,10 +1119,12 @@ class Matrix(object):
             pivots += 1
         return r, pivotlist
 
-    def nullspace(self):
-        # Returns list of vectors (Matrix objects) that span nullspace of self
+    def nullspace(self,simplified=False):
+        """
+        Returns list of vectors (Matrix objects) that span nullspace of self
+        """
         assert self.cols >= self.lines
-        reduced, pivots = self.rref()
+        reduced, pivots = self.rref(simplified)
         basis = []
         # create a set of vectors for the basis
         for i in range(self.cols - len(pivots)):
@@ -930,17 +1146,6 @@ class Matrix(object):
                         basis[basiskey.index(j)][i,0] = -1 * reduced[line, j]
         return basis
 
-    def charpoly(self, var):
-        assert self.lines == self.cols
-        if isinstance(var, Symbol):
-            x = var
-        else:
-            raise "Input variable to charpoly not valid"
-        copy = self[:,:]
-        for i in range(self.lines):
-            copy[i,i] -= x
-        return copy.det()
-
     def berkowitz(self):
         """The Berkowitz algorithm.
 
@@ -1044,49 +1249,62 @@ class Matrix(object):
     def berkowitz_charpoly(self, x):
         """Computes characteristic polynomial minors using Berkowitz method."""
         coeffs, monoms = self.berkowitz()[-1], range(self.lines+1)
-        return C.Poly(list(zip(coeffs, reversed(monoms))), x)
+        return Poly(list(zip(coeffs, reversed(monoms))), x)
 
-    def eigenvals(self, var=None):
-        """ Calls polynomials's roots(), doesn't support coeff type right now """
-        # returns list of pairs (eigenval, multiplicty)
-        if var == None:
-            var = Symbol('x')
-        p = self.charpoly(var)
-        num, den = p.as_numer_denom()
-        if den != 1:
-            divop = sympy.polynomials.div(num, den)
-            assert divop[1] == 0
-            p = divop[0]
-        rl = sympy.polynomials.roots(p, var)
-        assert len(rl) == self.lines
-        outlist = []
-        def f(num):
-            def g(n):
-                if num == n:
-                    return True
-                else:
-                    return False
-            return g
-        while rl != []:
-            n = len(filter(f(rl[0]), rl))
-            outlist.append([rl[0], n])
-            for i in range(n):
-                rl.remove(rl[0])
-        return outlist
+    charpoly = berkowitz_charpoly
 
-    def eigenvects(self):
-        # return list of triples (eigenval, multiplicty, basis)
-        out, vlist = [], self.eigenvals()
-        for i in range(len(vlist)):
-            tmp = self - eye(self.lines)*vlist[i][0]
+    def berkowitz_eigenvals(self, **flags):
+        """Computes eigenvalues of a Matrix using Berkowitz method. """
+        return roots(self.berkowitz_charpoly(Symbol('x', dummy=True)), **flags)
+
+    eigenvals = berkowitz_eigenvals
+
+    def eigenvects(self, **flags):
+        """Return list of triples (eigenval, multiplicty, basis)."""
+
+        if flags.has_key('multiple'):
+            del flags['multiple']
+
+        out, vlist = [], self.eigenvals(**flags)
+
+        for r, k in vlist.iteritems():
+            tmp = self - eye(self.lines)*r
             basis = tmp.nullspace()
             # check if basis is right size, don't do it if symbolic - too many solutions
             if not tmp.is_symbolic():
-                assert len(basis) == vlist[i][1]
-            vlist[i].append(basis)
-            out.append(vlist[i])
+                assert len(basis) == k
+            elif len(basis) != k:
+                # The nullspace routine failed, try it again with simplification
+                basis = tmp.nullspace(simplified=True)
+            out.append((r, k, basis))
         return out
 
+def matrix_multiply(A,B):
+    """
+    Return  A*B.
+    """
+    if A.shape[1] != B.shape[0]:
+        raise ShapeError()
+    blst = B.T.tolist()
+    alst = A.tolist()
+    return Matrix(A.shape[0], B.shape[1], lambda i,j:
+                                        reduce(lambda k,l: k+l,
+                                        map(lambda n,m: n*m,
+                                        alst[i],
+                                        blst[j])).expand())
+                                        # .expand() is a test
+
+def matrix_add(A,B):
+    """Return A+B"""
+    if A.shape != B.shape:
+        raise ShapeError()
+    alst = A.tolist()
+    blst = B.tolist()
+    ret = [0]*A.shape[0]
+    for i in xrange(A.shape[0]):
+        ret[i] = map(lambda j,k: j+k, alst[i], blst[i])
+    return Matrix(ret)
+
 def zero(n):
     """Create square zero matrix n x n"""
     return zeronm(n,n)
@@ -1095,7 +1313,7 @@ def zeronm(n,m):
     """Create zero matrix n x m"""
     assert n>0
     assert m>0
-    return Matrix(n,m, lambda i,j: 0)
+    return Matrix(n,m,[S.Zero]*m*n)
 
 def one(n):
     """Create square all-one matrix n x n"""
@@ -1109,7 +1327,7 @@ def eye(n):
     assert n>0
     out = zeronm(n,n)
     for i in range(n):
-        out[i,i]=1
+        out[i,i]=S.One
     return out
 
 def randMatrix(r,c,min=0,max=99,seed=[]):
@@ -1441,3 +1659,15 @@ def matrix2numpy(m):
         for j in range(m.cols):
             a[i, j] = m[i, j]
     return a
+
+def a2idx(a):
+    """
+    Tries to convert "a" to an index, returns None on failure.
+
+    The result of a2idx() (if not None) can be safely used as an index to
+    arrays/matrices.
+    """
+    if hasattr(a, "__int__"):
+        return int(a)
+    if hasattr(a, "__index__"):
+        return a.__index__()
