@@ -1,9 +1,21 @@
 from sympy import symbols, Matrix, eye, I, Symbol, Rational, wronskian, cos, \
-        sin, exp, hessian, sqrt, zero, randMatrix
-from sympy.matrices.matrices import ShapeError
+        sin, exp, hessian, sqrt, zero, randMatrix, Poly, S, pi
+from sympy.matrices.matrices import ShapeError, MatrixError
 from sympy.utilities.test import REPR0
 import py
 
+def test_division():
+    x, y, z = symbols('x','y','z')
+    v = Matrix(1,2,[x, y])
+    assert v.__div__(z) == Matrix(1,2,[x/z, y/z])
+    assert v.__truediv__(z) == Matrix(1,2,[x/z, y/z])
+    assert v/z == Matrix(1,2,[x/z, y/z])
+
+def test_sum():
+    x, y, z = symbols('xyz')
+    m = Matrix([[1,2,3],[x,y,x],[2*y,-50,z*x]])
+    assert m+m == Matrix([[2,4,6],[2*x,2*y,2*x],[4*y,-100,2*z*x]])
+
 def test_multiplication():
     a=Matrix((
         (1, 2),
@@ -54,6 +66,8 @@ def test_power():
     assert eye(2)**10000000 == eye(2)
 
 def test_creation():
+    py.test.raises(MatrixError, 'Matrix(5,5,range(20))')
+
     x = Symbol("x")
     a = Matrix([x, 0], [0, 0])
     m = a
@@ -68,48 +82,56 @@ def test_creation():
 
     assert a == b
 
+    assert Matrix(b) == b
+
+def test_tolist():
+    x, y, z = symbols('xyz')
+    lst = [[S.One,S.Half,x*y,S.Zero],[x,y,z,x**2],[y,-S.One,z*x,3]]
+    m = Matrix(lst)
+    assert m.tolist() == lst
+
 def test_determinant():
     x, y, z = Symbol('x'), Symbol('y'), Symbol('z')
 
     M = Matrix((1,))
 
-    assert M.det() == 1
-    assert M.berkowitz_det() == 1
+    assert M.det(method="bareis") == 1
+    assert M.det(method="berkowitz") == 1
 
     M = Matrix(( (-3,  2),
                  ( 8, -5) ))
 
-    assert M.det() == -1
-    assert M.berkowitz_det() == -1
+    assert M.det(method="bareis") == -1
+    assert M.det(method="berkowitz") == -1
 
     M = Matrix(( (x,   1),
                  (y, 2*y) ))
 
-    assert M.det() == 2*x*y-y
-    assert M.berkowitz_det() == 2*x*y-y
+    assert M.det(method="bareis") == 2*x*y-y
+    assert M.det(method="berkowitz") == 2*x*y-y
 
     M = Matrix(( (1, 1, 1),
                  (1, 2, 3),
                  (1, 3, 6) ))
 
-    assert M.det() == 1
-    assert M.berkowitz_det() == 1
+    assert M.det(method="bareis") == 1
+    assert M.det(method="berkowitz") == 1
 
     M = Matrix(( ( 3, -2,  0, 5),
                  (-2,  1, -2, 2),
                  ( 0, -2,  5, 0),
                  ( 5,  0,  3, 4) ))
 
-    assert M.det() == -289
-    assert M.berkowitz_det() == -289
+    assert M.det(method="bareis") == -289
+    assert M.det(method="berkowitz") == -289
 
     M = Matrix(( ( 1,  2,  3,  4),
                  ( 5,  6,  7,  8),
                  ( 9, 10, 11, 12),
                  (13, 14, 15, 16) ))
 
-    assert M.det() == 0
-    assert M.berkowitz_det() == 0
+    assert M.det(method="bareis") == 0
+    assert M.det(method="berkowitz") == 0
 
     M = Matrix(( (3, 2, 0, 0, 0),
                  (0, 3, 2, 0, 0),
@@ -117,8 +139,8 @@ def test_determinant():
                  (0, 0, 0, 3, 2),
                  (2, 0, 0, 0, 3) ))
 
-    assert M.det() == 275
-    assert M.berkowitz_det() == 275
+    assert M.det(method="bareis") == 275
+    assert M.det(method="berkowitz") == 275
 
     M = Matrix(( (1, 0,  1,  2, 12),
                  (2, 0,  1,  1,  4),
@@ -126,8 +148,8 @@ def test_determinant():
                  (3, 2, -1,  1,  8),
                  (1, 1,  1,  0,  6) ))
 
-    assert M.det() == -55
-    assert M.berkowitz_det() == -55
+    assert M.det(method="bareis") == -55
+    assert M.det(method="berkowitz") == -55
 
     M = Matrix(( (-5,  2,  3,  4,  5),
                  ( 1, -4,  3,  4,  5),
@@ -135,8 +157,8 @@ def test_determinant():
                  ( 1,  2,  3, -2,  5),
                  ( 1,  2,  3,  4, -1) ))
 
-    assert M.det() == 11664
-    assert M.berkowitz_det() == 11664
+    assert M.det(method="bareis") == 11664
+    assert M.det(method="berkowitz") == 11664
 
     M = Matrix(( ( 2,  7, -1, 3, 2),
                  ( 0,  0,  1, 0, 1),
@@ -144,15 +166,15 @@ def test_determinant():
                  (-3, -2,  4, 5, 3),
                  ( 1,  0,  0, 0, 1) ))
 
-    assert M.det() == 123
-    assert M.berkowitz_det() == 123
+    assert M.det(method="bareis") == 123
+    assert M.det(method="berkowitz") == 123
 
     M = Matrix(( (x,y,z),
                  (1,0,0),
                  (y,z,x) ))
 
-    assert M.det() == z**2 - x*y
-    assert M.berkowitz_det() == z**2 - x*y
+    assert M.det(method="bareis") == z**2 - x*y
+    assert M.det(method="berkowitz") == z**2 - x*y
 
 def test_submatrix():
     m0 = eye(4)
@@ -206,6 +228,13 @@ def test_applyfunc():
     assert m0.applyfunc(lambda x:2*x) == eye(3)*2
     assert m0.applyfunc(lambda x: 0 ) == zero(3)
 
+def test_expand():
+    x,y = symbols('xy')
+    m0 = Matrix([[x*(x+y),2],[((x+y)*y)*x,x*(y+x*(x+y))]])
+    # Test if expand() returns a matrix
+    m1 = m0.expand()
+    assert m1 == Matrix([[x*y+x**2,2],[x*y**2+y*x**2,x*y+y*x**2+x**3]])
+
 def test_random():
     M = randMatrix(3,3)
     M = randMatrix(3,3,seed=3)
@@ -272,13 +301,15 @@ def test_LUsolve():
 def test_inverse():
     A = eye(4)
     assert A.inv() == eye(4)
+    assert A.inv("LU") == eye(4)
+    assert A.inv("ADJ") == eye(4)
     A = Matrix([[2,3,5],
                 [3,6,2],
                 [8,3,6]])
     Ainv = A.inv()
     assert A*Ainv == eye(3)
-    Ainv2 = A.inv("LU")
-    assert Ainv == Ainv2
+    assert A.inv("LU") == Ainv
+    assert A.inv("ADJ") == Ainv
 
 def test_util():
     v1 = Matrix(1,3,[1,2,3])
@@ -335,6 +366,13 @@ def test_nullspace():
     assert out == Matrix([[1,0,-R(2)/23,R(13)/23],
                               [0,1,R(8)/23, R(-6)/23]])
 
+    M = Matrix([[-5,-1, 4,-3,-1],
+                [ 1,-1,-1, 1, 0],
+                [-1, 0, 0, 0, 0],
+                [ 4, 1,-4, 3, 1],
+                [-2, 0, 2,-2,-1]])
+    assert M*M.nullspace()[0] == Matrix(5,1,[0]*5)
+
     M = Matrix([[1,3,0,2,6,3,1],
                 [-2,-6,0,-2,-8,3,1],
                 [3,9,0,0,6,6,2],
@@ -344,6 +382,7 @@ def test_nullspace():
                                [0,0,0,1,2,0,0],
                                [0,0,0,0,0,1,R(1)/3],
                                [0,0,0,0,0,0,0]])
+
     # now check the vectors
     basis = M.nullspace()
     assert basis[0] == Matrix([-3,1,0,0,0,0,0])
@@ -362,32 +401,43 @@ def test_wronskian():
         exp(x)*cos(x)*x**3 + exp(x)*sin(x)*x**3
 
 def test_eigen():
-    # test charpoly
-    x = Symbol('x')
-    y = Symbol('y')
-    eye3 = eye(3)
-    assert eye3.charpoly(x) == ((1-x)**3).expand()
-    assert eye3.charpoly(y) == ((1-y)**3).expand()
-    # test values
-    M = Matrix([(0,1,-1),
-                (1,1,0),
-                (-1,0,1) ])
-    vals = M.eigenvals()
-    vals.sort()
-    assert vals == [[-1,1],[1,1],[2,1]]
-    # test vectors
-    R = Rational
-    M = Matrix([ [1,0,0],
-                 [0,1,0],
-                 [0,0,1]])
-    assert M.eigenvects() == [[1, 3, [Matrix([1,0,0]), Matrix([0,1,0]), Matrix([0,0,1])]]]
-    M = Matrix([ [5,0,2],
-                 [3,2,0],
-                 [0,0,1]])
-    assert M.eigenvects() == [[1, 1, [Matrix([R(-1)/2,R(3)/2,1])]],
-                              [2, 1, [Matrix([0,1,0])]],
-                              [5, 1, [Matrix([1,1,0])]]]
+    x,y = symbols('xy')
 
+    R = Rational
+
+    assert eye(3).charpoly(x) == Poly((x-1)**3, x)
+    assert eye(3).charpoly(y) == Poly((y-1)**3, y)
+
+    M = Matrix([[1,0,0],
+                [0,1,0],
+                [0,0,1]])
+
+    assert M.eigenvals() == {S.One: 3}
+
+    assert M.eigenvects() == \
+        [(1, 3, [Matrix([1,0,0]),
+                 Matrix([0,1,0]),
+                 Matrix([0,0,1])])]
+
+    M = Matrix([[0,1,1],
+                [1,0,0],
+                [1,1,1]])
+
+    assert M.eigenvals() == {2*S.One: 1, -S.One: 1, S.Zero: 1}
+
+    assert M.eigenvects() == \
+        [( 2, 1, [Matrix([R(2,3), R(1,3), 1])]),
+         (-1, 1, [Matrix([-1, 1, 0])]),
+         ( 0, 1, [Matrix([ 0,-1, 1])])]
+
+    eps = Symbol('eps',real=True)
+
+    M = Matrix([abs(eps), I*eps    ],
+               [-I*eps,   abs(eps) ])
+
+    assert M.eigenvects() ==  \
+        [( 2*abs(eps), 1, [ Matrix([I*eps/abs(eps)],[1]) ] ),
+         ( 0, 1, [Matrix([-I*eps/abs(eps)],[1])]) ]
 def test_sparse_matrix():
     return
     def eye(n):
@@ -602,13 +652,15 @@ def test_sparse_matrix():
     # test_inverse
     A = eye(4)
     assert A.inv() == eye(4)
+    assert A.inv("LU") == eye(4)
+    assert A.inv("ADJ") == eye(4)
     A = SMatrix([[2,3,5],
                 [3,6,2],
                 [8,3,6]])
     Ainv = A.inv()
     assert A*Ainv == eye(3)
-    Ainv2 = A.inv("LU")
-    assert Ainv == Ainv2
+    assert A.inv("LU") == Ainv
+    assert A.inv("ADJ") == Ainv
 
     # test_cross
     v1 = Matrix(1,3,[1,2,3])
@@ -698,7 +750,35 @@ def test_sparse_matrix():
 def test_subs():
     x = Symbol('x')
     assert Matrix([[1,x],[x,4]]).subs(x, 5) == Matrix([[1,5],[5,4]])
+    y = Symbol('y')
+    assert Matrix([[x,2],[x+y,4]]).subs([[x,-1],[y,-2]]) == Matrix([[-1,2],[-3,4]])
+    assert Matrix([[x,2],[x+y,4]]).subs([(x,-1),(y,-2)]) == Matrix([[-1,2],[-3,4]])
+    assert Matrix([[x,2],[x+y,4]]).subs({x:-1,y:-2}) == Matrix([[-1,2],[-3,4]])
 
+def test_simplify():
+    x,y,f,n = symbols('xyfn')
+    M = Matrix([ [    1/x + 1/y,               (x+x*y)/ x             ],
+                 [(f(x)+y*f(x))/f(x), (2 * (1/n - cos(n * pi)/n))/ pi ]
+                 ])
+    M.simplify()
+    assert M ==  Matrix([[(x+y)/(x*y),               1 + y           ],
+                         [   1 + y,       (2 - 2*cos(pi*n))/ (pi*n)   ]])
+
+def test_transpose():
+    M = Matrix([[1,2,3,4,5,6,7,8,9,0],
+                [1,2,3,4,5,6,7,8,9,0]])
+    assert M.T == Matrix( [ [1,1],
+                            [2,2],
+                            [3,3],
+                            [4,4],
+                            [5,5],
+                            [6,6],
+                            [7,7],
+                            [8,8],
+                            [9,9],
+                            [0,0] ])
+    assert M.T.T == M
+    assert M.T == M.transpose()
 
 def test_conjugate():
     M = Matrix([ [0,I,5],
@@ -710,6 +790,7 @@ def test_conjugate():
 
     assert M.C == Matrix([ [0,-I,5],
                            [1,2,0]])
+    assert M.C == M.conjugate()
 
     assert M.H == M.T.C
     assert M.H == Matrix([ [0,1],
@@ -783,3 +864,81 @@ Matrix([
 [x,     1]
 [y, y + x]\
 """ ]
+
+def test_issue851():
+    m = Matrix([1, 2, 3])
+    a = Matrix([1, 2, 3])
+    b = Matrix([2, 2, 3])
+    assert not (m in [])
+    assert not (m in [1])
+    assert m != 1
+    assert m == a
+    assert m != b
+
+def test_issue882():
+    class Int1(object):
+        def __int__(self):
+            return 1
+    class Int2(object):
+        def __int__(self):
+            return 2
+    class Index1(object):
+        def __index__(self):
+            return 1
+    class Index2(object):
+        def __index__(self):
+            return 2
+    int1 = Int1()
+    int2 = Int2()
+    index1 = Index1()
+    index2 = Index2()
+
+    m = Matrix([1, 2, 3])
+    assert m[int2] == 3
+    assert m[index2] == 3
+
+    m[int2] = 4
+    assert m[2] == 4
+    m[index2] = 5
+    assert m[2] == 5
+
+    m = Matrix([1, 2, 3], [4, 5, 6])
+    assert m[int1,int2] == 6
+    assert m[index1,int2] == 6
+    assert m[int1,index2] == 6
+    assert m[index1,index2] == 6
+    assert m[1,int2] == 6
+    assert m[1,index2] == 6
+    assert m[int1,2] == 6
+    assert m[index1,2] == 6
+
+    m[int1,int2] = 1
+    assert m[1, 2] == 1
+    m[index1,int2] = 2
+    assert m[1, 2] == 2
+    m[int1,index2] = 3
+    assert m[1, 2] == 3
+    m[index1,index2] = 4
+    assert m[1, 2] == 4
+    m[1,int2] = 5
+    assert m[1, 2] == 5
+    m[1,index2] = 6
+    assert m[1, 2] == 6
+    m[int1,2] = 7
+    assert m[1, 2] == 7
+    m[index1,2] = 8
+    assert m[1, 2] == 8
+
+def test_evalf():
+    a = Matrix([sqrt(5), 6])
+    assert abs(a.evalf()[0] - a[0].evalf()) < 1e-10
+    assert abs(a.evalf()[1] - a[1].evalf()) < 1e-10
+
+def test_is_symbolic():
+    x = Symbol('x')
+    a = Matrix([x,x],[x,x])
+    assert a.is_symbolic() == True
+    a = Matrix([1,2],[3,4])
+    assert a.is_symbolic() == False
+    a = Matrix([1,x],[3,4])
+    assert a.is_symbolic() == True
