@@ -2,7 +2,7 @@
 Integer factorization
 """
 
-from sympy.core import numbers
+from sympy.core.numbers import igcd
 import random
 from primetest import isprime
 from generate import sieve, prime
@@ -51,14 +51,16 @@ def pollard_rho(n, max_iters=5, seed=1234):
     """
     prng = random.Random(seed + max_iters)
     for i in range(max_iters):
+        # Alternative good nonrandom choice: a = 1
         a = prng.randint(1, n-3)
+        # Alternative good nonrandom choice: s = 2
         s = prng.randint(0, n-1)
         U = V = s
-        F = lambda x: (x**2 + a) % a
+        F = lambda x: (x**2 + a) % n
         while 1:
             U = F(U)
             V = F(F(V))
-            g = numbers.gcd(U-V, n)
+            g = igcd(abs(U-V), n)
             if g == 1:
                 continue
             if g == n:
@@ -97,7 +99,7 @@ def pollard_pm1(n, B=10, seed=1234):
     for p in sieve.primerange(2, B):
         e = int(log(B, p))
         a = pow(a, p**e, n)
-    g = numbers.gcd(a-1, n)
+    g = igcd(a-1, n)
     if 1 < g < n:
         return g
     else:
@@ -353,7 +355,7 @@ def totient(n):
 
     """
     if n < 1:
-        raise ValueError, "n must be a positive integer"
+        raise ValueError("n must be a positive integer")
     factors = factorint(n)
     t = 1
     for p, k in factors:
