@@ -1,11 +1,14 @@
 import py
-from sympy import Sieve
+from sympy import Sieve, binomial_coefficients, binomial_coefficients_list, \
+        multinomial_coefficients
 from sympy.ntheory import isprime, n_order, is_primitive_root, \
     is_quad_residue, legendre_symbol, npartitions, totient, trial, \
     factorint, primefactors, divisors, randprime, nextprime, prevprime, \
-    primerange, primepi, prime
+    primerange, primepi, prime, pollard_rho
 from sympy.ntheory.bbp_pi import pi_hex_digits
 
+from sympy.ntheory.modular import crt, crt1, crt2
+
 def test_isprime():
     s = Sieve()
     s.extend(100000)
@@ -100,6 +103,8 @@ def test_factor():
     assert divisors(10) == [1, 2, 5, 10]
     assert divisors(100) == [1, 2, 4, 5, 10, 20, 25, 50, 100]
     assert divisors(101) == [1, 101]
+    assert pollard_rho(2**64+1, max_iters=1, seed=1) == 274177
+    assert pollard_rho(19) is None
 
 def test_totient():
     assert [totient(k) for k in range(1, 12)] == \
@@ -115,6 +120,8 @@ def test_partitions():
     assert npartitions(200) == 3972999029388
     assert npartitions(1000) == 24061467864032622473692149727991
     assert npartitions(2000) == 4720819175619413888601432406799959512200344166
+    assert npartitions(10000) % 10**10 == 6916435144
+    assert npartitions(100000) % 10**10 == 9421098519
 
 def test_residue():
     assert n_order(2,13)==12
@@ -143,4 +150,41 @@ def test_hex_pi_nth_digits():
     assert pi_hex_digits(1) == '243f6a8885a308'
     assert pi_hex_digits(10000) == '68ac8fcfb8016c'
 
+def test_crt():
+    assert crt([2, 3, 5], [0, 0, 0]) == 0
+    assert crt([2, 3, 5], [1, 1, 1]) == 1
 
+    assert crt([2, 3, 5], [-1, -1, -1], True) == -1
+    assert crt([2, 3, 5], [-1, -1, -1], False) == 2*3*5 - 1
+
+
+def test_binomial_coefficients_list():
+    assert binomial_coefficients_list(0) == [1]
+    assert binomial_coefficients_list(1) == [1,1]
+    assert binomial_coefficients_list(2) == [1,2,1]
+    assert binomial_coefficients_list(3) == [1,3,3,1]
+    assert binomial_coefficients_list(4) == [1,4,6,4,1]
+    assert binomial_coefficients_list(5) == [1,5,10,10,5,1]
+    assert binomial_coefficients_list(6) == [1,6,15,20,15,6,1]
+
+def test_binomial_coefficients():
+    for n in range(15):
+        c = binomial_coefficients(n)
+        l = [c[k] for k in sorted(c)]
+        assert l==binomial_coefficients_list(n)
+
+def test_multinomial_coefficients():
+    assert multinomial_coefficients(1, 1) == {(1,): 1}
+    assert multinomial_coefficients(1, 2) == {(2,): 1}
+    assert multinomial_coefficients(1, 3) == {(3,): 1}
+    assert multinomial_coefficients(2, 1) == {(0, 1): 1, (1, 0): 1}
+    assert multinomial_coefficients(2, 2) == {(2, 0): 1, (0, 2): 1, (1, 1): 2}
+    assert multinomial_coefficients(2, 3) == {(3, 0): 1, (1, 2): 3, (0, 3): 1,
+            (2, 1): 3}
+    assert multinomial_coefficients(3, 1) == {(1, 0, 0): 1, (0, 1, 0): 1,
+            (0, 0, 1): 1}
+    assert multinomial_coefficients(3, 2) == {(0, 1, 1): 2, (0, 0, 2): 1,
+            (1, 1, 0): 2, (0, 2, 0): 1, (1, 0, 1): 2, (2, 0, 0): 1}
+    assert multinomial_coefficients(3, 3) == {(2, 1, 0): 3, (0, 3, 0): 1,
+            (1, 0, 2): 3, (0, 2, 1): 3, (0, 1, 2): 3, (3, 0, 0): 1,
+            (2, 0, 1): 3, (1, 2, 0): 3, (1, 1, 1): 6, (0, 0, 3): 1}
