@@ -5,6 +5,8 @@ from sympy.polynomials import div_, groebner_
 from sympy.functions.elementary.miscellaneous import sqrt
 from sympy.core import sympify
 
+from sympy.core.numbers import igcd
+
 def cubic(f):
     """Computes the roots of a cubic polynomial.
 
@@ -70,7 +72,7 @@ def n_poly(f):
         return result
 
     exponents = map(lambda t:int(t[1]), f.coeffs)
-    g = reduce(numbers.gcd, exponents)
+    g = reduce(igcd, exponents)
     if g == 1 or g == 0:
         return None
     n = int(f.coeffs[0][1]/g)
@@ -285,7 +287,7 @@ def roots(f, var=None):
         raise PolynomialException('Multivariate polynomials not supported.')
 
     # Determine type of coeffs (for factorization purposes)
-    symbols = f.sympy_expr.atoms(type=Symbol)
+    symbols = f.sympy_expr.atoms(Symbol)
     symbols = filter(lambda a: not a in f.var, symbols)
     if symbols:
         coeff = 'sym'
@@ -375,7 +377,7 @@ def solve_system(eqs, var=None, order=None):
         eqs = [eqs]
     if not isinstance(eqs[0], Polynomial):
         if var is None:
-            var = merge_var(*[f.atoms(type=Symbol) for f in eqs])
+            var = merge_var(*[f.atoms(Symbol) for f in eqs])
         eqs = [Polynomial(f, var=var, order='lex') for f in eqs]
     else:
         eqs = [Polynomial(f.sympy_expr, var=f.var, order='lex') for f in eqs]
