@@ -1,10 +1,12 @@
 import py
 from sympy.utilities.pytest import XFAIL
 
-from sympy import Polynomial, pi, symbols, Symbol, Rational, S, sqf_part, sqf,\
-        solve_system, Integer, I, roots, cos, sin, resultant, \
-        count_real_roots, lcm, groebner, gcd, factor, div, Real, \
-        PolynomialException, sqrt, sympify
+from sympy import pi, symbols, Symbol, Rational, \
+    S, Integer, I, cos, sin, Real, sqrt, sympify
+
+from sympy.polynomials import Polynomial, PolynomialException,  \
+    sqf_part, sqf, solve_system, roots, resultant, lcm, factor, \
+    groebner, count_real_roots, gcd, div
 
 def test_Polynomial():
     x = Symbol("x")
@@ -285,10 +287,16 @@ def test_roots():
            [-b/(a*2)+(((b/a)**2-4*c/a)**Rational(1,2))/2,
             -b/(a*2)-(((b/a)**2-4*c/a)**Rational(1,2))/2]
     assert roots(x**3 + x**2 + x + 1) == [-1, I, -I]
-    assert roots(x**3 + x**2 - x + 1) == \
+    assert roots(x**3 + x**2 - x + 1) in [
            [Rational(-1,3) - (Rational(19,27) + Rational(1,9)*3**Rational(1,2)*11**Rational(1,2))**Rational(1,3) - Rational(4,9)*(Rational(19,27) + Rational(1,9)*3**Rational(1,2)*11**Rational(1,2))**Rational(-1,3),
             Rational(-1,3) + Rational(1,2)*(Rational(19,27) + Rational(1,9)*3**Rational(1,2)*11**Rational(1,2))**Rational(1,3) + Rational(4,9)/(Rational(1,2) + Rational(1,2)*I*3**Rational(1,2))*(Rational(19,27) + Rational(1,9)*3**Rational(1,2)*11**Rational(1,2))**Rational(-1,3) + Rational(1,2)*I*3**Rational(1,2)*(Rational(19,27) + Rational(1,9)*3**Rational(1,2)*11**Rational(1,2))**Rational(1,3),
-            Rational(-1,3) + Rational(1,2)*(Rational(19,27) + Rational(1,9)*3**Rational(1,2)*11**Rational(1,2))**Rational(1,3) + Rational(4,9)/(Rational(1,2) - Rational(1,2)*I*3**Rational(1,2))*(Rational(19,27) + Rational(1,9)*3**Rational(1,2)*11**Rational(1,2))**Rational(-1,3) - Rational(1,2)*I*3**Rational(1,2)*(Rational(19,27) + Rational(1,9)*3**Rational(1,2)*11**Rational(1,2))**Rational(1,3)]
+            Rational(-1,3) + Rational(1,2)*(Rational(19,27) + Rational(1,9)*3**Rational(1,2)*11**Rational(1,2))**Rational(1,3) + Rational(4,9)/(Rational(1,2) - Rational(1,2)*I*3**Rational(1,2))*(Rational(19,27) + Rational(1,9)*3**Rational(1,2)*11**Rational(1,2))**Rational(-1,3) - Rational(1,2)*I*3**Rational(1,2)*(Rational(19,27) + Rational(1,9)*3**Rational(1,2)*11**Rational(1,2))**Rational(1,3)],
+
+           [Rational(-1,3) - (Rational(19,27) + Rational(1,9)*3**Rational(1,2)*11**Rational(1,2))**Rational(1,3) - Rational(4,9)*(Rational(19,27) + Rational(1,9)*3**Rational(1,2)*11**Rational(1,2))**Rational(-1,3),
+            Rational(-1,3) + Rational(1,2)*(Rational(19,27) + Rational(1,9)*3**Rational(1,2)*11**Rational(1,2))**Rational(1,3) - Rational(4,9)/(-Rational(1,2) - Rational(1,2)*I*3**Rational(1,2))*(Rational(19,27) + Rational(1,9)*3**Rational(1,2)*11**Rational(1,2))**Rational(-1,3) + Rational(1,2)*I*3**Rational(1,2)*(Rational(19,27) + Rational(1,9)*3**Rational(1,2)*11**Rational(1,2))**Rational(1,3),
+            Rational(-1,3) + Rational(1,2)*(Rational(19,27) + Rational(1,9)*3**Rational(1,2)*11**Rational(1,2))**Rational(1,3) + Rational(4,9)/(Rational(1,2) - Rational(1,2)*I*3**Rational(1,2))*(Rational(19,27) + Rational(1,9)*3**Rational(1,2)*11**Rational(1,2))**Rational(-1,3) - Rational(1,2)*I*3**Rational(1,2)*(Rational(19,27) + Rational(1,9)*3**Rational(1,2)*11**Rational(1,2))**Rational(1,3)],
+                                            ]
+
     assert roots(x**4 - 1) == [1, I, -1, -I]
     assert roots(x**4 + 1) == [((-1)**Rational(1,4)).expand(complex=True),
                                ((-1)**Rational(3,4)).expand(complex=True),
@@ -433,9 +441,9 @@ def test_roots_issue604():
     x = Symbol('x')
     p = x**3+2*x**2+8
     r = roots(p, x)
-    assert abs(p.subs(x, r[0]).evalf()) < 1E-20
-    assert abs(p.subs(x, r[1]).evalf()) < 1E-20
-    assert abs(p.subs(x, r[2]).evalf()) < 1E-20
+    assert abs(p.subs(x, r[0]).evalf()) < 1E-12
+    assert abs(p.subs(x, r[1]).evalf()) < 1E-12
+    assert abs(p.subs(x, r[2]).evalf()) < 1E-12
 
 
 def test_coeff1():
