@@ -9,15 +9,12 @@ from sympy.core.sympify import sympify
 
 from sympy.core.numbers import igcd, ilcm
 
-from polynomial import Poly, PolynomialError
+from polynomial import Poly, SymbolsError, MultivariatePolyError
 
 from monomial import monomial_cmp, monomial_lcm, \
     monomial_gcd, monomial_mul, monomial_div
 
-import sympy.matrices
-
-# Python2.4 compatibility
-from sympy.utilities.iterables import all
+from sympy.utilities.iterables import all, any
 
 def poly_div(f, g, *symbols):
     """Generalized polynomial division with remainder.
@@ -42,7 +39,7 @@ def poly_div(f, g, *symbols):
     if not isinstance(f, Poly):
         f = Poly(f, *symbols)
     elif symbols:
-        raise PolynomialError
+        raise SymbolsError("Redundant symbols were given")
 
     f, g = f.unify_with(g)
 
@@ -66,17 +63,13 @@ def poly_div(f, g, *symbols):
             r_coeffs, r_monoms = [], []
 
             for coeff, monom in f.iter_terms():
-                M = monomial_div(monom, LM)
+                quotient = monomial_div(monom, LM)
 
-                if M is not None:
+                if quotient is not None:
                     coeff /= LC
 
-                    if coeff.is_Atom:
-                        q_coeffs.append(coeff)
-                    else:
-                        q_coeffs.append(Poly._cancel(coeff))
-
-                    q_monoms.append(M)
+                    q_coeffs.append(Poly.cancel(coeff))
+                    q_monoms.append(quotient)
                 else:
                     r_coeffs.append(coeff)
                     r_monoms.append(monom)
@@ -93,10 +86,7 @@ def poly_div(f, g, *symbols):
             monom = monomial_div(f.LM, h.LM)
 
             if monom is not None:
-                coeff = f.LC / h.LC
-
-                if not coeff.is_Atom:
-                    coeff = Poly._cancel(coeff)
+                coeff = Poly.cancel(f.LC / h.LC)
 
                 q[i] = q[i].add_term(coeff, monom)
                 f -= h.mul_term(coeff, monom)
@@ -107,7 +97,7 @@ def poly_div(f, g, *symbols):
             f = f.kill_lead_term()
 
     if len(q) != 1:
-        return tuple(q), r
+        return q, r
     else:
         return q[0], r
 
@@ -130,12 +120,12 @@ def poly_pdiv(f, g, *symbols):
     if not isinstance(f, Poly):
         f = Poly(f, *symbols)
     elif symbols:
-        raise PolynomialError
+        raise SymbolsError("Redundant symbols were given")
 
     f, g = f.unify_with(g)
 
     if f.is_multivariate:
-        raise PolynomialError
+        raise MultivariatePolyError(f)
 
     symbols, flags = f.symbols, f.flags
 
@@ -203,7 +193,7 @@ def poly_groebner(f, *symbols, **flags):
         if not isinstance(f, Poly):
             f = Poly(f, *symbols, **flags)
         elif symbols or flags:
-            raise PolynomialError
+            raise SymbolsError("Redundant symbols or flags were given")
 
         f, g = f.unify_with(g)
 
@@ -212,16 +202,16 @@ def poly_groebner(f, *symbols, **flags):
         if not isinstance(f, Poly):
             f = Poly(f, *symbols, **flags)
         elif symbols or flags:
-            raise PolynomialError
+            raise SymbolsError("Redundant symbols or flags were given")
 
-        return (f.as_monic(),)
+        return [f.as_monic()]
 
     compare = monomial_cmp(flags.get('order'))
 
     f = [ h for h in [f] + g if h ]
 
     if not f:
-        return (Poly((), *symbols, **flags),)
+        return [Poly((), *symbols, **flags)]
 
     R, P, G, B, F = set(), set(), set(), {}, {}
 
@@ -337,7 +327,7 @@ def poly_groebner(f, *symbols, **flags):
 
     G = sorted(G, compare, lambda p: p.LM)
 
-    return tuple(reversed(G))
+    return list(reversed(G))
 
 def poly_lcm(f, g, *symbols):
     """Computes least common multiple of two polynomials.
@@ -359,7 +349,7 @@ def poly_lcm(f, g, *symbols):
     if not isinstance(f, Poly):
         f = Poly(f, *symbols)
     elif symbols:
-        raise PolynomialError
+        raise SymbolsError("Redundant symbols were given")
 
     f, g = f.unify_with(g)
 
@@ -434,12 +424,29 @@ def poly_gcd(f, g, *symbols):
     if not isinstance(f, Poly):
         f = Poly(f, *symbols)
     elif symbols:
-        raise PolynomialError
+        raise SymbolsError("Redundant symbols were given")
 
     f, g = f.unify_with(g)
 
     symbols, flags = f.symbols, f.flags
 
+    if f.is_zero and g.is_zero:
+        return f
+
+    if f.is_constant:
+        if f.is_zero:
+            cont, g = g.as_primitive()
+            return g.mul_term(cont / g.LC)
+        if f.is_one:
+            return f
+
+    if g.is_constant:
+        if g.is_zero:
+            cont, f = f.as_primitive()
+            return f.mul_term(cont / f.LC)
+        if g.is_one:
+            return g
+
     if f.is_monomial and g.is_monomial:
         monom = monomial_gcd(f.LM, g.LM)
 
@@ -500,12 +507,12 @@ def poly_half_gcdex(f, g, *symbols):
     if not isinstance(f, Poly):
         f = Poly(f, *symbols)
     elif symbols:
-        raise PolynomialError
+        raise SymbolsError("Redundant symbols were given")
 
     f, g = f.unify_with(g)
 
     if f.is_multivariate:
-        raise PolynomialError
+        raise MultivariatePolyError(f)
 
     symbols, flags = f.symbols, f.flags
 
@@ -550,12 +557,12 @@ def poly_resultant(f, g, *symbols):
     if not isinstance(f, Poly):
         f = Poly(f, *symbols)
     elif symbols:
-        raise PolynomialError
+        raise SymbolsError("Redundant symbols were given")
 
     f, g = f.unify_with(g)
 
     if f.is_multivariate:
-        raise PolynomialError
+        raise MultivariatePolyError(f)
 
     n, m = f.degree, g.degree
 
@@ -568,6 +575,8 @@ def poly_resultant(f, g, *symbols):
         q = f.as_uv_dict()
         p = g.as_uv_dict()
 
+    import sympy.matrices
+
     B = sympy.matrices.zero(N)
 
     for i in xrange(N):
@@ -601,7 +610,7 @@ def poly_resultant(f, g, *symbols):
         if det.is_Atom:
             return sign * det
         else:
-            return sign * Poly._cancel(det)
+            return sign * Poly.cancel(det)
 
 def poly_subresultants(f, g, *symbols):
     """Computes subresultant PRS of two univariate polynomials.
@@ -642,12 +651,12 @@ def poly_subresultants(f, g, *symbols):
     if not isinstance(f, Poly):
         f = Poly(f, *symbols)
     elif symbols:
-        raise PolynomialError
+        raise SymbolsError("Redundant symbols were given")
 
     f, g = f.unify_with(g)
 
     if f.is_multivariate:
-        raise PolynomialError
+        raise MultivariatePolyError(f)
 
     symbols, flags = f.symbols, f.flags
 
@@ -686,7 +695,7 @@ def poly_subresultants(f, g, *symbols):
 
         k = h.degree
 
-    return tuple(prs)
+    return prs
 
 def poly_sqf(f, *symbols):
     """Compute square-free decomposition of an univariate polynomial.
@@ -717,16 +726,16 @@ def poly_sqf(f, *symbols):
     if not isinstance(f, Poly):
         f = Poly(f, *symbols)
     elif symbols:
-        raise PolynomialError
+        raise SymbolsError("Redundant symbols were given")
 
     if f.is_multivariate:
-        raise PolynomialError
+        raise MultivariatePolyError(f)
 
     coeff, f = f.as_primitive()
 
-    sqf, x = [], f.symbols[0]
+    sqf = []
 
-    h = f.diff(x)
+    h = f.diff()
 
     g = poly_gcd(f, h)
 
@@ -734,7 +743,7 @@ def poly_sqf(f, *symbols):
     q = poly_div(h, g)[0]
 
     while True:
-        h = q - p.diff(x)
+        h = q - p.diff()
 
         if h.is_zero:
             break
@@ -751,7 +760,7 @@ def poly_sqf(f, *symbols):
     head, tail = sqf[0], sqf[1:]
     head = head.mul_term(coeff)
 
-    return (head,) + tuple(tail)
+    return [head] + tail
 
 def poly_decompose(f, *symbols):
     """Computes functional decomposition of an univariate polynomial.
@@ -793,10 +802,10 @@ def poly_decompose(f, *symbols):
     if not isinstance(f, Poly):
         f = Poly(f, *symbols)
     elif symbols:
-        raise PolynomialError
+        raise SymbolsError("Redundant symbols were given")
 
     if f.is_multivariate:
-        raise PolynomialError
+        raise MultivariatePolyError(f)
 
     symbols = f.symbols
     flags = f.flags
@@ -872,4 +881,4 @@ def poly_decompose(f, *symbols):
         else:
             break
 
-    return (f,) + tuple(F)
+    return [f] + F
