@@ -2,9 +2,9 @@
 from sympy.core.add import Add
 from sympy.core.mul import Mul
 from sympy.core.power import Pow
-from sympy.core.symbol import Symbol
 from sympy.core.sympify import sympify
 from sympy.core.basic import Basic, S, C
+from sympy.core.symbol import Symbol, Wild
 from sympy.core.numbers import Integer, igcd, ilcm
 from sympy.core.methods import RelMeths, ArithMeths
 
@@ -17,9 +17,31 @@ import sympy.polys
 
 import math
 
+class SymbolsError(Exception):
+    pass
+
 class PolynomialError(Exception):
     pass
 
+class CoefficientError(PolynomialError):
+    pass
+
+class UnivariatePolyError(PolynomialError):
+
+    def __init__(self, poly):
+        self.message = "%s is an univariate polynomial" % poly
+
+    def __str__(self):
+        return self.message
+
+class MultivariatePolyError(PolynomialError):
+
+    def __init__(self, poly):
+        self.message = "%s is a multivariate polynomial" % poly
+
+    def __str__(self):
+        return self.message
+
 ##
 ## TODO:
 ##
@@ -30,9 +52,6 @@ class PolynomialError(Exception):
 ##  [5] Improve coefficients analysis.
 ##  [6] Implement FactoredPoly.
 ##  [7] Concept of Monomial.
-##  [8] Generalize some polynomial functions, like factor(),
-##      to rational expressions. Move at least cancel() and
-##      maybe apart() to polys.fraction (or similar).
 ##
 
 class Poly(Basic, RelMeths, ArithMeths):
@@ -127,15 +146,23 @@ class Poly(Basic, RelMeths, ArithMeths):
 
        [4] Items generators:
 
-          [4.1] [U-] iter_coeffs --> iterate over coefficients
-          [4.2] [U-] iter_monoms --> iterate over monomials
-          [4.3] [U-] iter_terms  --> iterate over terms
+          [4.1] [U-] iter_coeffs     --> iterate over coefficients
+          [4.2] [U-] iter_monoms     --> iterate over monomials
+          [4.3] [U-] iter_terms      --> iterate over terms
 
-       [5] Leading items:
+          [4.4] [U-] iter_all_coeffs --> iterate over all coefficients
+          [4.5] [U-] iter_all_monoms --> iterate over all monomials
+          [4.6] [U-] iter_all_terms  --> iterate over all terms
 
-          [5.1] [UP] lead_coeff or LC --> returns leading coefficient
-          [5.2] [UP] lead_monom or LM --> returns leading monomial
-          [5.3] [UP] lead_term  or LT --> returns leading term
+       [5] Leading and trailing items:
+
+          [5.1] [UP] lead_coeff or LC --> returns the leading coefficient
+          [5.2] [UP] lead_monom or LM --> returns the leading monomial
+          [5.3] [UP] lead_term  or LT --> returns the leading term
+
+          [5.4] [UP] tail_coeff or TC --> returns the trailing coefficient
+          [5.5] [UP] tail_monom or TM --> returns the trailing monomial
+          [5.6] [UP] tail_term  or TT --> returns the trailing term
 
        [6] Arithmetic operators:
 
@@ -157,7 +184,7 @@ class Poly(Basic, RelMeths, ArithMeths):
           [7.3]  [-P] is_number        --> only numeric constant term
           [7.4]  [UP] is_constant      --> only arbitrary constant term
           [7.5]  [UP] is_monomial      --> number of terms == 1
-          [7.6]  [UP] is_univariate    --> number of variables = 1
+          [7.6]  [UP] is_univariate    --> number of variables == 1
           [7.7]  [UP] is_multivariate  --> number of variables > 1
           [7.8]  [UP] is_homogeneous   --> has constant term
           [7.9]  [UP] is_inhomogeneous --> no constant term
@@ -166,6 +193,7 @@ class Poly(Basic, RelMeths, ArithMeths):
           [7.12] [UP] is_monic         --> returns True if leading coeff is one
           [7.13] [UP] is_primitive     --> returns True if content is one
           [7.14] [UP] is_squarefree    --> no factors of multiplicity >= 2
+          [7.15] [UP] is_linear        --> all monomials of degree <= 1
 
        [8] Coefficients properties:
 
@@ -178,21 +206,22 @@ class Poly(Basic, RelMeths, ArithMeths):
        [9] Special functionality:
 
           [9.1]  [--] as_monic      --> divides all coefficients by LC
+          [9.2]  [--] as_integer    --> returns polynomial with integer coeffs
 
-          [9.2]  [-P] content       --> returns GCD of polynomial coefficients
-          [9.3]  [--] as_primitive  --> returns content and primitive part
+          [9.3]  [-P] content       --> returns GCD of polynomial coefficients
+          [9.4]  [--] as_primitive  --> returns content and primitive part
 
-          [9.4]  [U-] as_squarefree --> returns square-free part of a polynomial
+          [9.5]  [U-] as_squarefree --> returns square-free part of a polynomial
 
-          [9.5]  [U-] as_reduced    --> extract GCD of polynomial monomials
+          [9.6]  [U-] as_reduced    --> extract GCD of polynomial monomials
 
-          [9.6]  [--] map_coeffs    --> applies a function to all coefficients
-          [9.7]  [U-] coeff         --> returns coefficient of the given monomial
+          [9.7]  [--] map_coeffs    --> applies a function to all coefficients
+          [9.8]  [U-] coeff         --> returns coefficient of the given monomial
 
-          [9.8]  [U-] unify_with    --> returns polys with a common set of symbols
+          [9.9]  [U-] unify_with    --> returns polys with a common set of symbols
 
-          [9.9]  [U-] diff          --> efficient polynomial differentiation
-          [9.10] [U-] integrate     --> efficient polynomial integration
+          [9.10] [U-] diff          --> efficient polynomial differentiation
+          [9.11] [U-] integrate     --> efficient polynomial integration
 
        [10] Operations on terms:
 
@@ -217,15 +246,15 @@ class Poly(Basic, RelMeths, ArithMeths):
           [12.2] [U-] __ne__      --> P1 != P, up to order of monomials
           [12.3] [U-] __nonzero__ --> check if zero polynomial
 
-       [13] Basic functionality:
+       [13] String representation functions:
 
-          [13.1] [--] atoms       --> returns the atoms that form a polynomial
+          [13.1] [U-] torepr      --> returns represtation of 'self'
+          [13.3] [U-] tostr       --> returns pretty representation
 
-       [14] Other functionality:
+       [14] Other (derived from Basic) functionality:
 
           [14.1] [U-] _eval_is_polynomial --> checks if poly is a poly
 
-
        For general information on polynomials and algorithms, refer to:
 
        [1] D. E. Knuth, The Art of Computer Programming: Seminumerical
@@ -254,14 +283,34 @@ class Poly(Basic, RelMeths, ArithMeths):
 
     def __new__(cls, poly, *symbols, **flags):
         N, terms = len(symbols), {}
-        stamp = frozenset(symbols)
         coeffs, monoms = (), ()
 
-        if N == 0 or len(stamp) != N:
-            raise PolynomialError
-
         order = flags.pop('order', 'grlex')
 
+        if N == 0:
+            if isinstance(poly, Poly):
+                if order != poly.order:
+                    symbols = poly.symbols
+                    N = len(poly.symbols)
+                    poly = poly.as_dict()
+                else:
+                    return poly
+            else:
+                raise SymbolsError("No symbols were given")
+
+        stamp = frozenset(symbols)
+
+        if len(stamp) != N:
+            raise SymbolsError("Duplicate symbols: %s" % (symbols,))
+
+        symbols = tuple(map(sympify, symbols))
+
+        if any(not s.is_Symbol for s in symbols):
+            raise SymbolsError("Invalid symbols: %s" % (symbols,))
+
+        if any(not s.is_commutative for s in symbols):
+            raise SymbolsError("Non-commutative symbols: %s" % (symbols,))
+
         # { M1: c1, M2: c2, ... }
         if type(poly) is dict:
             if not poly:
@@ -287,7 +336,8 @@ class Poly(Basic, RelMeths, ArithMeths):
                         coeffs = (coeffs,)
                         monoms = (monoms,)
                 else:
-                    raise PolynomialError
+                    raise PolynomialError("Invalid arguments," \
+                        " should get '(coeffs, monoms)' tuple")
 
         # [ (c1,M1), (c2,M2), ... ]
         elif type(poly) is list:
@@ -299,13 +349,15 @@ class Poly(Basic, RelMeths, ArithMeths):
                     coeff = sympify(coeff)
 
                     if coeff.has_any_symbols(*symbols):
-                        raise PolynomialError
+                        raise CoefficientError("%s coefficient is dependent" \
+                            " of polynomial's symbols %s" % (coeff, symbols))
 
                     if type(monom) is int:
                         monom = (monom,)
 
                     if len(monom) != N:
-                        raise PolynomialError
+                        raise PolynomialError("Dimension of %s monomial does" \
+                            " not match the number of symbols (%d)" % (monom, N))
 
                     if not coeff:
                         continue
@@ -485,7 +537,7 @@ class Poly(Basic, RelMeths, ArithMeths):
                             monom[indices[factor]] += 1
                             continue
 
-                        raise PolynomialError
+                        raise PolynomialError("Can't decompose %s" % factor)
                     else:
                         coeff *= factor
 
@@ -506,45 +558,88 @@ class Poly(Basic, RelMeths, ArithMeths):
             return result
 
     @staticmethod
-    def _cancel(f):
+    def cancel(f, *symbols):
         """Cancel common factors in a fractional expression.
 
            Given a quotient of polynomials, cancel common factors from
-           the numerator and the denominator. The result is formed
-           in an expanded form, even if there was no cancellation.
+           the numerator and the denominator.  The result is formed in
+           an expanded form, even if there was no cancellation.
+
+           To improve the  speed of calculations a list of symbols can
+           be specified. This can be particularly useful in univariate
+           case with additional parameters, to avoid Groebner basis
+           computations.
+
+           The input fraction can be given as a single expression or
+           as a tuple consisting of the numerator and denominator.
 
            >>> from sympy import *
            >>> x,y = symbols('xy')
 
-           >>> Poly._cancel((x**2-y**2)/(x-y))
+           >>> Poly.cancel((x**2-y**2)/(x-y), x, y)
+           x + y
+
+           >>> Poly.cancel((x**2-y**2, x-y), x, y)
            x + y
 
         """
-        symbols = f.atoms(Symbol)
-
-        if not symbols:
-            return f
+        if type(f) is tuple:
+            numer, denom = f
         else:
+            if f.is_Atom:
+                return f
+
+            if not symbols:
+                symbols = f.atoms(Symbol)
+
+                if not symbols:
+                    symbols = None
+                else:
+                    symbols = sorted(symbols)
+
             numer, denom = f.as_numer_denom()
 
-            numer = numer.expand()
-            denom = denom.expand()
+        numer = numer.expand()
 
-            if numer.is_Number or denom.is_Number:
-                return numer / denom
+        if numer is S.Zero:
+            return numer
 
-            try:
-                p = Poly(numer, *symbols)
-                q = Poly(denom, *symbols)
-            except PolynomialError:
-                return numer / denom
+        denom = denom.expand()
 
-            g = sympy.polys.algorithms.poly_gcd(p, q)
+        if denom.is_Number:
+            return numer / denom
 
-            p = sympy.polys.algorithms.poly_div(p, g)[0]
-            q = sympy.polys.algorithms.poly_div(q, g)[0]
+        if symbols is None:
+            if denom.is_Add:
+                a, b, c = map(Wild, 'abc')
 
-            return p.as_basic() / q.as_basic()
+                r = denom.match(a + b*c**S.Half)
+
+                if r is not None:
+                    if r[b] is S.Zero:
+                        denom = match[a]
+                    else:
+                        a, b, c = r[a], r[b], r[c]
+
+                        numer *= a-b*c**S.Half
+                        numer = numer.expand()
+
+                        denom = a**2 - c*b**2
+
+            return numer / denom
+
+        try:
+            p = Poly(numer, *symbols)
+            q = Poly(denom, *symbols)
+        except PolynomialError:
+            return numer / denom
+
+        g = sympy.polys.algorithms.poly_gcd(p, q)
+
+        p = sympy.polys.algorithms.poly_div(p, g)[0]
+        q = sympy.polys.algorithms.poly_div(q, g)[0]
+
+        return p.as_basic() / q.as_basic()
 
     def as_basic(self):
         """Converts polynomial to a valid sympy expression.
@@ -592,10 +687,11 @@ class Poly(Basic, RelMeths, ArithMeths):
     def as_uv_dict(self):
         """Return dictionary representation with integer keys.
 
-           In case of univariate polynomials it is in efficient to
+           In case of univariate polynomials it is inefficient to
            to handle exponents as singleton tuples, as an example
-           see multiplication algorithm. This procedure will
-           convert tuples to explicit integers.
+           see multiplication algorithm.  This method will return
+           dictionary representation with all tuples converted to
+           explicit integers.
 
            >>> from sympy import *
            >>> x,y = symbols('xy')
@@ -603,11 +699,12 @@ class Poly(Basic, RelMeths, ArithMeths):
            >>> p = Poly(x**2+3, x)
            >>> p.as_uv_dict()
            {0: 3, 2: 1}
+
         """
         if self.is_univariate:
             return dict(zip([ M[0] for M in self.monoms ], self.coeffs))
         else:
-            raise PolynomialError
+            raise MultivariatePolyError(self)
 
     @property
     def coeffs(self):
@@ -660,6 +757,14 @@ class Poly(Basic, RelMeths, ArithMeths):
         """Computes oo-norm of polynomial's coefficients. """
         return max([ abs(coeff) for coeff in self.coeffs ])
 
+    def iter_basic_args(self):
+        for coeff in self.coeffs:
+            yield coeff
+
+        for i, symbol in enumerate(self.symbols):
+            if any(monom[i] != 0 for monom in self.monoms):
+                yield symbol
+
     def iter_coeffs(self):
         for coeff in self.coeffs:
             yield coeff
@@ -672,6 +777,37 @@ class Poly(Basic, RelMeths, ArithMeths):
         for coeff, monom in zip(self.coeffs, self.monoms):
             yield coeff, monom
 
+    def iter_all_coeffs(self):
+        if self.is_univariate:
+            terms = self.as_uv_dict()
+
+            for i in xrange(self.degree, -1, -1):
+                if terms.has_key(i):
+                    yield terms[i]
+                else:
+                    yield S.Zero
+        else:
+            raise MultivariatePolyError(self)
+
+    def iter_all_monoms(self):
+        if self.is_univariate:
+            for i in xrange(self.degree, -1, -1):
+                yield (i,)
+        else:
+            raise MultivariatePolyError(self)
+
+    def iter_all_terms(self):
+        if self.is_univariate:
+            terms = self.as_uv_dict()
+
+            for i in xrange(self.degree, -1, -1):
+                if terms.has_key(i):
+                    yield terms[i], (i,)
+                else:
+                    yield S.Zero, (i,)
+        else:
+            raise MultivariatePolyError(self)
+
     @property
     def lead_coeff(self):
         return self.coeffs[0]
@@ -688,6 +824,22 @@ class Poly(Basic, RelMeths, ArithMeths):
     LM = lead_monom
     LT = lead_term
 
+    @property
+    def tail_coeff(self):
+        return self.coeffs[-1]
+
+    @property
+    def tail_monom(self):
+        return self.monoms[-1]
+
+    @property
+    def tail_term(self):
+        return (self.coeffs[-1], self.monoms[-1])
+
+    TC = tail_coeff
+    TM = tail_monom
+    TT = tail_term
+
     def __abs__(self):
         return self.__class__(([ abs(coeff) for coeff in self.coeffs ],
             self.monoms), *self.symbols, **self.flags)
@@ -720,6 +872,8 @@ class Poly(Basic, RelMeths, ArithMeths):
             if terms.has_key(monom):
                 coeff += terms[monom]
 
+                coeff = Poly.cancel(coeff)
+
                 if not coeff:
                     del terms[monom]
                     continue
@@ -752,6 +906,8 @@ class Poly(Basic, RelMeths, ArithMeths):
             if terms.has_key(monom):
                 coeff -= terms[monom]
 
+                coeff = Poly.cancel(coeff)
+
                 if not coeff:
                     del terms[monom]
                     continue
@@ -834,12 +990,7 @@ class Poly(Basic, RelMeths, ArithMeths):
 
                 for i in xrange(k+1):
                     if p.has_key(i) and q.has_key(k-i):
-                        product = p[i] * q[k-i]
-
-                        if product.is_Atom:
-                            coeff += product
-                        else:
-                            coeff += Poly._cancel(product)
+                        coeff += Poly.cancel(p[i] * q[k-i])
 
                 if coeff:
                     coeffs.append(coeff)
@@ -866,9 +1017,7 @@ class Poly(Basic, RelMeths, ArithMeths):
                     monom = monomial_mul(M, N)
 
                     coeff = coeff_p * coeff_q
-
-                    if not coeff.is_Atom:
-                        coeff = Poly._cancel(coeff)
+                    coeff = Poly.cancel(coeff)
 
                     if terms.has_key(monom):
                         coeff += terms[monom]
@@ -1050,6 +1199,10 @@ class Poly(Basic, RelMeths, ArithMeths):
         """
         return self.as_squarefree() is self
 
+    @property
+    def is_linear(self):
+        return all(sum(monom) <= 1 for monom in self.iter_monoms())
+
     @property
     def has_integer_coeffs(self):
         return self.domain == 'Z'
@@ -1071,19 +1224,19 @@ class Poly(Basic, RelMeths, ArithMeths):
         return self.domain == 'S'
 
     def as_monic(self):
-        """Returns leading coefficient and monic polynomial.
+        """Returns a monic polynomial.
 
            >>> from sympy import *
            >>> x,y = symbols('xy')
 
            >>> Poly(x**2 + 4, x).as_monic()
-           Poly((1, 4), ((2,), (0,)), (x,), 'grlex')
+           Poly(x**2 + 4, x)
 
            >>> Poly(2*x**2 + 4, x).as_monic()
-           Poly((1, 2), ((2,), (0,)), (x,), 'grlex')
+           Poly(x**2 + 2, x)
 
-           >>> Poly(y*x**2 + (y**2+y), x).as_monic()
-           Poly((1, 1 + y), ((2,), (0,)), (x,), 'grlex')
+           >>> Poly(y*x**2 + y**2 + y, x).as_monic()
+           Poly(x**2 + 1 + y, x)
 
         """
         LC = self.lead_coeff
@@ -1094,12 +1247,47 @@ class Poly(Basic, RelMeths, ArithMeths):
         coeffs = [ coeff / LC for coeff in self.coeffs ]
 
         for i, coeff in enumerate(coeffs):
-            if not coeff.is_Atom:
-                coeffs[i] = Poly._cancel(coeff)
+            coeffs[i] = Poly.cancel(coeff)
 
         return self.__class__((coeffs, self.monoms),
             *self.symbols, **self.flags)
 
+    def as_integer(self):
+        """Makes all coefficients integers if possible.
+
+           Given a polynomial with rational coefficients,  returns a tuple
+           consisting of a common denominator of those coefficients and an
+           integer polynomial. Otherwise an exception is being raised.
+
+           >>> from sympy import *
+           >>> x,y = symbols("xy")
+
+           >>> Poly(3*x**2 + x, x).as_integer()
+           (1, Poly(3*x**2 + x, x))
+
+           >>> Poly(3*x**2 + x/2, x).as_integer()
+           (2, Poly(6*x**2 + x, x))
+
+        """
+        denom = 1
+
+        for coeff in self.iter_coeffs():
+            if coeff.is_Rational:
+                if not coeff.is_Integer:
+                    denom = ilcm(denom, coeff.q)
+            else:
+                raise CoefficientError("%s is not a rational number" % coeff)
+
+        denom = sympify(denom)
+
+        if denom is S.One:
+            return denom, self
+        else:
+            coeffs = [ coeff * denom for coeff in self.iter_coeffs() ]
+
+            return denom, self.__class__((coeffs, self.monoms),
+                *self.symbols, **self.flags)
+
     @property
     def content(self):
         """Returns integer GCD of all the coefficients.
@@ -1121,7 +1309,7 @@ class Poly(Basic, RelMeths, ArithMeths):
 
         """
         if self.is_zero:
-            return S.Zero
+            return S.One
         else:
             content = 0
 
@@ -1141,7 +1329,7 @@ class Poly(Basic, RelMeths, ArithMeths):
 
            >>> p = Poly(4*x**2 + 2*x, x)
            >>> p.as_primitive()
-           (2, Poly((2, 1), ((2,), (1,)), (x,), 'grlex'))
+           (2, Poly(2*x**2 + x, x))
 
         """
         content = self.content
@@ -1161,7 +1349,7 @@ class Poly(Basic, RelMeths, ArithMeths):
            >>> x,y = symbols('xy')
 
            >>> Poly((x-1)**2, x).as_squarefree()
-           Poly((1, -1), ((1,), (0,)), (x,), 'grlex')
+           Poly(x - 1, x)
 
         """
         f, A = self, sympy.polys.algorithms
@@ -1169,14 +1357,14 @@ class Poly(Basic, RelMeths, ArithMeths):
         if f.is_univariate:
             F = f.as_primitive()[1]
 
-            h = f.diff(f.symbols[0])
+            h = f.diff()
 
             g = A.poly_gcd(F, h)
             r = A.poly_div(f, g)
 
             return r[0]
         else:
-            raise PolynomialError
+            raise MultivariatePolyError(f)
 
     def as_reduced(self):
         """Remove GCD of monomials from 'self'.
@@ -1185,10 +1373,10 @@ class Poly(Basic, RelMeths, ArithMeths):
            >>> x,y = symbols('xy')
 
            >>> Poly(x**3 + x, x).as_reduced()
-           ((1,), Poly((1, 1), ((2,), (0,)), (x,), 'grlex'))
+           ((1,), Poly(x**2 + 1, x))
 
            >>> Poly(x**3*y+x**2*y**2, x, y).as_reduced()
-           ((2, 1), Poly((1, 1), ((1, 0), (0, 1)), (x, y), 'grlex'))
+           ((2, 1), Poly(x + y, x, y))
 
         """
         if self.is_inhomogeneous:
@@ -1237,7 +1425,8 @@ class Poly(Basic, RelMeths, ArithMeths):
             coeff = f(coeff, *args, **kwargs)
 
             if coeff.has_any_symbols(*self.symbols):
-                raise PolynomialError
+                raise CoefficientError("%s coefficient is dependent" \
+                    " of polynomial's symbols %s" % (coeff, self.symbols))
             elif coeff is not S.Zero:
                 terms.append((coeff, monom))
 
@@ -1337,20 +1526,18 @@ class Poly(Basic, RelMeths, ArithMeths):
            >>> x,y,z = symbols('xyz')
 
            >>> p = Poly(x**2*y + z**2, x, y)
-           >>> print p
-           Poly((1, z**2), ((2, 1), (0, 0)), (x, y), 'grlex')
 
            >>> p.diff(x)
-           Poly((2,), ((1, 1),), (x, y), 'grlex')
+           Poly(2*x*y, x, y)
 
            >>> p.diff(x, 2)
-           Poly((2,), ((0, 1),), (x, y), 'grlex')
+           Poly(2*y, x, y)
 
            >>> p.diff(x, 2, y)
-           Poly((2,), ((0, 0),), (x, y), 'grlex')
+           Poly(2, x, y)
 
            >>> p.diff(z)
-           Poly((2*z,), ((0, 0),), (x, y), 'grlex')
+           Poly(2*z, x, y)
 
         """
         if self.is_zero:
@@ -1369,6 +1556,12 @@ class Poly(Basic, RelMeths, ArithMeths):
             else:
                 raise TypeError
 
+        if not new_symbols:
+            if self.is_univariate:
+                new_symbols = [(self.symbols[0], 1)]
+            else:
+                return self
+
         indices, symbols = {}, self.stamp
 
         for i, s in enumerate(self.symbols):
@@ -1430,20 +1623,18 @@ class Poly(Basic, RelMeths, ArithMeths):
            >>> x,y,z = symbols('xyz')
 
            >>> p = Poly(x**2*y + z**2, x, y)
-           >>> print p
-           Poly((1, z**2), ((2, 1), (0, 0)), (x, y), 'grlex')
 
            >>> p.integrate(x)
-           Poly((1/3, z**2), ((3, 1), (1, 0)), (x, y), 'grlex')
+           Poly(1/3*x**3*y + z**2*x, x, y)
 
            >>> p.integrate(x, 2)
-           Poly((1/12, (1/2)*z**2), ((4, 1), (2, 0)), (x, y), 'grlex')
+           Poly(1/12*x**4*y + (1/2)*z**2*x**2, x, y)
 
            >>> p.integrate(x, 2, y)
-           Poly((1/24, (1/2)*z**2), ((4, 2), (2, 1)), (x, y), 'grlex')
+           Poly(1/24*x**4*y**2 + (1/2)*z**2*x**2*y, x, y)
 
            >>> p.integrate(z)
-           Poly((z, (1/3)*z**3), ((2, 1), (0, 0)), (x, y), 'grlex')
+           Poly(z*x**2*y + (1/3)*z**3, x, y)
 
         """
         if self.is_zero:
@@ -1638,8 +1829,7 @@ class Poly(Basic, RelMeths, ArithMeths):
                 coeffs = [ p_coeff * coeff for p_coeff in self.coeffs ]
 
                 for i, coeff in enumerate(coeffs):
-                    if not coeff.is_Atom:
-                        coeffs[i] = Poly._cancel(coeff)
+                    coeffs[i] = Poly.cancel(coeff)
 
             if monom is None:
                 monoms = self.monoms
@@ -1667,8 +1857,7 @@ class Poly(Basic, RelMeths, ArithMeths):
             coeffs = [ p_coeff / coeff for p_coeff in self.coeffs ]
 
             for i, coeff in enumerate(coeffs):
-                if not coeff.is_Atom:
-                    coeffs[i] = Poly._cancel(coeff)
+                coeffs[i] = Poly.cancel(coeff)
 
         if monom is None:
             monoms = self.monoms
@@ -1677,7 +1866,8 @@ class Poly(Basic, RelMeths, ArithMeths):
                 for p_monom in self.monoms ]
 
             if any(monom is None for monom in monoms):
-                raise PolynomialError
+                raise PolynomialError("%s monomial must divide" \
+                    " exactly the given polynomial" % (monom,))
 
         return self.__class__((coeffs, monoms),
             *self.symbols, **self.flags)
@@ -1728,7 +1918,8 @@ class Poly(Basic, RelMeths, ArithMeths):
         N = len(self.symbols)
 
         if len(point) != N:
-            raise ValueError
+            raise ValueError("Dimension of %s does not match" \
+                " the number of symbols (%d)" % (point, N))
 
         if self.is_univariate:
             terms = self.as_uv_dict()
@@ -1800,14 +1991,16 @@ class Poly(Basic, RelMeths, ArithMeths):
 
         for s, value in pattern:
             if s not in self.stamp:
-                raise PolynomialError
+                raise PolynomialError("%s symbol does" \
+                    " not belong to %s" % (s, self.symbols))
             else:
                 i = symbols.index(s)
 
             # 'value' might be int | long
             if isinstance(value, Symbol):
                 if value in self.stamp:
-                    raise PolynomialError
+                    raise PolynomialError("%s symbol must" \
+                        " not belong to %s" % (s, self.symbols))
 
             terms = {}
 
@@ -1815,23 +2008,29 @@ class Poly(Basic, RelMeths, ArithMeths):
                 monom = M[:i] + M[i+1:]
                 coeff *= value ** M[i]
 
+                coeff = Poly.cancel(coeff)
+
                 if terms.has_key(monom):
                     coeff += terms[monom]
 
                     if not coeff:
                         del terms[monom]
                         continue
+                elif not coeff:
+                    continue
 
                 terms[monom] = coeff
 
             del symbols[i]
             poly = terms
 
-        if len(poly) == 1:
-            return poly.popitem()[1]
+        if not symbols:
+            if not poly:
+                return S.Zero
+            else:
+                return poly.popitem()[1]
         else:
-            return self.__class__(terms,
-                *symbols, **self.flags)
+            return self.__class__(poly, *symbols, **self.flags)
 
     def _eval_subs(self, old, new):
         symbols = list(self.symbols)
@@ -1929,27 +2128,74 @@ class Poly(Basic, RelMeths, ArithMeths):
     def __nonzero__(self):
         return self.coeffs not in ((S.Zero,), (0,))
 
-    def atoms(self, type=None):
-        """Returns the atoms that form a polynomial.
+    def torepr(self):
+        terms = []
 
-           >>> from sympy import *
-           >>> x,y = symbols('xy')
+        for coeff, monom in self.iter_terms():
+            terms.append("(%s, %s)" % (coeff.torepr(), monom))
 
-           >>> Poly(x + 1, x, y).atoms()
-           set([1, x])
+        format = self.__class__.__name__ + "([%s], %s, order='%s')"
 
-        """
-        result = set()
+        symbols = [ s.torepr() for s in self.symbols ]
 
-        for coeff in self.coeffs:
-            result |= coeff.atoms(type)
+        return format % (', '.join(terms),
+            ', '.join(symbols), self.order)
 
-        if type is None or type is Symbol or Symbol in type:
-            for i, symbol in enumerate(self.symbols):
-                if any(monom[i] != 0 for monom in self.monoms):
-                    result.add(symbol)
+    def tostr(self, level=0):
+        terms, symbols = [], [ s.tostr() for s in self.symbols ]
 
-        return result
+        for coeff, monom in self.iter_terms():
+            s_monom = []
+
+            for i, exp in enumerate(monom):
+                if exp > 0:
+                    if exp == 1:
+                        s_monom.append(symbols[i])
+                    else:
+                        s_monom.append(symbols[i] + "**%d" % exp)
+
+            s_monom = "*".join(s_monom)
+
+            if coeff.is_Add:
+                if s_monom:
+                    s_coeff = "(" + coeff.tostr() + ")"
+                else:
+                    s_coeff = coeff.tostr()
+            else:
+                if s_monom and abs(coeff) is S.One:
+                    if coeff.is_negative:
+                        terms.extend(['-', s_monom])
+                    else:
+                        terms.extend(['+', s_monom])
+
+                    continue
+                else:
+                    s_coeff = coeff.tostr()
+
+            if not s_monom:
+                s_term = s_coeff
+            else:
+                s_term = s_coeff + "*" + s_monom
+
+            if s_term.startswith('-'):
+                terms.extend(['-', s_term[1:]])
+            else:
+                terms.extend(['+', s_term])
+
+        if terms[0] in ['-', '+']:
+            modifier = terms.pop(0)
+
+            if modifier == '-':
+                terms[0] = '-' + terms[0]
+
+        format = self.__class__.__name__ + "(%s, %s"
+
+        if self.is_multivariate and self.order != 'grlex':
+            format += ", order='%s')" % self.order
+        else:
+            format += ")"
+
+        return format % (' '.join(terms), ', '.join(symbols))
 
     def _eval_is_polynomial(self, symbols):
         try:
