@@ -1,9 +1,10 @@
 
-from sympy import symbols, expand, sin, sqrt, re, im, I, Rational
+from sympy import symbols, expand, sin, sqrt, re, im, I, Rational, Lambda, powsimp
 
 from sympy.polys.monomial import *
 from sympy.polys.polynomial import *
 from sympy.polys.algorithms import *
+from sympy.polys.rootfinding import *
 
 import py
 
@@ -85,6 +86,24 @@ def test_poly_basics():
     assert Poly({(4,): x, (2,): y, (0,): z}, t) == Poly(f, t)
     assert Poly({4: x, 2: y, 0: z}, t) == Poly(f, t)
 
+    assert Poly(Poly(x, x)) == Poly(x, x)
+
+    assert Poly(Poly(x*y, x, y), order='lex') == \
+        Poly(x*y, x, y, order='lex')
+
+    py.test.raises(SymbolsError, "Poly(x, 2)")
+    py.test.raises(SymbolsError, "Poly(x, 2*x)")
+
+    py.test.raises(SymbolsError, "Poly(x, 2, x)")
+    py.test.raises(SymbolsError, "Poly(x, 2*x, x)")
+
+    py.test.raises(SymbolsError, "Poly(x, x, 2)")
+    py.test.raises(SymbolsError, "Poly(x, x, 2*x)")
+
+    A, B = symbols('AB', commutative=False)
+
+    py.test.raises(SymbolsError, "Poly(x + A**2 + B**2, x, A, B)")
+
 def test_poly_internals():
     p = Poly(x**2*y*z + x*y*z**3 + x*y + y*z, x, y, z)
 
@@ -126,16 +145,84 @@ def test_poly_internals():
     assert Poly._permute(q, z, y, x) == \
         {(0, 1, 1): 3, (1, 1, 0): 4, (3, 1, 1): 2, (1, 1, 2): 1}
 
+def test_poly_repr():
+    assert Poly(7, x).torepr() == \
+        "Poly([(Integer(7), (0,))], Symbol('x'), order='grlex')"
+
+    assert Poly(2*x*y + 7, x, y).torepr() == \
+        "Poly([(Integer(2), (1, 1)), (Integer(7), (0, 0))]," \
+        " Symbol('x'), Symbol('y'), order='grlex')"
+
+    assert Poly(2*x*y - 7, x, y, order='grevlex').torepr() == \
+        "Poly([(Integer(2), (1, 1)), (Integer(-7), (0, 0))]," \
+        " Symbol('x'), Symbol('y'), order='grevlex')"
+
+    assert Poly(0, x).tostr() == "Poly(0, x)"
+
+    assert Poly(1, x).tostr() == "Poly(1, x)"
+    assert Poly(x, x).tostr() == "Poly(x, x)"
+
+    assert Poly(2*x + 1, x).tostr() == "Poly(2*x + 1, x)"
+    assert Poly(2*x - 1, x).tostr() == "Poly(2*x - 1, x)"
+
+    assert Poly(-1, x).tostr() == "Poly(-1, x)"
+    assert Poly(-x, x).tostr() == "Poly(-x, x)"
+
+    assert Poly(-2*x + 1, x).tostr() == "Poly(-2*x + 1, x)"
+    assert Poly(-2*x - 1, x).tostr() == "Poly(-2*x - 1, x)"
+
+    assert Poly(x - 1, x, order='lex').tostr() == "Poly(x - 1, x)"
+
+    assert Poly(x**2 + 1 + y, x).tostr() == "Poly(x**2 + 1 + y, x)"
+    assert Poly(x**2 - 1 + y, x).tostr() in [
+            "Poly(x**2 - 1 + y, x)",
+            "Poly(x**2 + (-1) + y, x)",
+            ]
+
+    assert Poly(-x*y*z + x*y - 1, x, y, z).tostr() == "Poly(-x*y*z + x*y - 1, x, y, z)"
+
+    assert Poly(-t*x**21*y**7*z + (1 + t)*z**3 - 2*x*z + 1, x, y, z).tostr() == \
+        "Poly(-t*x**21*y**7*z + (1 + t)*z**3 - 2*x*z + 1, x, y, z)"
+
+    assert Poly(x*y*z**2 - 27*x, x, y, z, order='lex').tostr() == \
+        "Poly(x*y*z**2 - 27*x, x, y, z, order='lex')"
+    assert Poly(x*y*z**2 - 27*x, x, y, z, order='grlex').tostr() == \
+        "Poly(x*y*z**2 - 27*x, x, y, z)"
+    assert Poly(x*y*z**2 - 27*x, x, y, z, order='grevlex').tostr() == \
+        "Poly(x*y*z**2 - 27*x, x, y, z, order='grevlex')"
+
 def test_poly_cancel():
-    assert Poly._cancel(x) == x
-    assert Poly._cancel(x+1) == x+1
-    assert Poly._cancel((x+1)/(x-1)) == (x+1)/(x-1)
+    assert Poly.cancel(x) == x
+    assert Poly.cancel(x+1) == x+1
+    assert Poly.cancel((x+1)/(1-x)) == (x+1)/(1-x)
 
-    assert Poly._cancel((x**2-1)/(x-1)) == x+1
-    assert Poly._cancel((x**2-y**2)/(x-y)) == x+y
+    assert Poly.cancel((x**2-1)/(x-1)) == x+1
+    assert Poly.cancel((x**2-y**2)/(x-y)) == x+y
 
-    assert Poly._cancel((x**2-y)/(x-y)) == (x**2 - y)/(x - y)
-    assert Poly._cancel((x**2-2)/(x+sqrt(2))) == x - sqrt(2)
+    assert Poly.cancel((x**2-y)/(x-y)) == (x**2 - y)/(x - y)
+    assert Poly.cancel((x**2-2)/(x+sqrt(2))) == x - sqrt(2)
+
+    assert Poly.cancel((x**2-y**2)/(x-y), x) == x+y
+
+    assert Poly.cancel((x, S.One), x) == x
+    assert Poly.cancel((x+1, S.One), x) == x+1
+    assert Poly.cancel((x+1, x-1), x) == (x+1)/(x-1)
+
+    assert Poly.cancel((x**2-1, x-1), x) == x+1
+    assert Poly.cancel((x**2-y**2, x-y), x, y) == x+y
+
+    assert Poly.cancel((x**2-y, x-y), x, y) == (x**2 - y)/(x - y)
+    assert Poly.cancel((x**2-2, x+sqrt(2)), x) == x - sqrt(2)
+
+    assert Poly.cancel((x**2-y**2, x-y), x) == x+y
+
+    assert Poly.cancel(((x**2-y**2).as_poly(x), (x-y).as_poly(x))) == x+y
+
+    f = -1/(3 + 2*sqrt(2))*(1 + 1/(3 + 2*sqrt(2))*(7 + 5*sqrt(2)))
+
+    assert Poly.cancel(f) == -2 + sqrt(2)
+
+    py.test.raises(SymbolsError, "Poly.cancel((x**2-y**2, x-y))")
 
 def test_poly_characteristics():
     f = -3*x**5*y*z**4 + 2*x**2*y**8 - x*y**4 + x*y*z**3
@@ -151,6 +238,10 @@ def test_poly_characteristics():
     assert p.LM == (5, 1, 4)
     assert p.LC == -3
 
+    assert p.TT == (1, (1, 1, 3))
+    assert p.TM == (1, 1, 3)
+    assert p.TC == 1
+
     assert p.coeffs == (-3,2,-1,1)
     assert p.monoms == ((5,1,4), (2,8,0), (1,4,0), (1,1,3))
 
@@ -165,6 +256,10 @@ def test_poly_characteristics():
     assert q.LM == (5, 1, 4)
     assert q.LC == -3
 
+    assert p.TT == (1, (1, 1, 3))
+    assert p.TM == (1, 1, 3)
+    assert p.TC == 1
+
     assert q.coeffs == (-3,2,-1,1)
     assert q.monoms == ((5,1,4), (2,8,0), (1,4,0), (1,1,3))
 
@@ -179,6 +274,10 @@ def test_poly_characteristics():
     assert r.LM == (2, 8, 0)
     assert r.LC == 2
 
+    assert p.TT == (1, (1, 1, 3))
+    assert p.TM == (1, 1, 3)
+    assert p.TC == 1
+
     assert r.coeffs == (2,-3,-1,1)
     assert r.monoms == ((2,8,0), (5,1,4), (1,4,0), (1,1,3))
 
@@ -274,6 +373,26 @@ def test_poly_properties():
     assert f.is_inhomogeneous == True
     assert f.is_monic == False
 
+    assert Poly(x + y - 1, x).is_linear == True
+    assert Poly(x + y - 1, y).is_linear == True
+
+    assert Poly(x + y - 1, x, y).is_linear == True
+
+    assert Poly(x*x + y - 1, x, y).is_linear == False
+    assert Poly(x*y + y - 1, x, y).is_linear == False
+
+def test_iterators():
+    f = Poly(x**5 + x, x)
+
+    assert list(f.iter_all_coeffs()) == \
+        [1, 0, 0, 0, 1, 0]
+
+    assert list(f.iter_all_monoms()) == \
+        [(5,), (4,), (3,), (2,), (1,), (0,)]
+
+    assert list(f.iter_all_terms()) == \
+        [(1, (5,)), (0, (4,)), (0, (3,)), (0, (2,)), (1, (1,)), (0, (0,))]
+
 def test_as_monic():
     assert Poly(0, x,y,z).as_monic() == Poly(0, x,y,z)
     assert Poly(1, x,y,z).as_monic() == Poly(1, x,y,z)
@@ -298,6 +417,15 @@ def test_as_monic():
 
     assert Poly(f, x).as_monic() == Poly(g, x)
 
+def test_as_integer():
+    assert Poly(3*x**2 + x, x).as_integer() == \
+        (Integer(1), Poly(3*x**2 + x, x))
+
+    assert Poly(x**2 + x/2, x).as_integer() == \
+        (Integer(2), Poly(2*x**2 + x, x))
+
+    py.test.raises(CoefficientError, "Poly(x**2 + t*x, x).as_integer()")
+
 def test_poly_add():
     f = -Rational(1,6)*x**2-Rational(5,36)+Rational(17,18)
     g = -Rational(1,6)*x**2-Rational(5,36)+Rational(35,18)
@@ -375,12 +503,12 @@ def test_poly_div():
     assert poly_div(x*y+2*x+y,x,x) == (Poly(2+y, x), Poly(y, x))
 
     assert poly_div(x*y**2 + 1, [x*y+1, y+1], x, y) == \
-        ((Poly(y, x, y), Poly(-1, x, y)), Poly(2, x, y))
+        ([Poly(y, x, y), Poly(-1, x, y)], Poly(2, x, y))
 
     assert poly_div(x**2*y+x*y**2+y**2, [x*y-1, y**2-1], x, y) == \
-        ((Poly(x+y, x, y), Poly(1, x, y)), Poly(1+x+y, x, y))
+        ([Poly(x+y, x, y), Poly(1, x, y)], Poly(1+x+y, x, y))
     assert poly_div(x**2*y+x*y**2+y**2, [y**2-1, x*y-1], x, y) == \
-        ((Poly(1+x, x, y), Poly(x, x, y)), Poly(1+2*x, x, y))
+        ([Poly(1+x, x, y), Poly(x, x, y)], Poly(1+2*x, x, y))
 
     f, g = 3*x**3 + x**2 + x + 5, 5*x**2 - 3*x + 1
 
@@ -428,6 +556,17 @@ def test_poly_lcm():
         Poly(-3*x*y**2/2+x*y**3+3*x*y**4-2*x*y**5-3*x*y**6/2+x*y**7, x, y)
 
 def test_poly_gcd():
+    assert poly_gcd(0, 0, x) == Poly(0, x)
+    assert poly_gcd(0, 1, x) == Poly(1, x)
+    assert poly_gcd(1, 0, x) == Poly(1, x)
+    assert poly_gcd(1, 1, x) == Poly(1, x)
+
+    assert poly_gcd(x-1, 0, x) == Poly(x-1, x)
+    assert poly_gcd(0, x-1, x) == Poly(x-1, x)
+
+    assert poly_gcd(-x-1, 0, x) == Poly(x+1, x)
+    assert poly_gcd(0, -x-1, x) == Poly(x+1, x)
+
     assert poly_gcd(2, 6, x) == Poly(2, x)
     assert poly_gcd(2, 6, x, y) == Poly(2, x, y)
 
@@ -494,33 +633,33 @@ def test_poly_subresultants():
     g = 3*x**6+5*x**4-4*x**2-9*x+21
 
     assert poly_subresultants(f, g, x) == \
-        (Poly(f, x), Poly(g, x),
+        [Poly(f, x), Poly(g, x),
          Poly(15*x**4 - 3*x**2 + 9,  x),
          Poly(65*x**2 + 125*x - 245, x),
          Poly(9326*x - 12300, x),
-         Poly(260708, x))
+         Poly(260708, x)]
 
     assert poly_subresultants((x-1)**2, x**2-1, x) == \
-        (Poly((x-1)**2, x), Poly(x**2-1, x), Poly(2*x - 2, x))
+        [Poly((x-1)**2, x), Poly(x**2-1, x), Poly(2*x - 2, x)]
 
 def test_poly_groebner():
-    assert poly_groebner(0, x) == (Poly((), x),)
+    assert poly_groebner(0, x) == [Poly((), x)]
 
-    assert poly_groebner(x*y, x) == (Poly(x, x),)
-    assert poly_groebner(x*y, z) == (Poly(1, z),)
+    assert poly_groebner(x*y, x) == [Poly(x, x)]
+    assert poly_groebner(x*y, z) == [Poly(1, z)]
 
     assert poly_groebner((x**2 + 2*x*y**2, x*y + 2*y**3 - 1), y, x, order='lex') == \
-        (Poly(y**3 - Rational(1,2), y, x, order='lex'),
-         Poly(x, y, x, order='lex'))
+        [Poly(y**3 - Rational(1,2), y, x, order='lex'),
+         Poly(x, y, x, order='lex')]
 
     assert poly_groebner((y-x**2, z-x**3), y, z, x, order='lex') == \
-        (Poly(-x**2+y, y, z, x, order='lex'),
-         Poly(z-x**3, y, z, x, order='lex'))
+        [Poly(-x**2+y, y, z, x, order='lex'),
+         Poly(z-x**3, y, z, x, order='lex')]
 
     assert poly_groebner((x**3-2*x*y, x**2*y-2*y**2+x), x, y, order='grlex') == \
-        (Poly(x**2, x, y, order='grlex'),
+        [Poly(x**2, x, y, order='grlex'),
          Poly(x*y, x, y, order='grlex'),
-         Poly(y**2-x/2, x, y, order='grlex'))
+         Poly(y**2-x/2, x, y, order='grlex')]
 
 def test_map_coeffs():
     p = Poly(x**2 + 2*x*y, x, y)
@@ -606,7 +745,7 @@ def test_call():
     assert Poly(f, x, y, z)(u, v, t) == u**2*(v*(u + t) + t)
 
 def test_content():
-    assert Poly(0, x, y, z).content == 0
+    assert Poly(0, x, y, z).content == 1
     assert Poly(1, x, y, z).content == 1
 
     assert Poly(2*x + 5*x*y, x, y).content == 1
@@ -614,7 +753,7 @@ def test_content():
     assert Poly(2*x + z*x*y, x, y).content == 1
 
 def test_primitive():
-    assert Poly(0, x, y, z).as_primitive() == (0, Poly(0, x, y, z))
+    assert Poly(0, x, y, z).as_primitive() == (1, Poly(0, x, y, z))
     assert Poly(1, x, y, z).as_primitive() == (1, Poly(1, x, y, z))
 
     assert Poly(2*x + 5*x*y, x, y).as_primitive() == (1, Poly(2*x + 5*x*y, x, y))
@@ -636,34 +775,34 @@ def test_squarefree():
     assert Poly(x**2+2*x+1, x).as_squarefree() == Poly(x+1, x)
     assert Poly(x**5-x**4-x+1, x).as_squarefree() == Poly(x**4-1, x)
 
-    assert poly_sqf(1, x) == (Poly(1, x),)
-    assert poly_sqf(x, x) == (Poly(x, x),)
+    assert poly_sqf(1, x) == [Poly(1, x)]
+    assert poly_sqf(x, x) == [Poly(x, x)]
 
-    assert poly_sqf(3*x**2, x) == (Poly(3, x), Poly(x, x))
-    assert poly_sqf(x**2+2*x+1, x) == (Poly(1, x), Poly(x+1, x))
+    assert poly_sqf(3*x**2, x) == [Poly(3, x), Poly(x, x)]
+    assert poly_sqf(x**2+2*x+1, x) == [Poly(1, x), Poly(x+1, x)]
 
     assert poly_sqf(x**5-x**4-x+1, x) == \
-        (Poly(x**3 + x**2 + x + 1, x), Poly(x-1, x))
+        [Poly(x**3 + x**2 + x + 1, x), Poly(x-1, x)]
     assert poly_sqf(x**8+6*x**6+12*x**4+8*x**2, x) == \
-        (Poly(1, x), Poly(x, x), Poly(x**2+2, x))
+        [Poly(1, x), Poly(x, x), Poly(x**2+2, x)]
 
 def test_decompose():
-    assert poly_decompose(1, x) == (Poly(1, x),)
-    assert poly_decompose(x, x) == (Poly(x, x),)
+    assert poly_decompose(1, x) == [Poly(1, x)]
+    assert poly_decompose(x, x) == [Poly(x, x)]
 
-    assert poly_decompose(z*x**3, x) == (Poly(z*x**3, x),)
-    assert poly_decompose(z*x**5, x) == (Poly(z*x**5, x),)
+    assert poly_decompose(z*x**3, x) == [Poly(z*x**3, x)]
+    assert poly_decompose(z*x**5, x) == [Poly(z*x**5, x)]
 
-    assert poly_decompose(x**4, x) == (Poly(x**2, x), Poly(x**2, x))
-    assert poly_decompose(z*x**4+1, x) == (Poly(z*x**2+1, x), Poly(x**2, x))
+    assert poly_decompose(x**4, x) == [Poly(x**2, x), Poly(x**2, x)]
+    assert poly_decompose(z*x**4+1, x) == [Poly(z*x**2+1, x), Poly(x**2, x)]
 
-    assert poly_decompose(x**4+2*x**2+z, x) == (Poly(x**2+2*x+z, x), Poly(x**2, x))
+    assert poly_decompose(x**4+2*x**2+z, x) == [Poly(x**2+2*x+z, x), Poly(x**2, x)]
 
     f, g = x**4 - 2*x + z, x**3 + 5*x
 
-    assert poly_decompose(f.subs(x, g), x) == (Poly(f, x), Poly(g, x))
-    assert poly_decompose(2*f.subs(x, g), x) == (Poly(2*f, x), Poly(g, x))
-    assert poly_decompose(f.subs(x, g-2), x) == (Poly(f.subs(x, x-2), x), Poly(g, x))
+    assert poly_decompose(f.subs(x, g), x) == [Poly(f, x), Poly(g, x)]
+    assert poly_decompose(2*f.subs(x, g), x) == [Poly(2*f, x), Poly(g, x)]
+    assert poly_decompose(f.subs(x, g-2), x) == [Poly(f.subs(x, x-2), x), Poly(g, x)]
 
 def test_evaluate():
     f = x**2*y*z + 2*x*y*z**3 + 3*x*y + 4*y*z
@@ -674,13 +813,26 @@ def test_evaluate():
     assert p.evaluate({x: 7, y: 5}) == Poly(f.subs({x: 7, y: 5}), z)
     assert p.evaluate({x: 7, y: 5, z: 4}) == f.subs({x: 7, y: 5, z: 4})
 
+    p = Poly(x**2 + x*y*sin(z), x, y, order='lex')
+
+    assert p.evaluate({x: 3, y: 0}) == 9
+    assert p.evaluate({x: 0, y: 0}) == 0
+
+    q = p.evaluate({x: 0})
+
+    assert q == Poly(0, y, order='lex')
+    assert q.is_zero == True
+
+    assert p.evaluate({y: 0}) == \
+        Poly(x**2, x, order='lex')
+
     py.test.raises(PolynomialError, "Poly(x + y, x, y).evaluate({x: y})")
 
 def test_subs():
     p = Poly(t*x*y**2 + x*y + t**2, x, y)
 
     assert p.subs(x, 2) == \
-        Poly(((2*t, 2, t**2), ((2,), (1,), (0,))), y, 'grlex')
+        Poly(((2*t, 2, t**2), ((2,), (1,), (0,))), y)
     assert p.subs(y, 2) == \
         Poly(((2 + 4*t, t**2), ((1,), (0,))), x)
 
@@ -727,6 +879,25 @@ def test_unify():
     assert p.unify_with((q, r)) == p.unify_with((x**2+2*x*y+1, r)) == \
         (Poly(x**2+x*y, x, y, z), (Poly(x**2+2*x*y+1, x,y,z), Poly(x*z+y*z+1, x,y,z)))
 
+def test_diff():
+    f = Poly(a*x**2 + b*x + 2, x)
+
+    assert f.diff(x) == Poly(2*a*x + b, x)
+    assert f.diff(y) == Poly(0, x)
+    assert f.diff(a) == Poly(x**2, x)
+    assert f.diff(b) == Poly(x, x)
+
+    assert f.diff() == Poly(2*a*x + b, x)
+
+    g = Poly(a*x**2 + b*x*y + 2, x, y)
+
+    assert g.diff(x) == Poly(2*a*x + b*y, x, y)
+    assert g.diff(y) == Poly(b*x, x, y)
+    assert g.diff(a) == Poly(x**2, x, y)
+    assert g.diff(b) == Poly(x*y, x, y)
+
+    assert g.diff() == g
+
 def test_eq_ne():
     p = Poly(x**2+x*y, x, y)
     q = Poly(x**2+x*y+1, x, y)
@@ -767,14 +938,232 @@ def test_nonzero():
     assert bool(Poly(102, x, y, z)) == True
     assert bool(Poly(x+21*y, x, y)) == True
 
-def test_atoms():
-    assert Poly(0, x).atoms() == set([S.Zero])
-    assert Poly(1, x).atoms() == set([S.One])
-    assert Poly(x, x).atoms() == set([S.One, x])
-    assert Poly(x, x, y).atoms() == set([S.One, x])
-    assert Poly(x + y, x, y).atoms() == set([S.One, x, y])
-    assert Poly(x + y, x, y, z).atoms() == set([S.One, x, y])
-    assert Poly(x + y*t, x, y, z).atoms() == set([S.One, x, t, y])
+def test_sturm():
+    assert poly_sturm(5, x) == [Poly(5, x)]
+    assert poly_sturm(x, x) == [Poly(x, x), Poly(1, x)]
 
-    assert Poly(x+1, x).atoms(type=Symbol) == set([x])
-    assert Poly(x+1, x).atoms(type=(Symbol,)) == set([x])
+    assert poly_sturm(x**3-2*x**2+3*x-5, x) == \
+        [Poly(x**3-2*x**2+3*x-5, x), Poly(3*x**2-4*x+3, x),
+         Poly(-Rational(10,9)*x+Rational(13,3), x),
+         Poly(Rational(-3303,100), x)]
+
+def test_number_of_real_roots():
+    f = Poly(x - 1, x)
+
+    assert number_of_real_roots(f) == 1
+    assert number_of_real_roots(f, sup=0) == 0
+    assert number_of_real_roots(f, inf=1) == 0
+    assert number_of_real_roots(f, sup=0, inf=1) == 1
+
+    assert number_of_real_roots(f, sup=1, inf=0) == 1
+
+    f = x**2 - 4
+
+    assert number_of_real_roots(f, x) == 2
+    assert number_of_real_roots(f, x, sup=0) == 1
+    assert number_of_real_roots(f, x, inf=-1, sup=1) == 0
+
+    py.test.raises(ValueError, "number_of_real_roots(f, x, inf=t)")
+    py.test.raises(ValueError, "number_of_real_roots(f, x, sup=t)")
+
+def test_roots():
+    assert roots_linear(Poly(2*x+1, x)) == [-Rational(1, 2)]
+
+    assert roots_quadratic(Poly(2*x**2, x)) == [0, 0]
+    assert roots_quadratic(Poly(2*x**2+3*x, x)) == [0, -Rational(3, 2)]
+    assert roots_quadratic(Poly(2*x**2+3, x)) == [I*sqrt(6)/2, -I*sqrt(6)/2]
+
+    assert roots_quadratic(Poly(2*x**2+4*x+3, x)) == \
+        [-1 + I*sqrt(2)/2, -1 - I*sqrt(2)/2]
+
+    assert roots_cubic(Poly(2*x**3, x)) == [0, 0, 0]
+    assert roots_cubic(Poly(x**3-3*x**2+3*x-1, x)) == [1, 1, 1]
+
+    assert roots_cubic(Poly(x**3+1, x)) == \
+        [-1, S.Half - I*sqrt(3)/2, S.Half + I*sqrt(3)/2]
+
+    assert roots_binomial(Poly(5*x, x)) == [0]
+    assert roots_binomial(Poly(5*x**4, x)) == [0, 0, 0, 0]
+    assert roots_binomial(Poly(5*x+2, x)) == [-Rational(2, 5)]
+
+    A = 10**Rational(3, 4)/10
+
+    assert roots_binomial(Poly(5*x**4+2, x)) == \
+        [A+I*A, -A+I*A, -A-I*A, A-I*A]
+
+    r0 = roots_quadratic(Poly(a*x**2 + b, x))
+    r1 = roots_binomial(Poly(a*x**2 + b, x))
+
+    assert powsimp(r0[0]) == powsimp(r1[0])
+    assert powsimp(r0[1]) == powsimp(r1[1])
+
+    assert roots_rational(Poly(x**2-1, x)) == [S.One, -S.One]
+    assert roots_rational(Poly(x**2-x, x)) == [S.Zero, S.One]
+
+    assert roots_rational(Poly(x**2-x/2, x)) == [S.Zero]
+    assert roots_rational(Poly(2*x**2-x, x)) == [S.Zero]
+
+    assert roots_rational(Poly(t*x**2-x, x)) == []
+
+    assert roots(1, x) == {}
+    assert roots(x, x) == {S.Zero: 1}
+    assert roots(x**9, x) == {S.Zero: 9}
+
+    assert roots(2*x+1, x) == {-S.Half: 1}
+    assert roots((2*x+1)**2, x) == {-S.Half: 2}
+    assert roots((2*x+1)**5, x) == {-S.Half: 5}
+    assert roots((2*x+1)**10, x) == {-S.Half: 10}
+
+    assert roots(x**4 - 1, x) == {I: 1, S.One: 1, -S.One: 1, -I: 1}
+    assert roots((x**4 - 1)**2, x) == {I: 2, S.One: 2, -S.One: 2, -I: 2}
+
+    assert roots(((2*x-3)**2).expand(), x) == { Rational(3,2): 2}
+    assert roots(((2*x+3)**2).expand(), x) == {-Rational(3,2): 2}
+
+    assert roots(((2*x-3)**3).expand(), x) == { Rational(3,2): 3}
+    assert roots(((2*x+3)**3).expand(), x) == {-Rational(3,2): 3}
+
+    assert roots(((2*x-3)**5).expand(), x) == { Rational(3,2): 5}
+    assert roots(((2*x+3)**5).expand(), x) == {-Rational(3,2): 5}
+
+    assert roots(((a*x-b)**5).expand(), x) == { b/a: 5}
+    assert roots(((a*x+b)**5).expand(), x) == {-b/a: 5}
+
+    assert roots(x**4-2*x**2+1, x) == {S.One: 2, -S.One: 2}
+
+    assert roots(x**6-4*x**4+4*x**3-x**2, x) == \
+        {S.One: 2, -1 - sqrt(2): 1, S.Zero: 2, -1 + sqrt(2): 1}
+
+    assert roots(x**8-1, x) == {
+         2**S.Half/2 + I*2**S.Half/2: 1,
+         2**S.Half/2 - I*2**S.Half/2: 1,
+        -2**S.Half/2 + I*2**S.Half/2: 1,
+        -2**S.Half/2 - I*2**S.Half/2: 1,
+        S.One: 1, -S.One: 1, I: 1, -I: 1
+    }
+
+    assert roots(x**3+x**2-x+1, x, cubics=False) == {}
+
+    r1_2, r1_3, r1_9, r4_9, r19_27 = [ Rational(*r) \
+        for r in ((1,2), (1,3), (1,9), (4,9), (19,27)) ]
+
+    assert roots(x**3+x**2-x+1, x, cubics=True) in [
+            {
+        -r1_3 - (r19_27 + r1_9*3**r1_2*11**r1_2)**r1_3 - \
+        r4_9*(r19_27 + r1_9*3**r1_2*11**r1_2)**(-r1_3): 1,
+
+        -r1_3 + r1_2*(r19_27 + r1_9*3**r1_2*11**r1_2)**r1_3 + \
+        r4_9/(r1_2 + r1_2*I*3**r1_2)*(r19_27 + r1_9*3**r1_2*11**r1_2)**(-r1_3) + \
+        r1_2*I*3**r1_2*(r19_27 + r1_9*3**r1_2*11**r1_2)**r1_3: 1,
+
+        -r1_3 + r1_2*(r19_27 + r1_9*3**r1_2*11**r1_2)**r1_3 + \
+        r4_9/(r1_2 - r1_2*I*3**r1_2)*(r19_27 + r1_9*3**r1_2*11**r1_2)**(-r1_3) - \
+        r1_2*I*3**r1_2*(r19_27 + r1_9*3**r1_2*11**r1_2)**r1_3: 1,
+            },
+            {
+        -r1_3 - (r19_27 + r1_9*3**r1_2*11**r1_2)**r1_3 - \
+        r4_9*(r19_27 + r1_9*3**r1_2*11**r1_2)**(-r1_3): 1,
+
+        -r1_3 + r1_2*(r19_27 + r1_9*3**r1_2*11**r1_2)**r1_3 - \
+        r4_9/(-r1_2 - r1_2*I*3**r1_2)*(r19_27 + r1_9*3**r1_2*11**r1_2)**(-r1_3) + \
+        r1_2*I*3**r1_2*(r19_27 + r1_9*3**r1_2*11**r1_2)**r1_3: 1,
+
+        -r1_3 + r1_2*(r19_27 + r1_9*3**r1_2*11**r1_2)**r1_3 + \
+        r4_9/(r1_2 - r1_2*I*3**r1_2)*(r19_27 + r1_9*3**r1_2*11**r1_2)**(-r1_3) - \
+        r1_2*I*3**r1_2*(r19_27 + r1_9*3**r1_2*11**r1_2)**r1_3: 1,
+            },
+            ]
+
+    f = (x**2+2*x+3).subs(x, 2*x**2 + 3*x).subs(x, 5*x-4)
+
+    r1_2, r2_25, r13_20, r1_100 = [ Rational(*r) \
+        for r in ((1,2), (2,25), (13,20), (1,100)) ]
+
+    assert roots(f, x) == {
+        r13_20 + r1_2*(r1_100 - r2_25*I*2**r1_2)**r1_2: 1,
+        r13_20 - r1_2*(r1_100 - r2_25*I*2**r1_2)**r1_2: 1,
+        r13_20 + r1_2*(r1_100 + r2_25*I*2**r1_2)**r1_2: 1,
+        r13_20 - r1_2*(r1_100 + r2_25*I*2**r1_2)**r1_2: 1,
+    }
+
+    p = Poly(z**3 + (-2 - y)*z**2 + (1 + 2*y - 2*x**2)*z - y + 2*x**2, z)
+
+    assert roots(p) == {
+        S.One: 1,
+        S.Half + S.Half*y + S.Half*(1 - 2*y + y**2 + 8*x**2)**S.Half: 1,
+        S.Half + S.Half*y - S.Half*(1 - 2*y + y**2 + 8*x**2)**S.Half: 1,
+    }
+
+    assert roots(x**3 + 2*x**2 + 4*x + 8, x) == {}
+    assert roots(x**3 + 2*x**2 + 4*x + 8, x, cubics=True) == \
+        {-2*I: 1, 2*I: 1, -Integer(2): 1}
+
+    assert roots(a*b*c*x**3 + 2*x**2 + 4*x + 8, x) == {}
+    assert roots(a*b*c*x**3 + 2*x**2 + 4*x + 8, x, cubics=True) != {}
+
+    assert roots(x**4-1, x, domain='Z') == {S.One: 1, -S.One: 1}
+    assert roots(x**4-1, x, domain='I') == {I: 1, -I: 1}
+
+    assert roots((x-1)*(x+1), x) == {S.One: 1, -S.One: 1}
+    assert roots((x-1)*(x+1), x, predicate=lambda r: r.is_positive) == {S.One: 1}
+
+    assert roots(x**4-1, x, domain='Z', multiple=True) == [S.One, -S.One]
+    assert roots(x**4-1, x, domain='I', multiple=True) in ([I, -I], [-I, I])
+
+    assert roots(x**3, x, multiple=True) == [S.Zero, S.Zero, S.Zero]
+    assert roots(1234, x, multiple=True) == []
+
+def test_factors():
+    assert poly_factors(1, x) == [Poly(1, x)]
+    assert poly_factors(x, x) == [Poly(x, x)]
+
+    assert poly_factors(x**2-1, x) == [Poly(x-1, x), Poly(x+1, x)]
+
+    assert set( poly_factors((x**4 - 1)**2, x) ) == \
+    set([Poly(((S.One, -I),     ((1,), (0,))), x),
+         Poly(((S.One, -I),     ((1,), (0,))), x),
+         Poly(((S.One, -S.One), ((1,), (0,))), x),
+         Poly(((S.One, -S.One), ((1,), (0,))), x),
+         Poly(((S.One, S.One),  ((1,), (0,))), x),
+         Poly(((S.One, S.One),  ((1,), (0,))), x),
+         Poly(((S.One, I),      ((1,), (0,))), x),
+         Poly(((S.One, I),      ((1,), (0,))), x)])
+
+    assert poly_factors(x**4-1, x, domain='Z') == \
+        [Poly(x-1, x), Poly(x+1, x), Poly(x**2+1, x)]
+
+def test_RootsOf():
+    f = Poly((x-4)**4, x)
+
+    roots = RootsOf(f)
+
+    assert roots.count == 4
+
+    assert list(roots.roots()) == [ Integer(4),
+        Integer(4), Integer(4), Integer(4) ]
+
+    assert RootSum(lambda r: r**2, f) == 64
+
+    roots = RootsOf(x**5+x+1, x)
+
+    assert roots.count == 5
+
+    f = Poly(x**5+x+1, x)
+
+    assert list(roots.roots()) == [ RootOf(f, 0), RootOf(f, 1),
+        RootOf(f, 2), RootOf(f, 3), RootOf(f, 4) ]
+
+    assert RootSum(lambda r: r**2, f).doit() == RootOf(f, 0)**2 + \
+        RootOf(f, 1)**2 + RootOf(f, 2)**2 + RootOf(f, 3)**2 + RootOf(f, 4)**2
+
+    assert RootSum(Lambda(x, x), Poly(0, x), evaluate=True)  == S.Zero
+    assert RootSum(Lambda(x, x), Poly(0, x), evaluate=False) != S.Zero
+
+    f = Poly(x**17 + x - 1, x)
+
+    assert RootsOf(f).tostr() == "RootsOf(x**17 + x - 1, x)"
+
+    assert RootOf(f, 0).tostr() == "RootOf(x**17 + x - 1, x, index=0)"
+
+    assert RootSum(Lambda(z, z**2), f).tostr() == \
+        "RootSum(Lambda(_z, _z**2), x**17 + x - 1, x)"
