@@ -3,6 +3,9 @@
 from pretty import *
 from latex import latex, print_latex
 from mathml import mathml, print_mathml
+from python import python, print_python
 from gtk import *
 
 from preview import preview, view, pngview, pdfview, dviview
+
+from tree import print_tree
