@@ -1,4 +1,4 @@
-from sympy import Basic
+from sympy import Basic, sympify
 from printer import Printer
 
 
@@ -31,8 +31,6 @@ class MathMLPrinter(Printer):
 
         # Not found in the MRO set
         n = e.__class__.__name__
-        if n.startswith('Apply'):
-            n = n[5:]
         return n.lower()
 
     def _print_Matrix(self, m):
@@ -112,7 +110,7 @@ class MathMLPrinter(Printer):
     def _print_Function(self, e):
         x = self.dom.createElement("apply")
         x.appendChild(self.dom.createElement(self.mathml_tag(e)))
-        for arg in e:
+        for arg in e.args:
             x.appendChild(self._print(arg))
         return x
 
@@ -126,14 +124,14 @@ class MathMLPrinter(Printer):
         x = self.dom.createElement('apply')
         x_1 = self.dom.createElement(self.mathml_tag(e))
         x.appendChild(x_1)
-        for arg in e:
+        for arg in e.args:
             x.appendChild(self._print(arg))
         return x
 
 def mathml(expr):
     """Returns the MathML representation of expr"""
     s = MathMLPrinter()
-    return s.doprint(Basic.sympify(expr))
+    return s.doprint(sympify(expr))
 
 def print_mathml(expr):
     """
@@ -145,12 +143,12 @@ def print_mathml(expr):
     >>> print_mathml(x+1) #doctest: +NORMALIZE_WHITESPACE
     <apply>
         <plus/>
-        <cn>
-                1
-        </cn>
         <ci>
                 x
         </ci>
+        <cn>
+                1
+        </cn>
     </apply>
     """
     print mathml(expr)
