@@ -81,12 +81,6 @@ class Order(Basic, ArithMeths, RelMeths):
 
     is_Order = True
 
-    # We are caching the order, e.g. O(1) < O(x) < O(x**2) etc in _cache.
-    # Unfortunately, this depends on limits and for very complicated
-    # expressions, it may easily cache a wrong result. Better is to do series
-    # expansion without limits completely.
-    _cache = {}
-
     __slots__ = []
 
     @cacheit
@@ -152,30 +146,9 @@ class Order(Basic, ArithMeths, RelMeths):
         if expr is S.Zero:
             return expr
 
-        # remove unused symbols
-        #symbols = tuple([s for s in symbols if expr.has(s)])
-        symbols = tuple(symbols)
-
-        # look Order symbols from cache, TODO: make cache a dictionary
-        cache = Order._cache.get(symbols,[])
-        for o in cache:
-            if o.expr==expr:
-                return o
-
         # create Order instance:
         obj = Basic.__new__(cls, expr, *symbols, **assumptions)
 
-        # cache univariate Order symbols:
-        if len(symbols)>1:
-            for s in symbols:
-                Order(expr, s)._get_cache_index(s)
-        elif symbols:
-            obj._get_cache_index(symbols[0])
-
-        # cache multivariate Order symbols:
-        cache.append(obj)
-        Order._cache[symbols] = cache
-
         return obj
 
     def oseries(self, order):
@@ -184,42 +157,6 @@ class Order(Basic, ArithMeths, RelMeths):
     def nseries(self, x, x0, n):
         return self
 
-    def _get_cache_index(obj, symbol):
-        if len(obj.symbols)>1:
-            obj = Order(obj.expr, symbol)
-        elif not obj.symbols:
-            obj = Order(obj.expr, symbol)
-        cache = Order._cache.get(symbol,[])
-        try: return cache.index(obj)
-        except ValueError: pass
-        i = -1
-        for o in cache:
-            i += 1
-            l = Order.find_limit(obj.expr/o.expr, symbol)
-            if l.is_unbounded:
-                cache.insert(i,obj)
-                break
-            if l.is_bounded:
-                continue
-            # If l.is_number is True, then l.is_bounded above should also be
-            # True, imho. It's probably a bug in assumptions.
-            if l.is_number:
-                continue
-            #try the new limit algorithm:
-            #from sympy import limit
-            #l = limit(obj.expr/o.expr, symbol, 0)
-            #if l.is_unbounded:
-            #    cache.insert(i,obj)
-            #    break
-            #if l.is_number:
-            #    continue
-            print obj.expr/o.expr,l
-            raise NotImplementedError("failed to determine the inclusion relation between %s and %s (got lim=%s)" % (o, obj, l))
-        else:
-            cache.append(obj)
-        Order._cache[symbol] = cache
-        return cache.index(obj)
-
     @classmethod
     def find_limit(cls, f, x):
         """Basically identical to:
@@ -278,7 +215,7 @@ class Order(Basic, ArithMeths, RelMeths):
         if len(self.symbols) <= 1:
             r = 'O(%s)' % self.expr.tostr()
         else:
-            r = 'O(%s)' % (', '.join([s.tostr() for s in self]))
+            r = 'O(%s)' % (', '.join([s.tostr() for s in self.args]))
         if self.precedence <= level:
             r = '(%s)' % (r)
         return r
@@ -320,13 +257,12 @@ class Order(Basic, ArithMeths, RelMeths):
                 return None
             r = None
             for s in common_symbols:
-                i1 = self._get_cache_index(s)
-                i2 = expr._get_cache_index(s)
+                l = Order.find_limit(self.expr/expr.expr, s) != 0
                 if r is None:
-                    r = (i1<=i2)
+                    r = l
                 else:
-                    if r != (i1<=i2):
-                        return None
+                    if r != l:
+                        return
             return r
         obj = Order(expr, *self.symbols)
         return self.contains(obj)
@@ -341,9 +277,6 @@ class Order(Basic, ArithMeths, RelMeths):
             return Order(self.expr.subs(old, new), *(self.symbols[:i]+self.symbols[i+1:]))
         return Order(self.expr.subs(old, new), *self.symbols)
 
-    def _calc_splitter(self, d):
-        return S.Zero
-
     def _sage_(self):
         #XXX: SAGE doesn't have Order yet. Let's return 0 instead.
         return Rational(0)._sage_()
