@@ -1,4 +1,4 @@
-from sympy import Symbol, Rational, Order, C, exp, ln, log, O
+from sympy import Symbol, Rational, Order, C, exp, ln, log, O, var
 from sympy.utilities.pytest import XFAIL
 from sympy.abc import w, x, y, z
 
@@ -93,13 +93,10 @@ def test_add_1():
     assert Order(exp(1/x)+x) == Order(exp(1/x))
     assert Order(exp(1/x)+1/x**20) == Order(exp(1/x))
 
-@XFAIL
 def test_ln_args():
-    #sometimes runs, sometimes fails, probably a caching bug:
-    assert Order(ln(2*x)).expr == ln(x) # ln(2*x) -> ln(2)+ln(x
-    assert Order(ln(y*x)).expr == ln(x)+ln(y) # ln(x*y) -> ln(x)+ln(y
-    assert Order(ln(x**3)).expr == ln(x) # ln(x**3) -> 3*ln(x
-    assert Order(ln(2*x)).expr == ln(x) # ln(2*x) -> ln(2)+ln(x
+    assert O(log(x)) + O(log(2*x)) == O(log(x))
+    assert O(log(x)) + O(log(x**3)) == O(log(x))
+    assert O(log(x*y)) + O(log(x)+log(y)) == O(log(x*y))
 
 def test_multivar_0():
     assert Order(x*y).expr == x*y
@@ -138,13 +135,6 @@ def test_multivar_3():
             (Order(y*x**2), Order(x*y**2))]
     assert (Order(x**2*y)+Order(y*x)) == Order(x*y)
 
-def test_w():
-    print
-    for k,v in Order._cache.items():
-        if isinstance(k, C.Symbol):
-            print k,v
-
-
 def test_issue369():
     x = Symbol('x')
     y = Symbol('y', negative=True)
@@ -162,3 +152,10 @@ def test_issue369():
     assert x.is_infinitesimal == None
     assert y.is_infinitesimal == None
     assert z.is_infinitesimal == None
+
+def test_order_str():
+    x, y = var("x y")
+    assert str(O(x)) == "O(x)"
+    assert str(O(x**2)) == "O(x**2)"
+
+    assert str(O(x*y)) == "O(x*y, x, y)"
