@@ -1,6 +1,7 @@
 from sympy import Symbol, symbols, together, hypersimp, factorial, binomial, \
         collect, Function, powsimp, separate, sin, exp, Rational, fraction, \
-        simplify, trigsimp, cos, tan, cot, log, ratsimp, Matrix, pi, integrate
+        simplify, trigsimp, cos, tan, cot, log, ratsimp, Matrix, pi, integrate, \
+        solve
 from sympy.utilities.pytest import XFAIL
 
 def test_ratsimp():
@@ -62,7 +63,7 @@ def test_factorial_simplify():
     assert simplify(factorial(factorial(x))) == factorial(factorial(x))
 
 def test_simplify():
-    x,y,k,n,m,w,f,s,A = symbols('xyknmwfsA')
+    x,y,z,k,n,m,w,f,s,A = symbols('xyzknmwfsA')
 
     e = 1/x + 1/y
     assert e != (x+y)/(x*y)
@@ -98,7 +99,18 @@ def test_simplify():
     A = Matrix([[2*k-m*w**2, -k],[-k,k-m*w**2]]).inv()
 
     assert simplify((A*Matrix([0,f]))[1]) == \
-        (-2*f*k + f*m*w**2)/(-k**2 + 3*k*m*w**2 - m**2*w**4)
+        (2*f*k - f*m*w**2)/(k**2 - 3*k*m*w**2 + m**2*w**4)
+
+    a,b,c,d,e,f,g,h,i = symbols('abcdefghi')
+
+    f_1 = x*a + y*b + z*c - 1
+    f_2 = x*d + y*e + z*f - 1
+    f_3 = x*g + y*h + z*i - 1
+
+    solutions = solve([f_1,f_2,f_3], x,y,z, simplified=False)
+
+    assert simplify(solutions[y]) == \
+        (a*i+c*d+f*g-a*f-c*g-d*i)/(a*e*i+b*f*g+c*d*h-a*f*h-b*d*i-c*e*g)
 
 def test_simplify_fail1():
     x = Symbol('x')
@@ -193,7 +205,7 @@ def test_hypersimp():
     assert hypersimp(factorial(k), k) == k + 1
     assert hypersimp(factorial(k**2), k) is None
 
-    assert hypersimp(2**k/factorial(k)**2, k) == 2/(k+1)**2
+    assert hypersimp(2**k/factorial(k)**2, k) == 2/(k**2+2*k+1)
 
     assert hypersimp(binomial(n, k), k) == (n-k)/(k+1)
     assert hypersimp(binomial(n+1, k), k) == (n-k+1)/(k+1)
@@ -202,10 +214,10 @@ def test_hypersimp():
     assert hypersimp(term, k) == (4*k + 5)/(6 + 16*k**2 + 28*k)
 
     term = 1/((2*k-1)*factorial(2*k+1))
-    assert hypersimp(term, k) == (2*k-1)/(2*k+1)/(2*k+2)/(2*k+3)
+    assert hypersimp(term, k) == (2*k-1)/(6 + 22*k + 24*k**2 + 8*k**3)
 
     term = binomial(n, k)*(-1)**k/factorial(k)
-    assert hypersimp(term, k) == (k - n)/(k+1)**2
+    assert hypersimp(term, k) == (k - n)/(k**2+2*k+1)
 
 def test_together2():
     x, y, z = symbols("xyz")
