@@ -11,3 +11,6 @@ else:
     all = all
 
 from lambdify import lambdify
+from source import source
+
+from decorator import threaded
